// disp_ooi_interface.h
// vl_display ooi interface.

#ifndef __DISP_OOI_INTERFACE__
#define __DISP_OOI_INTERFACE__

#ifndef __linux__
#pragma once
#endif

#include "vl_display.h"
#include <vector>
#include <string>

namespace ooi {

/// Defines the ooi_data that all oois have in common.
struct ooi_data
{
	std::vector<IPL_POINT> points_;
	std::string text_;
	IPL_COLOR line_color_;
	IPL_BOOL line_solid_;
	IPL_COLOR fill_color_;
	IPL_BOOL fill_transparent_;
	IPL_COLOR handle_line_color_;
	IPL_COLOR handle_fill_color_;
	IPL_COLOR text_color_;
	IPL_ULONG handle_size_;
	IPL_ULONG handle_style_;
	IPL_BOOL selected_;
	IPL_BOOL active_;
	IPL_BOOL ui_active_;
	IPL_BOOL visible_;
	IPL_BOOL lines_visible_;
	IPL_BOOL fill_visible_;
	IPL_BOOL text_visible_;
	IPL_BOOL handles_visible_;
	DISP_POI_DRAWING_STYLE poi_style_;
	IPL_ULONG width_;
	IPL_DOUBLE angle_;
	IPL_BOOL show_arrow_;
	IPL_BOOL drag_thickness_;
	IPL_BOOL constrain_;
	IPL_BOOL show_details_;
	IPL_ULONG dm_columns_;
	IPL_ULONG dm_rows_;

	/// constructor
	ooi_data()
	{ 
		line_color_ = (IPL_COLOR)PALETTERGB(255, 0, 0);
		line_solid_ = IPL_FALSE;
		fill_color_ = (IPL_COLOR)PALETTERGB(0, 0, 0);
		fill_transparent_ = IPL_FALSE;
#ifndef __linux__
		handle_line_color_ = (IPL_COLOR)::GetSysColor(COLOR_INACTIVECAPTION);
		handle_fill_color_ = (IPL_COLOR)::GetSysColor(COLOR_HIGHLIGHT);
#else
		handle_line_color_ = (IPL_COLOR)0;
		handle_fill_color_ = (IPL_COLOR)0;
#endif	// __linux__
		text_color_ = (IPL_COLOR)PALETTERGB(255, 255, 255);
		handle_size_ = 8;
		handle_style_ = 0x55555555;
		selected_ = IPL_FALSE;
		active_ = IPL_TRUE;
		ui_active_ = IPL_TRUE;
		visible_ = IPL_TRUE;
		lines_visible_ = IPL_TRUE;
		fill_visible_ = IPL_FALSE;
		text_visible_ = IPL_TRUE;
		handles_visible_ = IPL_TRUE;
		poi_style_ = DISP_POI_CROSS;
		width_ = 10;
		angle_ = 0.0;
		show_arrow_ = IPL_FALSE;
		drag_thickness_ = IPL_FALSE;
		constrain_ = IPL_FALSE;
		show_details_ = IPL_TRUE;
		dm_columns_ = 0;
		dm_rows_ = 0;
	}

	/// comparison
	bool operator==(const ooi_data& r) const
	{
		return
			0 == memcmp(&points_[0], &r.points_[0], sizeof(IPL_POINT)*points_.size()) &&
			text_ == r.text_ &&
			0 == memcmp(&line_color_, &r.line_color_, sizeof(IPL_COLOR)) &&
			0 == memcmp(&fill_color_, &r.fill_color_, sizeof(IPL_COLOR)) &&
			0 == memcmp(&handle_line_color_, &r.handle_line_color_, sizeof(IPL_COLOR)) &&
			0 == memcmp(&handle_fill_color_, &r.handle_fill_color_, sizeof(IPL_COLOR)) &&
			0 == memcmp(&text_color_, &r.text_color_, sizeof(IPL_COLOR)) &&
			line_solid_ == r.line_solid_ &&
			fill_transparent_ == r.fill_transparent_ &&
			handle_size_ == r.handle_size_ &&
			handle_style_ == r.handle_style_ &&
			selected_ == r.selected_ &&
			active_ == r.active_ &&
			ui_active_ == r.ui_active_ &&
			visible_ == r.visible_ &&
			lines_visible_ == r.lines_visible_ &&
			fill_visible_ == r.fill_visible_ &&
			text_visible_ == r.text_visible_ &&
			handles_visible_ == r.handles_visible_ &&
			poi_style_ == r.poi_style_ &&
			width_ == r.width_ &&
			angle_ == r.angle_ &&
			show_arrow_ == r.show_arrow_ &&
			drag_thickness_ == r.drag_thickness_ &&
			constrain_ == r.constrain_ &&
			show_details_ == r.show_details_ &&
			dm_columns_ == r.dm_columns_ &&
			dm_rows_ == r.dm_rows_;
	}
};


/// Object of Interest interface.
///
/// All actual ooi components must be derived from this interface.
/// For a new ooi type you derive a class from this interface and implement ooi
/// specifics in the respective functions.
class ooi_interface
{
protected:
	ooi_data data_;

public:
	/// Default constructor.
	ooi_interface() {};

	/// Destructor.
	virtual ~ooi_interface() {};

	/// Add an ooi at a specific location.
	/// \return IPL_TRUE if it worked, IPL_FALSE otherwise
	virtual IPL_BOOL Add(
		/// The location in IPL coordinates.
		const IPL_POINT* location,
		/// The count of points passed.
		IPL_ULONG count) =0;

	/// Get the ooi points.
	/// \return The list of points of the ooi in IPL coordinates.
	virtual IPL_POINT* Points()
	{
		return &data_.points_[0];
	}

	/// Get the number of points.
	/// \return The number of points of the ooi.
	virtual IPL_ULONG GetPointsCount() const
	{
		return (IPL_ULONG)data_.points_.size();
	}

	/// Get the number of handles.
	/// \return The number of handles of the ooi.
	virtual IPL_ULONG GetHandlesCount() const =0;

	/// Get the text drawing center of the ooi.
	/// \return The text center in IPL coordinates.
	virtual IPL_POINT GetTextCenter() const =0;

	/// Get the position of a specific handle.
	/// If the index is invalid, a point with the coordinates (-1, -1) will be
	/// returned.
	/// \return The position of the handle.
	virtual IPL_POINT GetHandlePos(
		/// The index of the handle
		IPL_ULONG index) const =0;

	/// The old and now deprecated get function that maps to DISP_InquireOoi
	/// DISP_OOI_POSINDEX. Retained for compatibility reasons.
	/// \return IPL_TRUE for a valid index, IPL_FALSE otherwise.
	virtual IPL_BOOL GetPos(
		/// The index. This is not a point or handle index, but a coordinate
		/// index, whose meaning is defined by the specific ooi dll.
		IPL_ULONG index,
		/// The returned coordinate.
		IPL_LONG* pos) const =0;

	/// The old and now deprecated set function that maps to DISP_ControlOoi
	/// DISP_OOI_POSINDEX. Retained for compatibility reasons.
	/// \return IPL_TRUE for a valid index, IPL_FALSE otherwise.
	virtual IPL_BOOL SetPos(
		/// The index. This is not a point or handle index, but a coordinate
		/// index, whose meaning is defined by the specific ooi dll.
		IPL_ULONG index, 
		/// The coordinate.
		IPL_LONG pos) =0;

	/// Get the cursor for a specific handle.
	/// If the index is invalid, the standard pointer cursor will be returned.
	/// \return A cursor handle.
	virtual const HCURSOR GetCursorHandle(
		/// The index of the handle.
		IPL_LONG index) const =0;

	/// Track a specific handle to a new location.
	virtual IPL_BOOL TrackHandleTo(
		/// The handle index.
		IPL_LONG index, 
		/// The location to track the handle to.
		const IPL_POINT* point) =0;

	/// Enumeration to specify possible return values of the HitTestHandle function,
	/// in addition to positive handle indices (starting from 0).
	enum { 
		/// The whole ooi was hit.
		hit_whole = -1, 
		/// The ooi was not hit.
		no_hit = -2, };

	/// Hit-tests a point with this ooi.
	/// \return The handle index if the point is over a handle, -1 if the point is
	/// over the rest of the ooi, -2 if the point is not on the ooi.
	virtual IPL_LONG HitTestHandle(
		/// The point to be checked in IPL coordinates.
		const IPL_POINT* point,
		/// The surrounding horizontal space (it's added to the left and right of the
		/// point, i.e. twice)
		IPL_ULONG space_x,
		/// The surrounding vertical space (it's added to the top and bottom of the
		/// point, i.e. twice)
		IPL_ULONG space_y) const =0;


	/// Gets a pointer to the visible flag.
	virtual IPL_BOOL* Visible()
	{
		return &data_.visible_;
	}


	/// Gets a pointer to the text visible flag.
	virtual IPL_BOOL* TextVisible()
	{
		return &data_.text_visible_;
	}

	/// Gets a pointer to the text color.
	virtual IPL_COLOR* TextColor()
	{
		return &data_.text_color_;
	}

	/// Gets the text.
	virtual const IPL_CHAR* GetText()
	{
		return data_.text_.c_str();
	}

	/// Sets the text.
	virtual IPL_VOID SetText(const IPL_CHAR* t)
	{
		data_.text_ = t;
	}


	/// Gets a pointer to the lines visible flag.
	virtual IPL_BOOL* LinesVisible()
	{
		return &data_.lines_visible_;
	}

	/// Gets a pointer to the line color.
	virtual IPL_COLOR* LineColor()
	{
		return &data_.line_color_;
	}

	/// Gets a pointer to the line solid flag.
	virtual IPL_BOOL* LineSolid()
	{
		return &data_.line_solid_;
	}


	/// Gets a pointer to the fill visible flag.
	virtual IPL_BOOL* FillVisible()
	{
		return &data_.fill_visible_;
	}

	/// Gets a pointer to the fill color.
	virtual IPL_COLOR* FillColor()
	{
		return &data_.fill_color_;
	}

	/// Gets a pointer to the fill transparent flag.
	virtual IPL_BOOL* FillTransparent()
	{
		return &data_.fill_transparent_;
	}


	/// Gets a pointer to the handles visible flag.
	virtual IPL_BOOL* HandlesVisible()
	{
		return &data_.handles_visible_;
	}

	/// Gets a pointer to the handle line color.
	virtual IPL_COLOR* HandleLineColor()
	{
		return &data_.handle_line_color_;
	}

	/// Gets a pointer to the handle fill color.
	virtual IPL_COLOR* HandleFillColor()
	{
		return &data_.handle_fill_color_;
	}

	/// Gets a pointer to the handle size.
	virtual IPL_ULONG* HandleSize()
	{
		return &data_.handle_size_;
	}

	/// Gets a pointer to the handle style.
	virtual IPL_ULONG* HandleStyle()
	{
		return &data_.handle_style_;
	}

	/// Gets a pointer to the selected flag.
	virtual IPL_BOOL* Selected()
	{
		return &data_.selected_;
	}


	/// Gets a pointer to the ui active flag.
	virtual IPL_BOOL* UiActive()
	{
		return &data_.ui_active_;
	}


	/// Gets a pointer to the active flag.
	virtual IPL_BOOL* Active()
	{
		return &data_.active_;
	}


	/// Gets a pointer to the poi styles flag.
	virtual DISP_POI_DRAWING_STYLE* POIStyle()
	{
		return &data_.poi_style_;
	}


	/// Get a pointer to the width.
	virtual IPL_ULONG* Width()
	{
		return &data_.width_;
	}


	/// Get a pointer to the angle.
	virtual IPL_DOUBLE* Angle()
	{
		return &data_.angle_;
	}


	/// Get a pointer to the show arrow flag.
	virtual IPL_BOOL* ShowArrow()
	{
		return &data_.show_arrow_;
	}

	/// Get a pointer to the drag thickness flag.
	virtual IPL_BOOL* DragThickness()
	{
		return &data_.drag_thickness_;
	}

	/// Get a pointer to the constrain flag.
	virtual IPL_BOOL* Constrain()
	{
		return &data_.constrain_;
	}

	/// Get a pointer to the show details flag.
	virtual IPL_BOOL* ShowDetails()
	{
		return &data_.show_details_;
	}

	/// Get a pointer to the datamatrix columns.
	virtual IPL_ULONG* DmColumns()
	{
		return &data_.dm_columns_;
	}

	/// Get a pointer to the datamatrix rows.
	virtual IPL_ULONG* DmRows()
	{
		return &data_.dm_rows_;
	}

	typedef IPL_VOID (__stdcall * draw_callback)(IPL_VOID* cookie, IPL_HDC Dc);
	typedef IPL_VOID (__stdcall * zoom_callback)(IPL_VOID* cookie, const IPL_POINT* in, POINT* out, int count);


	virtual IPL_BOOL DrawPolyline(IPL_VOID* cookie, draw_callback draw, IPL_HDC Dc, zoom_callback zoom)
	{
		UNUSED( cookie );
		UNUSED( draw );
		UNUSED( Dc );
		UNUSED( zoom );
		return IPL_FALSE;
	}

	virtual IPL_BOOL FillPolygon(IPL_VOID* cookie, draw_callback draw, IPL_HDC Dc, zoom_callback zoom)
	{
		UNUSED( cookie );
		UNUSED( draw );
		UNUSED( Dc );
		UNUSED( zoom );
		return IPL_FALSE;
	}

	virtual IPL_BOOL DrawText(IPL_VOID* cookie, draw_callback draw, IPL_HDC Dc, zoom_callback zoom)
	{
		UNUSED( cookie );
		UNUSED( draw );
		UNUSED( Dc );
		UNUSED( zoom );
		return IPL_FALSE;
	}

	virtual IPL_BOOL DrawHandles(IPL_VOID* cookie, draw_callback draw, IPL_HDC Dc, zoom_callback zoom)
	{
		UNUSED( cookie );
		UNUSED( draw );
		UNUSED( Dc );
		UNUSED( zoom );
		return IPL_FALSE;
	}
};


}	// namespace ooi

#endif // __DISP_OOI_INTERFACE__
