#ifndef MVIMPACT_CALIBRATION_H_
#define MVIMPACT_CALIBRATION_H_

/// \file
///

#ifndef WRAP_PYTHON
#include "mvIMPACT.h"
#endif // #ifndef WRAP_PYTHON

#ifndef  DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_camcal.h>


#endif // #ifndef WRAP_PYTHON
#endif // #ifndef  DOXYGEN_SHOULD_SKIP_THIS

namespace mvIMPACT {

        /// \namespace mvIMPACT::calibration This namespace contains all classes and functions from mvIMPACT Calibration.
	namespace calibration {


/// \brief Coordinate system orientation.
enum OrientationMode
{
	/// \brief Normal orientation.
	///
	/// This means that the \c Y coordinate is the same as
	/// the orientation of the \c Y coordinate in the image.
	omNormal = mvIMPACT_C::CC_ORIENTATION_NORMAL,
	/// \brief Inverse orientation.
	///
	/// This means that the \c Y coordinate is the inverse as
	/// the orientation of the \c Y coordinate in the image, i.e.
	/// the y axis is mirrored with respect to the x axis.
	omInverse = mvIMPACT_C::CC_ORIENTATION_INVERSE
};

/// calibration types.
enum CalibrationType
{
	/// Auto calibration. 
	ctAuto = mvIMPACT_C::CC_CALIB_AUTO,
	/// Scaling calibration. 
	ctScaling = mvIMPACT_C::CC_SCALING,
	/// Perspective calibration.
	ctPerspective = mvIMPACT_C::CC_PERSPECTIVE,
	/// Distortion calibration.
	ctDistortion = mvIMPACT_C::CC_DISTOR	
};

/// \brief Units used in the world coordinate system
enum WorldUnit
{
	/// Measurement is done in millimeters.
	wuMm = mvIMPACT_C::CC_WU_MM,
	/// Measurement is done in centimeters.
	wuCm = mvIMPACT_C::CC_WU_CM,
	/// Measurement is done in meters.
	wuM = mvIMPACT_C::CC_WU_M
};

/// \brief Coordinate system.
///
/// This class represents a coordinate system with origin, inclination angle and
/// orientation of the y axis.
class CoordinateSystem
{
public:
	/// \brief Construct a CoordinateSystem object
	CoordinateSystem(
		///the origin of the coordinate system
		const PointDouble& origin = PointDouble(0,0), 
		/// the inclination angle in degrees
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="360.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
   		/// \endxmlonly    			  						
		double angle=0.0, 
		/// orientation
		OrientationMode orientation=omNormal);

	/// \brief Retrieve the origin of the coordinate system
	PointDouble getOrigin() const;

	/// \brief Set the origin of the coordinate system
	CoordinateSystem& setOrigin(
		///new origin
		const PointDouble& origin);
	
	/// \brief Retrieve the inclination angle of the coordinate system
	///
	/// The inclination angle is the angle between the y axis of the 
	/// standard coordinate system and the y axis of the coordinate system described
	/// by this CoordinateSystem object.
	double getAngle() const;
	
	/// \brief Set the inclination angle of the coordinate system
	///
	/// The inclination angle is the angle between the y axis of the 
	/// standard coordinate system and the y axis of the coordinate system described
	/// by this CoordinateSystem object.
	CoordinateSystem& setAngle(
		///new angle in degrees
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="360.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
   		/// \endxmlonly    			  						
		double angle);

	/// \brief Retrieve the orientation.
	///
	/// The orientation defines, wether the y axis is inverse, i.e. mirrored on the x axis or not.
	OrientationMode getOrientation() const;
	
	
	/// \brief Set the orientation.
	///
	/// The orientation defines, wether the y axis is inverse, i.e. mirrored on the x axis or not.
	CoordinateSystem& setOrientation(
		///new orientation
		OrientationMode orientation);


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	operator const mvIMPACT_C::CC_COORDSYST() const;

	CoordinateSystem(const mvIMPACT_C::CC_COORDSYST &p);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

private:
	PointDouble m_origin;
	double m_angle;
	OrientationMode m_orientation;
};

/// \brief wrapper class for calibration statistics
class CalibrationStatistics
{
public:

CalibrationStatistics();

/// \brief construct a CalibrationStatistics object
CalibrationStatistics(
	double meanXDifference, double meanYDifference,	double meanEuclideanDistance,
	double stddevXDifference, double stddevYDifference, double stddevEuclideanDistance
	);

/// \brief Retrieve the mean x difference
double getMeanXDifference() const;

/// \brief Retrieve the mean y difference
double getMeanYDifference() const;

/// \brief Retrieve the mean euclidean distance
double getMeanEuclideanDistance() const;

/// \brief Retrieve the standard deviation of the x differences
double getStddevXDifference() const;

/// \brief Retrieve the standard deviation of the y differences
double getStddevYDifference() const;

/// \brief Retrieve the standard deviation of the euclidean distances
double getStddevEuclideanDistance() const;

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	CalibrationStatistics(const mvIMPACT_C::CC_STATISTICS &p);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

private:
		double m_meanXDifference;
		double m_meanYDifference;
		double m_meanEuclideanDistance;
		double m_stddevXDifference;
		double m_stddevYDifference;
		double m_stddevEuclideanDistance;
};


/// \brief This class is used to teach and apply image calibration
class CameraCalibration 
{
private:
	mvIMPACT_C::CC_BUFHANDLE m_handle;
	int* m_counter;
	
public:

	/// \brief Construct a CameraCalibration object with default parameters.
	CameraCalibration();
	
	/// \brief Construct a CameraCalibration object from an XML file
	///
	/// The XML file can be created with mvIMPACT::Calibration::CameraCalibration::writeToXMLFile
	CameraCalibration(
		/// XML file name to read from.
		const std::string& xmlFileName);
	
	/// \brief Copy-construct a CameraCalibration object.
	CameraCalibration(
		/// The source object.
		const CameraCalibration& src);
	
	/// \brief Destruct a CameraCalibration object.
	~CameraCalibration();
	
#ifndef WRAP_ANY
	/// \brief Overwrite object with \a src.
	CameraCalibration& operator=(const CameraCalibration& src);
#endif // #ifndef WRAP_ANY

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	/// \brief return handle
	mvIMPACT_C::CC_BUFHANDLE getHandle() const;
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

// properties:

/// \brief Retrieve \c X distance of the grid dots.
///
/// This value is measured in world units. 
///
/// \see mvIMPACT::calibration::CameraCalibration::getWorldUnit
/// \see mvIMPACT::calibration::CameraCalibration::setWorldUnit
double getGridPointDistanceX() const;

/// \brief Set x distance of the grid dots.
///
/// This value is measured in world units. 
///
/// \see mvIMPACT::calibration::CameraCalibration::getWorldUnit
/// \see mvIMPACT::calibration::CameraCalibration::setWorldUnit
CameraCalibration& setGridPointDistanceX(
	/// New x distance for grid dots.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	/// <mvParamStep>0.01</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	double gridPointDistanceX);


/// \brief Retrieve y distance of the grid dots.
///
/// This value is measured in world units. 
///
/// \see mvIMPACT::calibration::CameraCalibration::getWorldUnit
/// \see mvIMPACT::calibration::CameraCalibration::setWorldUnit
double getGridPointDistanceY() const;

/// \brief Set y distance of the grid dots.
///
/// This value is measured in world units. 
///
/// \see mvIMPACT::calibration::CameraCalibration::getWorldUnit
/// \see mvIMPACT::calibration::CameraCalibration::setWorldUnit
CameraCalibration& setGridPointDistanceY(
	/// New y distance for grid dots.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	/// <mvParamStep>0.01</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	double gridPointDistanceY);

/// \brief Retrieve tolerance of the algorithm.
///
/// This value is needed for the calibration and gives the
/// desired accuracy of the results. By default it is set to \c 1e-15.
double getTolerance() const;

/// \brief Set tolerance of the algorithm.
///
/// This value is needed for the calibration and gives the
/// desired accuracy of the results. By default it is set to \c 1e-15.
CameraCalibration& setTolerance(
	/// New value for tolerance.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	double tolerance);


/// \brief Retrieve coordinate system.
///
/// Retrieves the used coordinate system.
/// \see mvIMPACT::calibration::CoordinateSystem
CoordinateSystem getCoordinateSystem() const;

/// \brief Set coordinate system.
///
/// Retrieves the used coordinate system.
/// \see mvIMPACT::calibration::CoordinateSystem
CameraCalibration& setCoordinateSystem(
	/// New coordinate system.
	const CoordinateSystem& coordinateSystem);

/// \brief Retrieve minimum grid dot area.
///
/// All found objects which are smaller than this value
/// will not be considered as a grid dot.
int getMinGridPointArea() const;

/// \brief Set minimum grid dot area.
///
/// All found objects which are smaller than this value
/// will not be considered as a grid dot.
CameraCalibration& setMinGridPointArea(
	/// New minimum area of grid dots.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	int minGridPointArea);


/// \brief Retrieve maximum grid dot area.
///
/// All found objects which are larger than this value
/// will not be considered as a grid dot.
int getMaxGridPointArea() const;

/// \brief Set maximum grid dot area.
///
/// All found objects which are larger than this value
/// will not be considered as a grid dot.
CameraCalibration& setMaxGridPointArea(
	/// New maximum size of grid dots.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	int maxGridPointArea);

/// \brief Retrieve maximum number of grid dots
///
/// If more than this number of points are recognized on the grid 
/// the calibration will be stopped and an exception will be thrown.
int getMaxGridPointCount() const;

/// \brief Set maximum number of grid dots
///
/// If more than this number of points are recognized on the grid 
/// the calibration will be stopped and an exception will be thrown.
CameraCalibration& setMaxGridPointCount(
	/// New maximum count of grid dots.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	int maxGridPointCount);

/// \brief Retrieve maximum number of iterations
///
/// If the calibration algorithm reaches this number of iterations and has
/// the calibration data is not valid yet, calibration will stopped an exception 
/// will be thrown.
int getMaxIterations() const;

/// \brief Set maximum number of iterations
///
/// If the calibration algorithm reaches this number of iterations and has
/// the calibration data is not valid yet, calibration will stopped an exception 
/// will be thrown.
CameraCalibration& setMaxIterations(
	/// New maximum number of iterations.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	int maxIterations);

/// \brief Retrieve number of iterations done during calibration.
int getIterationCount() const;

/// \brief Restrieve the threshold used for binarization. 
///
/// This threshold is used when the calibration grid image is binarized.
int getThreshold() const;

/// \brief Set the threshold used for binarization. 
///
/// This threshold is used when the calibration grid image is binarized.
CameraCalibration& setThreshold(
	/// New threshold for binarization.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	int threshold);

/// \brief Retrieve polarity of the calibration grid image.
///
/// If the calibration grid image has bright dots on dark background,
/// gridPolarity should be set to pmWhiteOnBlack, if it has dark dots
/// on bright background, it should be set to pmBlackOnWhite. If
/// it is unknown which type the grid is of, it should be set to pmAuto.
///
/// \see #PolarityMode
PolarityMode getGridPolarity() const;

/// \brief Set polarity of the calibration grid image.
///
/// If the calibration grid image has bright dots on dark background,
/// gridPolarity should be set to pmWhiteOnBlack, if it has dark dots
/// on bright background, it should be set to pmBlackOnWhite. If
/// it is unknown which type the grid is of, it should be set to pmAuto.
///
/// \see #PolarityMode
CameraCalibration& setGridPolarity(
	/// New polarity mode.
	PolarityMode gridPolarity);

/// \brief Retrieve type of calibration.
///
/// \see #CalibrationType
CalibrationType getCalibrationType() const;

/// \brief Set type of calibration.
///
/// \see #CalibrationType
CameraCalibration& setCalibrationType(
	/// New calibration type.
	CalibrationType calibrationType);

/// \brief Retrieve background value
///
/// For areas of resulting image where no calibration data is available the
/// transformation functions fill the destination image with this value.
int getBackgroundValue() const;


/// \brief Set background value
///
/// For areas of resulting image where no calibration data is available the
/// transformation functions fill the destination image with this value.
CameraCalibration& setBackgroundValue(
	/// New background value.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	int backgroundValue);

/// \brief Retrieve unit used for the world coordinate system.
WorldUnit getWorldUnit() const;


/// \brief Set unit used for the world coordinate system.
CameraCalibration& setWorldUnit(
	/// New unit for the world coordinate system.
	WorldUnit worldUnit);

/// \brief Retrieve statistics.
///
/// Retrieve statistics gathered when running the calibration algorithm.
CalibrationStatistics getStatistics() const;

/// \brief Retrieve region of image used for calibration.
///
/// This allows to define a rectangular region of the input image
/// which is exclusively used to search for the grid points.
Rectangle getRegionOfInterest() const;

/// \brief Set region of image used for calibration.
///
/// This allows to define a rectangular region of the input image
/// which is exclusively used to search for the grid points.
CameraCalibration& setRegionOfInterest(
	/// New rectangular region of interest.
	const Rectangle& regionOfInterest);

/// \brief Get interpolation type
///
/// This defines the type of interpolation used when transforming
/// a non-calibrated image into a calibrated one.
GeometricInterpolation getInterpolationType() const;

/// \brief Set interpolation type
///
/// This defines the type of interpolation used when transforming
/// a non-calibrated image into a calibrated one.
CameraCalibration& setInterpolationType(
	/// New interpolation type.
	GeometricInterpolation interpolationType);

/// \brief Get flag indicating if calibration data is available.
///
/// This flag is \c true, if calibration data is available, i.e. one 
/// of the teach member functions has been called already.
///
/// \see mvIMPACT::calibration::CameraCalibration::teachByGrid
/// \see mvIMPACT::calibration::CameraCalibration::teachByPoints
bool getHasCalibrationData() const;

/// \brief Get x direction scaling factor.
///
/// When the calibration type is set to \c ctScaling, this factor
/// is used for the scaling in x direction.
///
/// \see #CalibrationType
/// \see mvIMPACT::calibration::CameraCalibration::setCalibrationType
/// \see mvIMPACT::calibration::CameraCalibration::getCalibrationType
double getScalingFactorX() const;

/// \brief Set x direction scaling factor.
///
/// When the calibration type is set to \c ctScaling, this factor
/// is used for the scaling in x direction.
///
/// \see #CalibrationType
/// \see mvIMPACT::calibration::CameraCalibration::setCalibrationType
/// \see mvIMPACT::calibration::CameraCalibration::getCalibrationType
CameraCalibration& setScalingFactorX(
	/// New scaling facotr in x direction.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.01"/></mvParamMin>
	/// <mvParamStep>0.01</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	double scalingFactorX);


/// \brief Get y direction scaling factor.
///
/// When the calibration type is set to \c ctScaling, this factor
/// is used for the scaling in y direction.
///
/// \see #CalibrationType
/// \see mvIMPACT::calibration::CameraCalibration::setCalibrationType
/// \see mvIMPACT::calibration::CameraCalibration::getCalibrationType
double getScalingFactorY() const;

/// \brief Set y direction scaling factor.
///
/// When the calibration type is set to \c ctScaling, this factor
/// is used for the scaling in y direction.
///
/// \see #CalibrationType
/// \see mvIMPACT::calibration::CameraCalibration::setCalibrationType
/// \see mvIMPACT::calibration::CameraCalibration::getCalibrationType
CameraCalibration& setScalingFactorY(
	/// New scaling facotr in y direction.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.01"/></mvParamMin>
	/// <mvParamStep>0.01</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly    			  						
	double scalingFactorY);

/// \brief Construct a CameraCalibration object from an XML file
///
/// Afterwards the XML file can be used to created a mvIMPACT::Calibration::CameraCalibration object
/// with the constructor mvIMPACT::Calibration::CameraCalibration(const std::string&).
void writeToXMLFile(
	/// File name of output XML file.
	const std::string& fileName) const;


/// \brief Get the grid coordinates from an image
///
/// This function analyses the input image and returns the coordinates of the
/// found grid points. The returned coordinates are not sorted and not calibrated.
/// 
/// This allows the user to get the use his own algorithm to match grid points from
/// the image to the corresponding points of the grid and call 
/// mvIMPACT::calibration::teachByPoints afterwards.
///
std::vector<PointDouble> getGridPointsFromImage(
				///Image containing a calibration grid.
				const Image& source) const;


/// \brief Teach the calibration data from a grid image.
///
/// This function analyses the input image and teaches the calibration data
/// from it. 
///
/// This function or teachByPoints() needs to be called to teach the
/// calibration data.
///
void teachByGrid(
				///Image containing a calibration grid.
				const Image& source);


/// \brief Teach the calibration data from point coordinates measured in pixels.
///
/// In conjunction with mvIMPACT::calibration::getGridPointsFromImage this function 
/// allows the user to use his own algorithm to match grid points from
/// the image to the corresponding points of the grid.
///
/// The only difference to teachBy3DPoints() is that here the points in \a objectPoints
/// are measured in pixels.
///
/// This function or teachByGrid() needs to be called to teach the
/// calibration data.
///
/// \see mvIMPACT::calibration::CameraCalibration::teachBy3DPoints
void teachByPoints(
				/// Vector containing the image coordinates of the calibration points
				const std::vector<PointDouble> & PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR, imagePoints),
				/// Vector containing the grid coordinates of the calibration points
				/// each point in \a imagePoints corresponds to the point in \a objectPoints
				/// with the same index.
				const std::vector<PointDouble> & PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR, objectPoints)
				);


/// \brief Teach the calibration data from point coordinates measured in world units.
///
/// In conjunction with mvIMPACT::calibration::getGridPointsFromImage this function 
/// allows the user to use his own algorithm to match grid points from
/// the image to the corresponding points of the grid.
///
/// The only difference to teachByPoints() is that here the points in \a objectPoints
/// are measured in world units.
///
/// This function or teachByGrid() needs to be called to teach the
/// calibration data.
///
/// \see mvIMPACT::calibration::CameraCalibration::teachByPoints
/// \see mvIMPACT::calibration::CameraCalibration::setWorldUnit
/// \see mvIMPACT::calibration::CameraCalibration::getWorldUnit
void teachBy3DPoints(
				/// Vector containing the image coordinates of the calibration points
				const std::vector<PointDouble> & PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR, imagePoints),
				/// Vector containing the grid coordinates of the calibration points
				/// each point in \a imagePoints corresponds to the point in \a objectPoints
				/// with the same index. The z coordinate of the points is considered to be
				/// 0.
				const std::vector<Point3DDouble> & PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR, objectPoints)
				);
				


/// \brief Transform uncalibrated points into calibrated ones.
///
/// This function transforms uncalibrated points, i.e. points "living" in the coordinate system of the uncalibrated
/// input image into calibrated ones, i.e. coordinates "living" in the coordinate system defined by the calibration grid.
/// Both coordinates are measured in pixels.
std::vector<PointDouble> transformPoints(
				/// Vector containing the image coordinates of the uncalibrated source points
				const std::vector<PointDouble>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR, sourcePoints));

/// \brief Transform back calibrated points into uncalibrated ones.
///
/// This function transforms back calibrated points, i.e. coordinates "living" in the coordinate system defined by the calibration grid.
/// into uncalibrated ones, i.e. points "living" in the coordinate system of the uncalibrated input image.
/// Both coordinates are measured in pixels.
std::vector<PointDouble> inverseTransformPoints(
				/// Vector containing the image coordinates of the calibrated source points
				const std::vector<PointDouble>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR, sourcePoints));

/// \brief Transform an uncalibrated image into a calibrated one.
///
/// This function transforms the uncalibrated image by using the
/// calibration data into a calibrated image. The result image will be generated
/// by using the interpolation type specified with property interpolationType.
///
/// \see mvIMPACT::calibration::CameraCalibration::getInterpolationType
/// \see mvIMPACT::calibration::CameraCalibration::setInterpolationType
Image transformImage(
				/// Source image
				const Image& source);

/// \brief Transform an uncalibrated image into a calibrated one.
///
/// This function transforms the uncalibrated image by using the
/// calibration data into a calibrated image. The result image will be generated
/// by using the interpolation type specified with property interpolationType.
///
/// \see mvIMPACT::calibration::CameraCalibration::getInterpolationType
/// \see mvIMPACT::calibration::CameraCalibration::setInterpolationType
void transformImage(
				/// Destination image
				Image& dest, 
				/// Source image
				const Image& source);


/// \brief Transforms uncalibrated 2D image coordinates into calibrated 3D world coordinates.
///
/// The world coordinates "live" in the coordinate system defined by the calibration grid,
/// and the properties GridPointDistanceX, GridPointDistanceY, coordinateSystem and worldUnit. They are measured
/// in world units.
///
/// The image coordinates "live" in the coordinate system of the uncalibrated input image. They are measured
/// in pixels.
///
/// \see mvIMPACT::calibration::CameraCalibration::getGridPointDistanceX
/// \see mvIMPACT::calibration::CameraCalibration::setGridPointDistanceX
/// \see mvIMPACT::calibration::CameraCalibration::getGridPointDistanceY
/// \see mvIMPACT::calibration::CameraCalibration::setGridPointDistanceX
/// \see mvIMPACT::calibration::CameraCalibration::getCoordinateSystem
/// \see mvIMPACT::calibration::CameraCalibration::setCoordinateSystem
/// \see mvIMPACT::calibration::CameraCalibration::getWorldUnit
/// \see mvIMPACT::calibration::CameraCalibration::setWorldUnit
std::vector<Point3DDouble> transformPointsToWorld(
	/// Source points for the transformation. Each point in the result corresponds to the
	/// source point with the same index in \c srcPoints.
	const std::vector<PointDouble>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR, sourcePoints));

/// \brief Transforms back calibrated 3D world coordinates into uncalibrated 2D image coordinates.
///
/// The world coordinates "live" in the coordinate system defined by the calibration grid,
/// and the properties GridPointDistanceX, GridPointDistanceY, coordinateSystem and worldUnit. They are measured
/// in world units.
///
/// The image coordinates "live" in the coordinate system of the uncalibrated input image. They are measured
/// in pixels.
///
/// \see mvIMPACT::calibration::CameraCalibration::getGridPointDistanceX
/// \see mvIMPACT::calibration::CameraCalibration::setGridPointDistanceX
/// \see mvIMPACT::calibration::CameraCalibration::getGridPointDistanceY
/// \see mvIMPACT::calibration::CameraCalibration::setGridPointDistanceX
/// \see mvIMPACT::calibration::CameraCalibration::getCoordinateSystem
/// \see mvIMPACT::calibration::CameraCalibration::setCoordinateSystem
/// \see mvIMPACT::calibration::CameraCalibration::getWorldUnit
/// \see mvIMPACT::calibration::CameraCalibration::setWorldUnit
std::vector<PointDouble> transformPointsFromWorld(
	/// Source points for the transformation. Each point in the result corresponds to the
	/// source point with the same index in \c srcPoints.
	const std::vector<Point3DDouble>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR, sourcePoints));

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
static CameraCalibration cameraCalibrationFactory(mvIMPACT_C::CC_BUFHANDLE handle);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

private:
	/// construct a CameraCalibration object from a handle
	explicit CameraCalibration(mvIMPACT_C::CC_BUFHANDLE handle);

	void copy(const CameraCalibration & src);
	
	void dispose();
};

/// \brief Return version number from vl_camcal module.

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionCalibration();


	} // namespace calibration
} // namespace mvIMPACT

#include "mvIMPACT_calibration_impl.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::calibration;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS



#endif // #ifndef MVIMPACT_CALIBRATION_H_
