#ifndef MVIMPACT_CALIBRATION_IMPL_H_
#define MVIMPACT_CALIBRATION_IMPL_H_

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	namespace calibration {


inline CoordinateSystem::CoordinateSystem(
	const PointDouble& origin, 
	double angle, 
	OrientationMode orientation)
		: 	m_origin(origin), m_angle(angle), m_orientation(orientation)
{
}

inline PointDouble CoordinateSystem::getOrigin() const
{
	return m_origin;
}

inline CoordinateSystem& CoordinateSystem::setOrigin(const PointDouble& origin)
{
	m_origin = origin;
	return *this;
}

inline double CoordinateSystem::getAngle() const
{
	return m_angle;
}

inline CoordinateSystem& CoordinateSystem::setAngle(double angle)
{
	m_angle = angle;
	return *this;
}

inline OrientationMode CoordinateSystem::getOrientation() const
{
	return m_orientation;
}

inline CoordinateSystem& CoordinateSystem::setOrientation(OrientationMode orientation)
{
	m_orientation = orientation;
	return *this;
}

inline CoordinateSystem::operator const mvIMPACT_C::CC_COORDSYST() const
{
	mvIMPACT_C::CC_COORDSYST result;
	
	result.orig = m_origin;
	result.angle = m_angle;
	result.orientation = static_cast<mvIMPACT_C::CC_ORIENTATION>(m_orientation);
	
	return result;
}

inline CoordinateSystem::CoordinateSystem(const mvIMPACT_C::CC_COORDSYST &p)
		:	m_origin(p.orig), m_angle(p.angle), 
			m_orientation(OrientationMode(p.orientation))
{
}

inline CalibrationStatistics::CalibrationStatistics()
	: 	m_meanXDifference(0.0), m_meanYDifference(0.0),	
		m_meanEuclideanDistance(0.0), 
		m_stddevXDifference(0.0), m_stddevYDifference(0.0),
		m_stddevEuclideanDistance(0.0)
{
}

inline CalibrationStatistics::CalibrationStatistics(
	double meanXDifference, double meanYDifference,	double meanEuclideanDistance,
	double stddevXDifference, double stddevYDifference, double stddevEuclideanDistance
	)
	: 	m_meanXDifference(meanXDifference), m_meanYDifference(meanYDifference),	
		m_meanEuclideanDistance(meanEuclideanDistance), 
		m_stddevXDifference(stddevXDifference), m_stddevYDifference(stddevYDifference),
		m_stddevEuclideanDistance(stddevEuclideanDistance)
{
}

inline double CalibrationStatistics::getMeanXDifference() const
{
	return m_meanXDifference;
}

inline double CalibrationStatistics::getMeanYDifference() const
{
	return m_meanYDifference;
}

inline double CalibrationStatistics::getMeanEuclideanDistance() const
{
	return m_meanEuclideanDistance;
}

inline double CalibrationStatistics::getStddevXDifference() const
{
	return m_stddevXDifference;
}

inline double CalibrationStatistics::getStddevYDifference() const
{
	return m_stddevYDifference;
}

inline double CalibrationStatistics::getStddevEuclideanDistance() const
{
	return m_stddevEuclideanDistance;
}

inline CalibrationStatistics::CalibrationStatistics(const mvIMPACT_C::CC_STATISTICS &p)
		:	m_meanXDifference(p.dxavg), m_meanYDifference(p.dyavg), m_meanEuclideanDistance(p.euclavg),
			m_stddevXDifference(p.xstddev), m_stddevYDifference(p.ystddev), m_stddevEuclideanDistance(p.euclstddev)
{	
}


inline CameraCalibration::CameraCalibration()
{
	m_handle = IPL_DONT_CARE;
	
	if(!mvIMPACT_C::CC_BufAlloc(&m_handle, 0)) 
	{ throwImpactError(); }
	
	m_counter = new int(1);
}
	
inline CameraCalibration::CameraCalibration(mvIMPACT_C::CC_BUFHANDLE handle)
{
	m_handle = handle;
	m_counter = new int(1);
}

inline CameraCalibration::CameraCalibration(const std::string& xmlFileName)
{
	m_handle = IPL_DONT_CARE;
	
	if(!mvIMPACT_C::CC_ReadFromFile(&m_handle, xmlFileName.c_str()))
	{ throwImpactError(); }
	
	m_counter = new int(1);
}
	
inline CameraCalibration::CameraCalibration(const CameraCalibration& src)
{
	copy(src);
}
	
inline CameraCalibration::~CameraCalibration()
{
	try {
		dispose();
	} catch(...) {
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}
	
inline CameraCalibration& CameraCalibration::operator=(const CameraCalibration& src)
{
	if(this != &src) {
		dispose();
		copy(src);
	}
	return *this;
}

inline mvIMPACT_C::CC_BUFHANDLE CameraCalibration::getHandle() const
{
	return m_handle;
}


inline double CameraCalibration::getGridPointDistanceX() const
{
	mvIMPACT_C::IPL_DOUBLE result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_DX, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setGridPointDistanceX(double dX)
{
	mvIMPACT_C::IPL_DOUBLE val = dX;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_DX, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline double CameraCalibration::getGridPointDistanceY() const
{
	mvIMPACT_C::IPL_DOUBLE result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_DY, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setGridPointDistanceY(double dY)
{
	mvIMPACT_C::IPL_DOUBLE val = dY;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_DY, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline double CameraCalibration::getTolerance() const
{
	mvIMPACT_C::IPL_DOUBLE result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_TOLERANCE, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setTolerance(double tolerance)
{
	mvIMPACT_C::IPL_DOUBLE val = tolerance;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_TOLERANCE, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline CoordinateSystem CameraCalibration::getCoordinateSystem() const
{
	mvIMPACT_C::CC_COORDSYST result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_COORDINATE_SYSTEM, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setCoordinateSystem(const CoordinateSystem& system)
{
	mvIMPACT_C::CC_COORDSYST val = system;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_COORDINATE_SYSTEM, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline int CameraCalibration::getMinGridPointArea() const
{
	mvIMPACT_C::IPL_LONG result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_MIN_GRID_AREA, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setMinGridPointArea(int size)
{
	mvIMPACT_C::IPL_LONG val = size;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_MIN_GRID_AREA, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline int CameraCalibration::getMaxGridPointArea() const
{
	mvIMPACT_C::IPL_LONG result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_MAX_GRID_AREA, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setMaxGridPointArea(int size)
{
	mvIMPACT_C::IPL_LONG val = size;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_MAX_GRID_AREA, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline int CameraCalibration::getMaxGridPointCount() const
{
	mvIMPACT_C::IPL_LONG result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_MAX_GRIDPT_CNT, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setMaxGridPointCount(int size)
{
	mvIMPACT_C::IPL_LONG val = size;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_MAX_GRIDPT_CNT, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline int CameraCalibration::getMaxIterations() const
{
	mvIMPACT_C::IPL_LONG result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_MAX_ITER_NR, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setMaxIterations(int iterations)
{
	mvIMPACT_C::IPL_LONG val = iterations;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_MAX_ITER_NR, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline int CameraCalibration::getIterationCount() const
{
	mvIMPACT_C::IPL_LONG result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RO_ITER_CNT, &result))
	{ throwImpactError(); }
	
	return result;
}

inline int CameraCalibration::getThreshold() const
{
	mvIMPACT_C::IPL_LONG result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_THRESHOLD, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setThreshold(int threshold)
{
	mvIMPACT_C::IPL_LONG val = threshold;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_THRESHOLD, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline PolarityMode CameraCalibration::getGridPolarity() const
{
	mvIMPACT_C::IPL_POLARITY result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_POLARITY, &result))
	{ throwImpactError(); }
	
	return PolarityMode(result);
}

inline CameraCalibration& CameraCalibration::setGridPolarity(PolarityMode mode)
{
	mvIMPACT_C::IPL_POLARITY val(static_cast<mvIMPACT_C::IPL_POLARITY>(mode));
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_POLARITY, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline CalibrationType CameraCalibration::getCalibrationType() const
{
	mvIMPACT_C::CC_CALIB_TYPE result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_CALIBRATION_TYPE, &result))
	{ throwImpactError(); }
	
	return CalibrationType(result);
}

inline CameraCalibration& CameraCalibration::setCalibrationType(CalibrationType type)
{
	mvIMPACT_C::CC_CALIB_TYPE val(static_cast<mvIMPACT_C::CC_CALIB_TYPE>(type));
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_CALIBRATION_TYPE, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline int CameraCalibration::getBackgroundValue() const
{
	mvIMPACT_C::IPL_LONG result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_BACKGROUND_VALUE, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setBackgroundValue(int value)
{
	mvIMPACT_C::IPL_LONG val = value;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_BACKGROUND_VALUE, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline WorldUnit CameraCalibration::getWorldUnit() const
{
	mvIMPACT_C::CC_WORLD_UNIT result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_WORLD_UNIT, &result))
	{ throwImpactError(); }
	
	return WorldUnit(result);
}

inline CameraCalibration& CameraCalibration::setWorldUnit(WorldUnit unit)
{
	mvIMPACT_C::CC_WORLD_UNIT val(static_cast<mvIMPACT_C::CC_WORLD_UNIT>(unit));
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_WORLD_UNIT, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline CalibrationStatistics CameraCalibration::getStatistics() const
{
	mvIMPACT_C::CC_STATISTICS result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RO_STATISTICS, &result))
	{ throwImpactError(); }
	
	return result;
}

inline Rectangle CameraCalibration::getRegionOfInterest() const
{
	mvIMPACT_C::IPL_RECT result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_ROI, &result))
	{ throwImpactError(); }
	
	return result;
}

inline CameraCalibration& CameraCalibration::setRegionOfInterest(const Rectangle& roi)
{
	mvIMPACT_C::IPL_RECT val = roi;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_ROI, &val))
	{ throwImpactError(); }
	
	return *this;
}

inline GeometricInterpolation CameraCalibration::getInterpolationType() const
{
	mvIMPACT_C::IPL_GEOTRANS result;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_INTERPOLATION_TYPE, &result))
	{ throwImpactError(); }
	
	return GeometricInterpolation(result);
}

inline CameraCalibration& CameraCalibration::setInterpolationType(GeometricInterpolation interpolationType)
{
	mvIMPACT_C::IPL_GEOTRANS val(static_cast<mvIMPACT_C::IPL_GEOTRANS>(interpolationType));
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_INTERPOLATION_TYPE, &val))
	{ throwImpactError(); }
	
	return *this;
}


inline bool CameraCalibration::getHasCalibrationData() const
{
	mvIMPACT_C::IPL_BOOL iplResult;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RO_HAVE_CALIB_DATA, &iplResult))
	{ throwImpactError(); }
	
	return (IPL_TRUE == iplResult);
}

inline double CameraCalibration::getScalingFactorX() const
{
	mvIMPACT_C::IPL_DOUBLE iplResult;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_SCALE_XFACT, &iplResult))
	{ throwImpactError(); }
	
	return iplResult;
}

inline CameraCalibration& CameraCalibration::setScalingFactorX(double f)
{
	mvIMPACT_C::IPL_DOUBLE val = f;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_SCALE_XFACT, &val))
	{ throwImpactError(); }
	
	return *this;
}


inline double CameraCalibration::getScalingFactorY() const
{
	mvIMPACT_C::IPL_DOUBLE iplResult;
	
	if(!mvIMPACT_C::CC_Inquire(getHandle(), mvIMPACT_C::CC_RW_SCALE_YFACT, &iplResult))
	{ throwImpactError(); }
	
	return iplResult;
}

inline CameraCalibration& CameraCalibration::setScalingFactorY(double f)
{
	mvIMPACT_C::IPL_DOUBLE val = f;
	
	if(!mvIMPACT_C::CC_Control(getHandle(), mvIMPACT_C::CC_RW_SCALE_YFACT, &val))
	{ throwImpactError(); }
	
	return *this;
}


inline void CameraCalibration::writeToXMLFile(const std::string& fileName) const
{
	if(!mvIMPACT_C::CC_WriteToFile(getHandle(), fileName.c_str()))
	{ throwImpactError(); }
}

inline std::vector<PointDouble> CameraCalibration::getGridPointsFromImage(
				const Image& image) const
{
	// get maximum number of points as an upper limit for the destination
	// image size
	int maxPointCount = getMaxGridPointCount();
	
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplPoints(maxPointCount);
	
	mvIMPACT_C::IPL_ULONG actPointCount; // the actual point count
	if(!mvIMPACT_C::CC_GetGridPointsFromImage(
			getHandle(),
			image.getHandle(),
			iplPoints,
			&actPointCount))
	{ throwImpactError(); }

	std::vector<PointDouble> result = iplPoints.asVector();
	result.erase(result.begin()+actPointCount, result.end());
	// remove invalid part of the result vector
	
	assert(result.size() == actPointCount);
	return result;
}

inline void CameraCalibration::teachByGrid(
				const Image& image)
{
	if(!mvIMPACT_C::CC_TeachCalibrationByGrid(
			getHandle(),
			image.getHandle()))
	{ throwImpactError(); }
}

inline void CameraCalibration::teachByPoints(
				const std::vector<PointDouble> & imagePoints,
				const std::vector<PointDouble> & objectPoints
				)
{
	if(imagePoints.size() != objectPoints.size()) { 
		throwImpactException("Vectors imagePoints and objectPoints not of equal size");
	}
	
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplImagePoints(imagePoints);
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplObjectPoints(objectPoints);
	if(!mvIMPACT_C::CC_TeachCalibrationByPoints(
		getHandle(), iplImagePoints, iplObjectPoints, iplImagePoints.getSize()))	
	{ throwImpactError(); }
}

inline void CameraCalibration::teachBy3DPoints(
				const std::vector<PointDouble> & imagePoints,
				const std::vector<Point3DDouble> & objectPoints
				)
{
	if(imagePoints.size() != objectPoints.size()) { 
		throwImpactException("Vectors imagePoints and objectPoints not of equal size");
	}
	
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplImagePoints(imagePoints);
	DynArray<mvIMPACT_C::IPL_POINT3D_DOUBLE, Point3DDouble> iplObjectPoints(objectPoints);
	if(!mvIMPACT_C::CC_TeachCalibrationBy3DPoints(
		getHandle(), iplImagePoints, iplObjectPoints, iplImagePoints.getSize()))	
	{ throwImpactError(); }
}				


inline std::vector<PointDouble> CameraCalibration::transformPoints(
		const std::vector<PointDouble>& srcPoints)
{
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplSrcPoints(srcPoints);
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplDestPoints(srcPoints.size());
		
	if(!mvIMPACT_C::CC_TrObjectSpace2ImagePlane(
		getHandle(),
		iplSrcPoints,
		iplDestPoints,
		srcPoints.size()))
	{ throwImpactError(); }
	
	return iplDestPoints.asVector();		
}

inline std::vector<PointDouble> CameraCalibration::inverseTransformPoints(const std::vector<PointDouble>& srcPoints)
{
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplSrcPoints(srcPoints);
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplDestPoints(srcPoints.size());
		
	if(!mvIMPACT_C::CC_TrImagePlane2ObjectSpace(
		getHandle(),
		iplSrcPoints,
		iplDestPoints,
		srcPoints.size()))
	{ throwImpactError(); }
	
	return iplDestPoints.asVector();		
}

inline Image CameraCalibration::transformImage(const Image& S)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if(!mvIMPACT_C::CC_TrCalibratedImage(
		getHandle(),
		S.getHandle(),
		&hD))
	{ throwImpactError(); }
	
	return Image::imageFactory(hD);
}

inline void CameraCalibration::transformImage(Image& D, const Image& S)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if(!mvIMPACT_C::CC_TrCalibratedImage(
		getHandle(),
		S.getHandle(),
		&hD))
	{ throwImpactError(); }
	
	if(D.getHandle() != hD) { D = Image::imageFactory(hD); }
}


inline std::vector<Point3DDouble> CameraCalibration::transformPointsToWorld(const std::vector<PointDouble>& srcPoints)
{
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplSrcPoints(srcPoints);
	DynArray<mvIMPACT_C::IPL_POINT3D_DOUBLE, Point3DDouble> iplDestPoints(srcPoints.size());
	
	if(!mvIMPACT_C::CC_Tr2DCoordinatesTo3D(
			getHandle(),
			iplSrcPoints,
			iplDestPoints,
			srcPoints.size()))
	{ throwImpactError(); }
	
	return iplDestPoints.asVector();
}

inline std::vector<PointDouble> CameraCalibration::transformPointsFromWorld(const std::vector<Point3DDouble>& srcPoints)
{
	DynArray<mvIMPACT_C::IPL_POINT3D_DOUBLE, Point3DDouble> iplSrcPoints(srcPoints);
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble>  iplDestPoints(srcPoints.size());
	
	if(!mvIMPACT_C::CC_Tr3DCoordinatesTo2D(
			getHandle(),
			iplSrcPoints,
			iplDestPoints,
			srcPoints.size()))
	{ throwImpactError(); }
	
	return iplDestPoints.asVector();
}

inline CameraCalibration CameraCalibration::cameraCalibrationFactory(mvIMPACT_C::CC_BUFHANDLE handle)
{
	if (IPL_DONT_CARE == handle)
		return CameraCalibration();

	return CameraCalibration(handle);
}

inline void CameraCalibration::copy(const CameraCalibration & src)
{
	m_handle = src.m_handle;
	m_counter = src.m_counter;
	
	(*m_counter)++;
}
	
inline void CameraCalibration::dispose()
{
	(*m_counter)--;
	
	if(0 == (*m_counter)) {
		if(!mvIMPACT_C::CC_BufFree(getHandle()))
		{ throwImpactError(); }
		
		delete m_counter;
	}
}

inline std::string getVersionCalibration()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::CC_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

	} // namespace calibration
} // namespace mvIMPACT
#endif // #ifndef WRAP_ANY
#endif// DOXYGEN_SHOULD_SKIP_THIS


#endif // #ifndef MVIMPACT_CALIBRATION_IMPL_H_
