#ifndef MVIMPACT_DATAMATRIX_H_
#define MVIMPACT_DATAMATRIX_H_


/// \file
///

#ifndef WRAP_PYTHON
#include "mvIMPACT.h"
#endif // #ifndef WRAP_PYTHON



#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_cr2d.h>


#endif // #ifndef WRAP_PYTHON
#endif // DOXYGEN_SHOULD_SKIP_THIS

namespace mvIMPACT {

        /// \namespace mvIMPACT::datamatrix This namespace contains all classes and functions from mvIMPACT Datamatrix.
	namespace datamatrix {


/// \brief The grid trimming mode.

/// Constants that specify different grid trimming modes.
///GridTrimming selects among different ways of building a grid around the
///DataMatrix to isolate it.This parameter is intended for
///advanced users and is normally not touched. The recommended value is \c gt4.
enum GridTrimming
{
        /// Keep nodes with 3 adjacent links or more.
    gt3Adj=mvIMPACT_C::CR2D_TRIM_3_ADJ_PLUS,
	/// Keep nodes with 4 links or more.
	gt4=mvIMPACT_C::CR2D_TRIM_4_PLUS,
	/// Keep nodes with 4 adjacent links or more.
	gt4Adj=mvIMPACT_C::CR2D_TRIM_4_ADJ_PLUS,
};


/// \brief This class holds all configuration parameters of the datamatrix decoder.

/// The following parameters can be set or retrieved:
/// - spacing (distance of locator spacing rays)
/// - contrast
/// - gap size
/// - Datamatrix is black on white or white on black
/// - grid triming mode
//-----------------------------------------------------------------------------
class DatamatrixConfiguration
//-----------------------------------------------------------------------------
{
private:
	/// Unassigned buffer constant.
	enum { unassigned = -1 };
	/// The buffer handle.
	mvIMPACT_C::CR2D_BUFHANDLE handle_;
	/// Pointer to the reference counter.
	mutable int* counter_;

public:
	/// \brief Set spacing between scanlines.

	///For efficiency, the image is only probed along different profiles each spaced by a given
	///number of pixels. The spacing parameter should be chosen small enough to be sure that a
	///sufficient number of profiles hit the datamatrix, for robustness, but large enough to reduce the
	///running time.
	///The default value, is appropriate for the standard sizes.
	/// \note Odd values are always rounded to the next even value.
	/// \see mvIMPACT::datamatrix::DatamatrixConfiguration::getSpacing
	DatamatrixConfiguration& setSpacing(
		/// Scanline spacing in pixels
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="100"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int spacing); 
		

	/// \brief Get spacing between scanlines.
	
 	///For efficiency, the image is only probed along different profiles each spaced by a given
	///number of pixels. The spacing parameter should be chosen small enough to be sure that a
	///sufficient number of profiles hit the datamatrix, for robustness, but large enough to reduce the
	///running time.
	///The default value, is appropriate for the standard sizes.
	/// \see mvIMPACT::datamatrix::DatamatrixConfiguration::setSpacing
	int getSpacing() const;
	
	
	/// \brief Set gradient contrast for the locator.

	///DataMatrix cell edges correspond to steep changes between the foreground color (white label)
	///and foreground marking (black ink). The contrast parameter allows selecting the edges that
	///are strong enough. Setting a low value can result in false edge detection because of noise or
	///background texture. Setting a high value can cause some of the edges to be missed. In
	///practice, a value of 20 deals with poor contrast, 30 is appropriate for normal contrast, and 40
	///or more is suited for printed samples.
	///The default value, usually works fine for DataMatrix codes.
	/// \see mvIMPACT::datamatrix::DatamatrixConfiguration::getContrast
	DatamatrixConfiguration& setContrast(
		/// Gradient contrast
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int contrast); 
		

	/// \brief Retrieve gradient contrast for the locator.

	///DataMatrix cell edges correspond to steep changes between the foreground color (white label)
	///and foreground marking (black ink). The contrast parameter allows selecting the edges that
	///are strong enough. Setting a low value can result in false edge detection because of noise or
	///background texture. Setting a high value can cause some of the edges to be missed. In
	///practice, a value of 20 deals with poor contrast, 30 is appropriate for normal contrast, and 40
	///or more is suited for printed samples.
	///The default value, usually works fine for DataMatrix codes.
	/// \see mvIMPACT::datamatrix::DatamatrixConfiguration::setContrast
	int getContrast() const;
	
	
	
	
	/// \brief Set black on white or white on black mode.

	///Defines whether the datamatrix is black on white or white on black.
	///  - Set to \c true for black on white 
	///  - Set to \c false for white on black
	/// \see mvIMPACT::datamatrix::DatamatrixConfiguration::getBlackOnWhite
	DatamatrixConfiguration& setBlackOnWhite(
		/// Black on white mode.
		bool blackOnWhite); 
		

	/// \brief Returns which mode is used. Either black on white or white on black.

	///The flag defines whether the datamatrix is black on white or white on black.
	///  - Returns  \c true for black on white 
	///  - Returns  \c false for white on black
	/// \see mvIMPACT::datamatrix::DatamatrixConfiguration::setBlackOnWhite
	bool getBlackOnWhite() const;
	
	
	/// \brief Set grid trimming mode.

	///GridTrimming selects among different ways of building a grid around the
	///DataMatrix to isolate it.This parameter is intended for
	///advanced users and is normally not touched. The recommended value is \c gt4.
	/// \see mvIMPACT::datamatrix::DatamatrixConfiguration::getTrimming
	/// \see #GridTrimming
	DatamatrixConfiguration& setTrimming(
		/// Grid trimming mode.
		GridTrimming trimming); 
		

	/// \brief Get grid trimming mode.

	///GridTrimming selects among different ways of building a grid around the
	///DataMatrix to isolate it.This parameter is intended for
	///advanced users and is normally not touched. The recommended value is \c gt4.
	/// \see mvIMPACT::datamatrix::DatamatrixConfiguration::setTrimming
	/// \see #GridTrimming
	GridTrimming getTrimming() const;
	
	
public:
	/// \brief Construct a default DatamatrixConfiguration.
	DatamatrixConfiguration();

	/// \brief Construct a DatamatrixConfiguration object from \a src
	DatamatrixConfiguration(const DatamatrixConfiguration& src);

#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	DatamatrixConfiguration& operator=(
		const DatamatrixConfiguration& src);
#endif // #ifndef WRAP_ANY

	/// \brief Destructor
	virtual ~DatamatrixConfiguration();
	
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY    
	mvIMPACT_C::CR2D_BUFHANDLE getHandle  () const; 
	int getUseCount() const; 
	bool getIsUnique   () const; 
#endif // #ifndef WRAP_ANY
#endif
	
	
private:
	/// Dispose the buffer. If the reference count drops to zero,
	/// and the buffer was allocated it is freed.
	void dispose();
	
	void copy(const DatamatrixConfiguration& src);
	
	/// \brief Construct an allocated DatamatrixConfiguration from a handle
	explicit DatamatrixConfiguration(
		/// the buffer handle
		mvIMPACT_C::CR2D_BUFHANDLE bufhandle);
};





/// \brief Datamatrix results buffer. This class contains the results of mvIMPACT::datamatrix::decode
/// \brief or mvIMPACT::datamatrix::locate.

/// After calling mvIMPACT::datamatrix::decode or mvIMPACT::datamatrix::locate the results are kept in
/// an instance of this class.
//-----------------------------------------------------------------------------
class DatamatrixResults
//-----------------------------------------------------------------------------
{
private:
	/// Unassigned buffer constant.
	enum { unassigned = -1 };
	/// The buffer handle.
	mvIMPACT_C::CR2D_BUFHANDLE handle_;
	/// Pointer to the reference counter.
	mutable int* counter_;

public:
	/// \brief Return the number of candidates.
	int getNumberOfCandidates() const;
		
   
	/// \brief Return number of codes for a specific candidate.
		
	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	int getNumberOfCodes(
				///the index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
		
   
	/// \brief Get the type of a specific code.

	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// Valid range for \a codeIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes -1
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes
	std::string getCodeType(
				///the index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex, 
				///the index of the code
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int codeIndex=0) const;
		
   
	/// \brief Get the decoded information from a specific code.

	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// Valid range for \a codeIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes -1
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes
	std::string getCode(
				/// index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex, 
			    /// index of the code
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int codeIndex=0) const;
		
   
	/// \brief Get the number of datamatrix rows and columns from a specific code.

	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// Valid range for \a codeIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes -1
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes
	Size getDatamatrixDims(
					///index of the candidate
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int candidateIndex, 
					/// index of the code
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int codeIndex=0) const;
		

	/// \brief Get the flipped status from a specific code.

	/// A code is not flipped if seen from the front and flipped if seen from the back.
	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// Valid range for \a codeIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes -1
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes
	bool getFlipped(
		///index of the candidate
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int candidateIndex, 
		/// index of the code
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int codeIndex=0) const;


	/// \brief Get the number of datamatrix errors for a specific code

	/// Errors is the number of cell's in the datamatrix that have been read incorrectly. The amount is not normalized to the size of the code.
	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// Valid range for \a codeIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes -1
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes
	int getNumberOfErrors(
					///index of the candidate
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int candidateIndex, 
					/// index of the code
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int codeIndex=0) const;
		
   
	/// \brief Get the readability score for a specific code

	///Every cell in the datamatrix is assigned a score between 0 and 1 telling how
	///close it was found to the correct polarity. 
	/// Then the scores of all cells are averaged together.
	/// Thus, a readability score of 1, means that the code is perfectly readable. 
	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// Valid range for \a codeIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes -1
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes
	double getReadabilityScore(
					///index of the candidate
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int candidateIndex, 
					///index of the code
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int codeIndex=0) const;
   
		
	/// \brief Check if datamatrix is black on white or white on black
   
	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// Valid range for \a codeIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes -1
	///  - Returns  \c true for black on white 
	///  - Returns  \c false for white on black
	/// \see mvIMPACT::datamatrix::DatamatrixConfiguration::setBlackOnWhite
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCodes
	bool getIsBlackOnWhite(
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex, 
				///index of the code
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int codeIndex=0) const;
		
   
	/// \brief Return location of a sepecific candidate
		
	///Returns four points, that make up a rectangle that surrounds the candidate.
	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	std::vector<Point> getLocation(
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
   
		
	/// \brief Return total decoding time for all codes in milliseconds.
	double getTotalDecodingTime () const;
   
		
	/// \brief Return decoding time for a specific candidate in milliseconds.
   
	/// Valid range for \a candidateIndex is from 0 to mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates -1
	/// \see mvIMPACT::datamatrix::DatamatrixResults::getNumberOfCandidates
	double getDecodingTime (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;



public:
	/// \brief Construct a non-allocated DatamatrixResults.
	DatamatrixResults();
	
	///\brief Construct a DatamatrixResults object from \a src.
	DatamatrixResults(///the othe DatamatrixResults object
			  const DatamatrixResults& src);

#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	DatamatrixResults& operator=(///the othe DatamatrixResults object
				     const DatamatrixResults& src);
#endif // #ifndef WRAP_ANY


	/// \brief Destructor
	virtual ~DatamatrixResults();

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	mvIMPACT_C::CR2D_BUFHANDLE getHandle  () const; 
	int getUseCount() const; 
	bool getIsUnique   () const; 
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS
	
	
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	/// Factory to return a DatamatrixResults object from a handle.
	/// \module datamatrix
	static DatamatrixResults datamatrixResultsFactory(mvIMPACT_C::CR2D_BUFHANDLE buffer_handle);
#endif // #ifndef WRAP_ANY
#endif

private:
	/// construct an allocated buffer from a handle
	explicit DatamatrixResults(
		/// the buffer handle
		mvIMPACT_C::CR2D_BUFHANDLE bufhandle);
	/// increment the reference counter
	void addref() const;

	/// Dispose the buffer. If the reference count drops to zero,
	/// and the buffer was allocated it is freed.
	void dispose();
	
	void copy(const DatamatrixResults& src);
	/// share
	void share(mvIMPACT_C::CR2D_BUFHANDLE rhandle, int *r_counter);
};





/// \brief Detect and decode Data Matrix codes in an image using the default configuration.
///
/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
//-----------------------------------------------------------------------------
DatamatrixResults decodeDM(
	/// The source Image buffer to be decoded.
	const Image& source);

#ifndef I386_LINUX
/// \brief Detect and decode Data Matrix codes in an image using the default configuration.
///
/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
/// 
/// This overload can be used to visualize the runs of the quiet and busy grids 
/// used by the Data Matrix locator.
//-----------------------------------------------------------------------------
DatamatrixResults decodeDM(
	/// The source Image buffer to be decoded.
	const Image& source,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the busy runs of the Data Matrix locator.
	LineArrayOfInterest& busy_runs,
	/// The LineArrayOfInterest containing the quiet runs of the Data Matrix locator.
	LineArrayOfInterest& quiet_runs);
#endif // !I386_LINUX

/// \brief Detect and decode Data Matrix codes in an image using the default configuration.
///
/// The results will be stored in \a dest.
/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
//-----------------------------------------------------------------------------
void decodeDM(
	/// The results buffer.
	DatamatrixResults& dest,
	/// The source image buffer to be decoded.
	const Image& source);

#ifndef I386_LINUX
/// \brief Detect and decode Data Matrix codes in an image using the default configuration.
///
/// The results will be stored in \a dest.
/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
/// 
/// This overload can be used to visualize the runs of the quiet and busy grids 
/// used by the Data Matrix locator.
//-----------------------------------------------------------------------------
void decodeDM(
	/// The results buffer.
	DatamatrixResults& dest,
	/// The source image buffer to be decoded.
	const Image& source,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the busy runs of the Data Matrix locator.
	LineArrayOfInterest& busy_runs,
	/// The LineArrayOfInterest containing the quiet runs of the Data Matrix locator.
	LineArrayOfInterest& quiet_runs);
#endif // !I386_LINUX^M

/// \brief Detect and decode Data Matrix codes in an image using a specific configuration.
///
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
//-----------------------------------------------------------------------------
DatamatrixResults decodeDM(
	/// The source image buffer to be decoded.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration);

#ifndef I386_LINUX
/// \brief Detect and decode Data Matrix codes in an image using a specific configuration.
///
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
/// 
/// This overload can be used to visualize the runs of the quiet and busy grids 
/// used by the Data Matrix locator.
//-----------------------------------------------------------------------------
DatamatrixResults decodeDM(
	/// The source image buffer to be decoded.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the busy runs of the Data Matrix locator.
	LineArrayOfInterest& busy_runs,
	/// The LineArrayOfInterest containing the quiet runs of the Data Matrix locator.
	LineArrayOfInterest& quiet_runs);
#endif // !I386_LINUX^M

/// \brief Detect and decode Data Matrix codes in an image using a specific configuration.
/// 
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
/// The results will be stored in \a dest.
//-----------------------------------------------------------------------------
void decodeDM(
	/// The results buffer.
	DatamatrixResults& dest, 
	/// The source image buffer to be decoded.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration);

#ifndef I386_LINUX
/// \brief Detect and decode Data Matrix codes in an image using a specific configuration.
/// 
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
/// The results will be stored in \a dest.
/// 
/// This overload can be used to visualize the runs of the quiet and busy grids 
/// used by the Data Matrix locator.
//-----------------------------------------------------------------------------
void decodeDM(
	/// The results buffer.
	DatamatrixResults& dest, 
	/// The source image buffer to be decoded.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the busy runs of the Data Matrix locator.
	LineArrayOfInterest& busy_runs,
	/// The LineArrayOfInterest containing the quiet runs of the Data Matrix locator.
	LineArrayOfInterest& quiet_runs);
#endif // !I386_LINUX^M

// \brief Decodes a Data Matrix code at a given location.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
/// Parameter \a location takes the 4 points that form the quadrilateral of 
/// the datamatrix code in clockwise order.
//-----------------------------------------------------------------------------
DatamatrixResults decodeDMAt(
	/// The source image buffer to be decoded.
	const Image& source,
	/// The location of the Data Matrix code.
	const std::vector<Point>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,location));


/// \brief Decodes a Data Matrix code at a given location.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
/// Parameter \a location takes the 4 points that form the quadrilateral of 
/// the datamatrix code in clockwise order.
//-----------------------------------------------------------------------------
void decodeDMAt(
	/// The DatamatrixResults object.
	DatamatrixResults& dest,
	/// The source image buffer to be decoded.
	const Image& source,
	/// The location of the Data Matrix code.
	const std::vector<Point>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,location));


/// \brief Decodes a Data Matrix code at a given location.

/// This function decodes an image using the decoder configuration given with
/// parameter \a datamatrixConfiguration. 
///
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
/// Parameter \a location takes the 4 points that form the quadrilateral of 
/// the datamatrix code in clockwise order.
//-----------------------------------------------------------------------------
DatamatrixResults decodeDMAt(
	/// The source image buffer to be decoded.
	const Image& source,
	/// The datamatrix configuration to use.
	const DatamatrixConfiguration& datamatrixConfiguration,
	/// The location of the Data Matrix code.
	const std::vector<Point>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,location));


/// \brief Decodes a Data Matrix code at a given location.

/// This function decodes an image using the decoder configuration given with
/// parameter \a datamatrixConfiguration. 
///
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
/// Parameter \a location takes the 4 points that form the quadrilateral of 
/// the datamatrix code in clockwise order.
//-----------------------------------------------------------------------------
void decodeDMAt(
	/// The results buffer.
	DatamatrixResults& dest, 
	/// The source image buffer to be decoded.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration,
	/// The location of the Data Matrix code.
	const std::vector<Point>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,location));


// \brief Decodes a Data Matrix code from a cell bitmap.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
//-----------------------------------------------------------------------------
DatamatrixResults decodeDMBitmap(
	/// The cell bitmap to be decoded.
	const Image& bitmap);


/// \brief Decodes a Data Matrix code from a cell bitmap.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
//-----------------------------------------------------------------------------
void decodeDMBitmap(
	/// The DatamatrixResults object.
	DatamatrixResults& dest,
	/// The cell bitmap to be decoded.
	const Image& bitmap);


/// \brief Decodes a Data Matrix code from a cell bitmap.

/// This function decodes a cell bitmap the decoder configuration given with
/// parameter \a datamatrixConfiguration. \a datamatrixConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr2d.dec file.
//-----------------------------------------------------------------------------
DatamatrixResults decodeDMBitmap(
	/// The cell bitmap to be decoded.
	const Image& bitmap,
	/// The datamatrix configuration to use.
	const DatamatrixConfiguration& datamatrixConfiguration);


/// \brief Decodes a Data Matrix code from a cell bitmap.

/// This function decodes a cell bitmap using the decoder configuration given with
/// parameter \a datamatrixConfiguration. \a datamatrixConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr2d.dec file.
//-----------------------------------------------------------------------------
void decodeDMBitmap(
	/// The results buffer.
	DatamatrixResults& dest, 
	/// The cell bitmap to be decoded.
	const Image& bitmap,
	/// The datamatrix configuration to use.
	const DatamatrixConfiguration& datamatrixConfiguration);


/// \brief Locate Data Matrix codes in an image using the default configuration.
///
/// This function uses the standard configuration for the locating process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
//-----------------------------------------------------------------------------
DatamatrixResults locateDM(
	/// The source image buffer where Data Matrix codes will be located.
	const Image& source);

#ifndef I386_LINUX
/// \brief Locate Data Matrix codes in an image using the default configuration.
///
/// This function uses the standard configuration for the locating process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
/// 
/// This overload can be used to visualize the runs of the quiet and busy grids 
/// used by the Data Matrix locator.
//-----------------------------------------------------------------------------
DatamatrixResults locateDM(
	/// The source image buffer where Data Matrix codes will be located.
	const Image& source,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the busy runs of the Data Matrix locator.
	LineArrayOfInterest& busy_runs,
	/// The LineArrayOfInterest containing the quiet runs of the Data Matrix locator.
	LineArrayOfInterest& quiet_runs);
#endif // !I386_LINUX^M

/// \brief Locate Data Matrix codes in an image using the default configuration.
///
/// The results will be stored in \a dest.
/// This function uses the standard configuration for the locating process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
//-----------------------------------------------------------------------------
void locateDM(
	/// The results buffer.
	DatamatrixResults& dest,
	/// The source image buffer where Data Matrix codes will be located.
	const Image& source);

#ifndef I386_LINUX
/// \brief Locate Data Matrix codes in an image using the default configuration.
///
/// The results will be stored in \a dest.
/// This function uses the standard configuration for the locating process.
/// The built-in standard values suit for most Data Matrix codes,
/// however, these values can be overridden by the vl_cr2d.dec file!
/// 
/// This overload can be used to visualize the runs of the quiet and busy grids 
/// used by the Data Matrix locator.
//-----------------------------------------------------------------------------
void locateDM(
	/// The results buffer.
	DatamatrixResults& dest,
	/// The source image buffer where Data Matrix codes will be located.
	const Image& source,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the busy runs of the Data Matrix locator.
	LineArrayOfInterest& busy_runs,
	/// The LineArrayOfInterest containing the quiet runs of the Data Matrix locator.
	LineArrayOfInterest& quiet_runs);
#endif // !I386_LINUX^M

/// \brief Locate Data Matrix codes in an image using a specific configuration.
//
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
//-----------------------------------------------------------------------------
DatamatrixResults locateDM(
	/// The source image buffer where Data Matrix codes will be located.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration);

#ifndef I386_LINUX
/// \brief Locate Data Matrix codes in an image using a specific configuration.
//
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
/// 
/// This overload can be used to visualize the runs of the quiet and busy grids 
/// used by the Data Matrix locator.
//-----------------------------------------------------------------------------
DatamatrixResults locateDM(
	/// The source image buffer where Data Matrix codes will be located.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the busy runs of the Data Matrix locator.
	LineArrayOfInterest& busy_runs,
	/// The LineArrayOfInterest containing the quiet runs of the Data Matrix locator.
	LineArrayOfInterest& quiet_runs);
#endif // !I386_LINUX^M

/// \brief Locate Data Matrix codes in an image using a specific configuration.
///
/// The results will be stored in \a dest.
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
//-----------------------------------------------------------------------------
void locateDM(
	/// The results buffer.
	DatamatrixResults& dest, 
	/// The source image buffer where Data Matrix codes will be located.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration);

#ifndef I386_LINUX
/// \brief Locate Data Matrix codes in an image using a specific configuration.
///
/// The results will be stored in \a dest.
/// The configuration given with \a datamatrixConfiguration overrides the built-in configuration parameters as well
/// as the configuration parameters given in the vl_cr2d.dec file!
/// 
/// This overload can be used to visualize the runs of the quiet and busy grids 
/// used by the Data Matrix locator.
//-----------------------------------------------------------------------------
void locateDM(
	/// The results buffer.
	DatamatrixResults& dest, 
	/// The source image buffer where Data Matrix codes will be located.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the busy runs of the Data Matrix locator.
	LineArrayOfInterest& busy_runs,
	/// The LineArrayOfInterest containing the quiet runs of the Data Matrix locator.
	LineArrayOfInterest& quiet_runs);
#endif // !I386_LINUX^M


/// \brief Calculates an edge map image.

///	The so called edge map image is the basis for datamatrix location. 
///	The contents of an edge map image is affected by the contrast setting.
//-----------------------------------------------------------------------------
Image edgeMapDM(
	/// The source image.
	const Image& source);

/// \brief Calculates an edge map image.

///	The so called edge map image is the basis for datamatrix location. 
///	The contents of an edge map image is affected by the contrast setting.
//-----------------------------------------------------------------------------
 void edgeMapDM(
	    /// The destination image.
	    Image& dest,
	    /// The source image.
	    const Image& source);

/// \brief Calculates an edge map image.

///	The so called edge map image is the basis for datamatrix location. 
///	The contents of an edge map image is affected by the contrast setting.
//-----------------------------------------------------------------------------
Image edgeMapDM(
	/// The source image.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration); 

/// \brief Calculates an edge map image.

///	The so called edge map image is the basis for datamatrix location. 
///	The contents of an edge map image is affected by the contrast setting.
//-----------------------------------------------------------------------------
 void edgeMapDM(
	/// The destination image.
	Image& dest,
	/// The source image.
	const Image& source,
	/// The datamatrix configuration.
	const DatamatrixConfiguration& datamatrixConfiguration); 


/// \brief Add a datamatrix decoder.

///This function adds a decoder to the currently selected decoders. The parameter \a decoder
///specifies the decoder to be added.
///Under Windows the string passed in is used to build the name of a dynamic link library by
///adding ".dll" to the end. For example, if you pass "s1d_code39" in this string, the actual
///decoder dll that is used is named "s1d_code39.dll".
///Under Linux, the string passed in is used to build the name of a shared object by prefixing it
///with "lib" and by adding ".so" to the end. For example, if you pass "s1d_code39" in this
///string, the actual decoder dll that is used is named "libs1d_code39.so".
///The function tries to load the decoder and adds it to the list of decoders. If the decoder is
///already loaded an mvIMPACT::ImpactException is thrown.
///If the decoder could not be loaded because it does not exist or because it is not a valid
///decoder, the an mvIMPACT::ImpaictException is thrown, too.
///The \a index parameter specifies where to add the decoder. Pass 0 to add the decoder to the
///begin of the list of decoders. Pass -1 to add the decoder to the end of the list of decoders. Pass
///a positive integer to insert the decoder at the specific position. If the Index parameter is out of
///range an mvIMPACT::ImpactException is thrown.
/// \see mvIMPACT::datamatrix::removeDMDecoder
//-----------------------------------------------------------------------------
void addDMDecoder(
	/// Specifies the decoder.
	const std::string& decoder,
	/// The index of the decoder.
	/// Set to 0 to insert the decoder at the beginning of the list.
	/// Set to -1 to insert the decoder at the end of the list.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="-1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getDMDecoderCount()"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index=-1);


/// \brief Remove a datamatrix decoder.

///This function removes a decoder from the currently selected decoders. The parameter
///Decoder specifies the decoder to be removed.
///Under Windows the string passed in is used to build the name of a dynamic link library by
///adding ".dll" to the end. For example, if you pass "s1d_code39" in this string, the actual
///decoder dll that is used is named "s1d_code39.dll".
///Under Linux, the string passed in is used to build the name of a shared object by prefixing it
///with "lib" and by adding ".so" to the end. For example, if you pass "s1d_code39" in this
///string, the actual decoder dll that is used is named "libs1d_code39.so".
///The function tries to remove the decoder from the list of decoders. If the decoder is not
///loaded, the function fails and an mvIMPACT::ImpactException is thrown.
/// \see mvIMPACT::datamatrix::addDMDecoder
//-----------------------------------------------------------------------------
void removeDMDecoder(
	/// Specifies the decoder.
	const std::string& decoder);



/// \brief Return the number of loaded decoders.

///Returns the number of decoders, that were loaded via the vl_cr2d.dec file or
/// with serveral calls to mvIMPACT::datamatrix::addDMDecoder.
///\see mvIMPACT::datamatrix::addDMDecoder
///\see mvIMPACT::datamatrix::removeDMDecoder
//-----------------------------------------------------------------------------
int getDMDecoderCount();



/// \brief Return the name of a specific decoder.

///Returns the name of a specific decoder, that was loaded via the vl_cr2d.dec file
/// or by calling mvIMPACT::datamatrix::addDMDecoder.
/// \note This is the name of the decoder, as it was passed to mvIMPACT::datamatrix::addDMDecoder and not the
/// actual filename of the decoder module.
///\see mvIMPACT::datamatrix::addDMDecoder
//-----------------------------------------------------------------------------
std::string getDMDecoderName(
	/// Specifies the decoder index.							
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getDMDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index);



/// \brief Return the filename of a decoder.

///Returns the filename of a specific decoder module, that was loaded via the vl_cr2d.dev file
/// or by calling mvIMPACT::datamatrix::addDMDecoder.
/// \note This is the filename of the decoder, thus Windows and Linux deliver different results!
///\see mvIMPACT::datamatrix::addDMDecoder
///\see mvIMPACT::datamatrix::getDMDecoderName
//-----------------------------------------------------------------------------
std::string getDMDecoderFileName(
	/// Specifies the decoder index.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getDMDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index);

/// \brief Gets the expected datamatrix symbol size.
/// If you have codes with an expected size, you can speed up the decoder by setting the expected
/// size. You can use -1 to signal to the decoder that you do not know an expected size.
/// The table explains the possible row/column settings:
/// 0:	10x10
/// 1:	12x12
/// 2:	14x14
/// 3:	16x16
/// 4:	18x18
/// 5:	20x20
/// 6:	22x22
/// 7:	24x24
/// 8:	26x26
/// 9:	32x32
/// 10:	36x36
/// 11:	40x40
/// 12: 44x44
/// 13: 48x48
/// 14:	52x52
/// 15:	64x64
/// 16:	72x72
/// 17:	80x80
/// 18:	88x88
/// 19:	96x96
/// 20: 104x104
/// 21:	120x120
/// 22:	132x132
/// 23:	144x144
/// 24:	12x26
/// 25:	12x36
/// 26:	16x36
/// 27:	16x48
/// 28:	8x18
/// 29:	8x32
///Returns The expected datamatrix symbol size or -1.
/// \see mvIMPACT::barcode::setBCTryNextOnSuccess
//-----------------------------------------------------------------------------
int getDMExpectedSize(
	/// Specifies the decoder index.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getDMDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index);

/// \brief Sets the expected datamatrix symbol size.
/// If you have codes with an expected size, you can speed up the decoder by setting the expected
/// size. You can use -1 to signal to the decoder that you do not know an expected size.
/// The table explains the possible row/column settings:
/// 0:	10x10
/// 1:	12x12
/// 2:	14x14
/// 3:	16x16
/// 4:	18x18
/// 5:	20x20
/// 6:	22x22
/// 7:	24x24
/// 8:	26x26
/// 9:	32x32
/// 10:	36x36
/// 11:	40x40
/// 12: 44x44
/// 13: 48x48
/// 14:	52x52
/// 15:	64x64
/// 16:	72x72
/// 17:	80x80
/// 18:	88x88
/// 19:	96x96
/// 20: 104x104
/// 21:	120x120
/// 22:	132x132
/// 23:	144x144
/// 24:	12x26
/// 25:	12x36
/// 26:	16x36
/// 27:	16x48
/// 28:	8x18
/// 29:	8x32
//-----------------------------------------------------------------------------
void setDMExpectedSize(
	/// Specifies the decoder index.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getDMDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index,
	/// The new expected size (see table) or -1 for unknown expectation.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="29"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int expectedSize);

/// \brief Return version number from vl_cr2d module.

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionDM();

/// \brief Return version number from a decoder used by datamatrix module.

/// Under Windows the string passed in is used to build the name of a dynamic link 
/// library by adding ".dll" to the end. For example, if you pass "cr2d_datamatrix" in
/// this string, the actual decoder dll that is used is named "cr2d_datamatrix.dll".
///
/// Under Linux, the string passed in is used to build the name of a shared object
/// by prefixing it with "lib" and by adding ".so" to the end. For example, if you
/// pass "cr2d_datamatrix" in this string, the actual decoder dll that is used is
/// named "libcr2d_datamatrix.so".

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionDMDecoder(
	/// Specifies the decoder.
	const std::string& decoder);

	} // namespace datamatrix
} // namespace mvIMPACT


#include "mvIMPACT_datamatrix_impl.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::datamatrix;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

#endif // MVIMPACT_DATAMATRIX_H_
