#ifndef MVIMPACT_GMM_IMPL_H_
#define MVIMPACT_GMM_IMPL_H_


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	namespace gmm {


inline ArcPiece::ArcPiece() 
	: m_startAngle(0.0f), m_incAngle(0.0f)
{ }

inline ArcPiece::ArcPiece(double start, double inc)
	: m_startAngle(start), m_incAngle(inc)
{ }

inline double ArcPiece::getStartAngle() const
{
	return m_startAngle;
}

inline double ArcPiece::getIncAngle() const
{
	return m_incAngle;
}

inline Arc::Arc()
	: m_counter(new int(1)), m_data(new ArcData)
{
	m_data->center = PointDouble(0,0);
	m_data->radius = 1.0;
	m_data->vPieces = std::vector<ArcPiece>();
}

inline Arc::Arc(PointDouble center, double radius, const std::vector<ArcPiece>& pieces)
	: m_counter(new int(1)), m_data(new ArcData)

{
	m_data->center = center;
	m_data->radius = radius;
	m_data->vPieces = pieces;
}

inline Arc::Arc(mvIMPACT_C::GMM_ARC iplArc)
	: m_counter(new int(1)), m_data(new ArcData)
{
	m_data->center = PointDouble(iplArc.x, iplArc.y);
	m_data->radius = iplArc.radius; 
	m_data->vPieces = std::vector<ArcPiece>(iplArc.subArcs);

	for(int i = 0; i <(int)iplArc.subArcs; i++)
	{
		m_data->vPieces.at(i) = 
			ArcPiece(iplArc.pfStartAngles[i], iplArc.pfIncAngles[i]); 
	}
}

inline Arc::Arc(const Arc& src)
{
	copy(src);
}

inline Arc::~Arc()
{
	// a destructor must never throw
	try 
	{ 
		dispose(); 
	} 
	catch(...) 
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline Arc& Arc::operator=(const Arc& src)
{
	if(this != &src)
	{
		dispose();
		copy(src);
	}
	return *this;
}

inline PointDouble Arc::getCenter() const
{ 
	return m_data->center; 
}

inline double Arc::getRadius() const
{ 
	return m_data->radius; 
}

inline int  Arc::getNumberOfArcPieces() const
{ 
	return(int)(m_data->vPieces.size()); 
}

inline ArcPiece Arc::getArcPiece(int index) const
{
	if(index < 0 || index >= getNumberOfArcPieces()) 
		throwImpactException("Index out of range");

	return m_data->vPieces[index];
}

inline const std::vector<ArcPiece>& Arc::getArcPieces() const
{
	return m_data->vPieces;
}

inline GeoFeatureID Arc::getID() const
{
	return m_counter;
}

inline void Arc::dispose()
{
	(*m_counter)--;
	if(0 == *m_counter) 
	{
		delete m_data;
		delete m_counter;
	}
}

inline void Arc::copy(const Arc& src)
{
	m_data = src.m_data;
	m_counter = src.m_counter;

	(*m_counter)++;
}

inline Corner::Corner()
	: m_counter(new int(1)), m_data(new CornerData)
{
	m_data->cornerPoint = PointDouble(0,0);
	m_data->openingAngle = M_PI/2;
	m_data->bisector = M_PI/4;
	m_data->line1 = LineDouble(PointDouble(0,0), PointDouble(1,0));
	m_data->line2 = LineDouble(PointDouble(0,0), PointDouble(0,1));
}

inline Corner::Corner(const PointDouble &cornerPoint, double openingAngle, double bisector, double edge1Length, double edge2Length)
	: m_counter(new int(1)), m_data(new CornerData)
{
	m_data->cornerPoint = cornerPoint;
	m_data->openingAngle = openingAngle;
	m_data->bisector = bisector;
	m_data->line1 = 
		LineDouble(
			cornerPoint, 
			PointDouble(
				cornerPoint.getX()+cos(bisector-openingAngle/2.)*edge1Length, 
				cornerPoint.getY()+sin(bisector-openingAngle/2.)*edge1Length));
	m_data->line2 = 
		LineDouble(
			cornerPoint,
			PointDouble(
				cornerPoint.getX()+cos(bisector+openingAngle/2.)*edge2Length,
				cornerPoint.getY()+sin(bisector+openingAngle/2.)*edge2Length));
}

inline Corner::Corner(const PointDouble& cornerPoint, const PointDouble& edge1EndPoint, const PointDouble& edge2EndPoint)
	: m_counter(new int(1)), m_data(new CornerData)
{
	m_data->cornerPoint = cornerPoint;

	LineDouble line1 = m_data->line1 = LineDouble(cornerPoint, edge1EndPoint);
	LineDouble line2 = m_data->line2 = LineDouble(cornerPoint, edge2EndPoint);

	// calculate angles of lines and ensure they lie between 0 and 2*M_PI	
	double angle1 = atan2(
		line1.getP1().getY()-line1.getP0().getY(),
		line1.getP1().getX()-line1.getP0().getX());
	angle1 = fmod(2*M_PI + angle1, 2*M_PI);
	double angle2 = atan2(
		line2.getP1().getY()-line2.getP0().getY(),
		line2.getP1().getX()-line2.getP0().getX());
	angle2 = fmod(2*M_PI + angle2, 2*M_PI);

	double angleDiff = angle2 - angle1;

	// ensure that angle difference is positive
	double posAngleDiff = angleDiff + 2*M_PI;

	// ensure that angle difference is between 0 and 2*M_PI
	double normAngleDiff = fmod(posAngleDiff, 2*M_PI);

	// ensure the smaller difference angle is used
	if(normAngleDiff >= M_PI) { normAngleDiff = 2*M_PI - normAngleDiff; }

	m_data->openingAngle = normAngleDiff;

	m_data->bisector =(angle1 < angle2 ? angle1 : angle2) + m_data->openingAngle / 2.;
}

inline Corner::Corner(const Corner& src)
{
	copy(src);
}

inline Corner::Corner(mvIMPACT_C::GMM_CORNER iplCorner)
	: m_counter(new int(1)), m_data(new CornerData)
{ 
	m_data->cornerPoint = PointDouble(iplCorner.x, iplCorner.y);
	m_data->openingAngle = iplCorner.openingAngle;
	m_data->bisector = iplCorner.bisector;
	m_data->line1 = LineDouble(iplCorner.line1.x1, iplCorner.line1.y1, iplCorner.line1.x2, iplCorner.line1.y2);
	m_data->line2 = LineDouble(iplCorner.line2.x1, iplCorner.line2.y1, iplCorner.line2.x2, iplCorner.line2.y2);
}

inline Corner::~Corner() 
{ 
	// a destructor must never throw
	try 
	{ 
		dispose(); 
	} 
	catch(...) 
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline Corner& Corner::operator=(const Corner& src)
{
	if(this != &src)
	{
		dispose();
		copy(src);
	}
	return *this;
}

inline PointDouble Corner::getCornerPoint()  const
{ 
	return m_data->cornerPoint; 
}

inline double Corner::getOpeningAngle() const
{ 
	return m_data->openingAngle; 
}

inline double Corner::getBisector() const
{ 
	return m_data->bisector; 
}

inline LineDouble Corner::getLine1() const
{ 
	return m_data->line1;
}

inline LineDouble Corner::getLine2() const
{ 
	return m_data->line2;
}

inline GeoFeatureID Corner::getID() const
{
	return m_counter;
}

inline void Corner::dispose()
{
	(*m_counter)--;
	if(0 == *m_counter) 
	{
		delete m_data;
		delete m_counter;
	}
}

inline void Corner::copy(const Corner& src)
{
	m_data = src.m_data;
	m_counter = src.m_counter;

	(*m_counter)++;
}

inline GeoModel::GeoModel()
	: m_handle(unassigned), m_counter(new int(1)), m_pModelImage(NULL), 
	m_ArcHandleMap(NULL),
	m_ArcIDMap(NULL),
	m_CornerHandleMap(NULL),
	m_CornerIDMap(NULL)
{
}

inline GeoModel::GeoModel(int width, int height)
	: m_handle(unassigned), m_counter(new int(1)), m_pModelImage(NULL),
	m_ArcHandleMap(new std::map<mvIMPACT_C::GMM_FEATUREHANDLE, Arc>),
	m_ArcIDMap(new std::map<GeoFeatureID, mvIMPACT_C::GMM_FEATUREHANDLE>),
	m_CornerHandleMap(new std::map<mvIMPACT_C::GMM_FEATUREHANDLE, Corner>),
	m_CornerIDMap(new std::map<GeoFeatureID, mvIMPACT_C::GMM_FEATUREHANDLE>)
{
	if(!mvIMPACT_C::GMM_ModelAlloc(&m_handle))
		throwImpactError();

	if(!mvIMPACT_C::GMM_ModelCreateUserDefined(m_handle, width, height))
		throwImpactError();
}

inline GeoModel::GeoModel( const Image& S, int numberOfFeatures, int threshold)
	: m_handle(unassigned), m_counter(new int(1)), m_pModelImage(0),
	m_ArcHandleMap(new std::map<mvIMPACT_C::GMM_FEATUREHANDLE, Arc>),
	m_ArcIDMap(new std::map<GeoFeatureID, mvIMPACT_C::GMM_FEATUREHANDLE>),
	m_CornerHandleMap(new std::map<mvIMPACT_C::GMM_FEATUREHANDLE, Corner>),
	m_CornerIDMap(new std::map<GeoFeatureID, mvIMPACT_C::GMM_FEATUREHANDLE>)
{
	if(!mvIMPACT_C::GMM_ModelAlloc(&m_handle))
		throwImpactError();

	if(!mvIMPACT_C::GMM_ModelCreateFromImage(
		m_handle,
		S.getHandle(),
		numberOfFeatures,
		threshold))
		throwImpactError();

	// Make copy on the heap.
	m_pModelImage =(Image*)S.makeShallowCopy();
}

inline GeoModel::GeoModel( const std::string& fileName)
	: m_handle(unassigned), m_counter(new int(1)), m_pModelImage(0),
	m_ArcHandleMap(new std::map<mvIMPACT_C::GMM_FEATUREHANDLE, Arc>),
	m_ArcIDMap(new std::map<GeoFeatureID, mvIMPACT_C::GMM_FEATUREHANDLE>),
	m_CornerHandleMap(new std::map<mvIMPACT_C::GMM_FEATUREHANDLE, Corner>),
	m_CornerIDMap(new std::map<GeoFeatureID, mvIMPACT_C::GMM_FEATUREHANDLE>)
{
	if(!mvIMPACT_C::GMM_ModelAlloc(&m_handle))
		throwImpactError();

	if(!mvIMPACT_C::GMM_ModelCreateFromFile(m_handle,(mvIMPACT_C::IPL_CHAR*)fileName.c_str()))
		throwImpactError();
}


inline GeoModel::GeoModel(const GeoModel& m)
{
	copy(m);
}

inline GeoModel& GeoModel::operator=(const GeoModel& rhs)
{
	if(this != &rhs)
	{
		dispose();
		copy(rhs);
	}
	return *this;
}

inline GeoModel::~GeoModel()
{
	// a destructor must never throw
	try 
	{ 
		dispose(); 
	} 
	catch(...) 
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline int GeoModel::getUseCount() const 
{ 
	return *m_counter; 
}

inline bool GeoModel::getIsUnique() const 
{ 
	return 1 == *m_counter; 
}

inline mvIMPACT_C::GMM_MODELHANDLE GeoModel::getHandle() const 
{ 
	return m_handle; 
}

inline void GeoModel::enableCorner(const Corner& corner)
{
	if(m_CornerIDMap->find(corner.getID()) == m_CornerIDMap->end())
		throwImpactException("Corner not in model");

	// enable corner in C API:
	mvIMPACT_C::GMM_FEATUREHANDLE cornerhandle =(*m_CornerIDMap)[corner.getID()];
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(),mvIMPACT_C::MODEL_ENABLE_CORNER,&cornerhandle))
		throwImpactError();
}

inline void GeoModel::enableArc(const Arc& arc)
{
	if(m_ArcIDMap->find(arc.getID()) == m_ArcIDMap->end())
		throwImpactException("Arc not in model");

	// enable arc in C API:
	mvIMPACT_C::GMM_FEATUREHANDLE archandle =(*m_ArcIDMap)[arc.getID()];
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(),mvIMPACT_C::MODEL_ENABLE_ARC,&archandle))
		throwImpactError();
}

inline void GeoModel::disableCorner(const Corner& corner)
{
	if(m_CornerIDMap->find(corner.getID()) == m_CornerIDMap->end())
		throwImpactException("Corner not in model");

	// disable corner in C API:
	mvIMPACT_C::GMM_FEATUREHANDLE cornerhandle =(*m_CornerIDMap)[corner.getID()];
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(),mvIMPACT_C::MODEL_DISABLE_CORNER,&cornerhandle))
		throwImpactError();
}

inline void GeoModel::disableArc(const Arc& arc)
{
	if(m_ArcIDMap->find(arc.getID()) == m_ArcIDMap->end())
		throwImpactException("Arc not in model");

	// enable arc in C API:
	mvIMPACT_C::GMM_FEATUREHANDLE archandle =(*m_ArcIDMap)[arc.getID()];
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(),mvIMPACT_C::MODEL_DISABLE_ARC,&archandle))
		throwImpactError();
}

inline void GeoModel::ignoreRect(const Rectangle& rect)
{
	mvIMPACT_C::IPL_RECT iplRect = rect;
	if(!mvIMPACT_C::GMM_ModelIgnoreRect(getHandle(),&iplRect))
		throwImpactError();
}

inline void GeoModel::useRect(const Rectangle& rect)
{
	mvIMPACT_C::IPL_RECT iplRect = rect;
	if(!mvIMPACT_C::GMM_ModelUseRect(getHandle(),&iplRect))
		throwImpactError();
}

inline void GeoModel::selectEdgePoints(int threshold)
{
	if(!mvIMPACT_C::GMM_ModelSelectEdgePoints(getHandle(),threshold))
		throwImpactError();
}

inline void GeoModel::findFeatures(int numberOfFeatures)
{
	if(!mvIMPACT_C::GMM_ModelFindFeatures(getHandle(),numberOfFeatures))
		throwImpactError();
}

inline int GeoModel::countCorners(FeatureInquireScope scope) const
{
	mvIMPACT_C::IPL_ULONG count;

	if (!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_INQUIRE_FUNCTION_SCOPE, &scope))
		throwImpactError();
	if (!mvIMPACT_C::GMM_ModelCountCorners(getHandle(),&count))
		throwImpactError();

	return count;
} 

inline int GeoModel::countEnabledCorners() const
{
	mvIMPACT_C::IPL_ULONG count;
	FeatureInquireScope scope = fisOnlyEnabled;

	if (!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_INQUIRE_FUNCTION_SCOPE, &scope))
		throwImpactError();
	if (!mvIMPACT_C::GMM_ModelCountCorners(getHandle(),&count))
		throwImpactError();

	return count;
} 

inline int GeoModel::countDisabledCorners() const
{
	mvIMPACT_C::IPL_ULONG count;
	FeatureInquireScope scope = fisOnlyDisabled;

	if (!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_INQUIRE_FUNCTION_SCOPE, &scope))
		throwImpactError();
	if (!mvIMPACT_C::GMM_ModelCountCorners(getHandle(),&count))
		throwImpactError();

	return count;
} 

inline int GeoModel::countAllCorners() const
{
	mvIMPACT_C::IPL_ULONG count;
	FeatureInquireScope scope = fisAll;

	if (!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_INQUIRE_FUNCTION_SCOPE, &scope))
		throwImpactError();
	if (!mvIMPACT_C::GMM_ModelCountCorners(getHandle(),&count))
		throwImpactError();

	return count;
} 

inline int GeoModel::countArcs(FeatureInquireScope scope) const
{
	mvIMPACT_C::IPL_ULONG count;

	if (!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_INQUIRE_FUNCTION_SCOPE, &scope))
		throwImpactError();
	if (!mvIMPACT_C::GMM_ModelCountArcs(getHandle(),&count))
		throwImpactError();

	return count;
}

inline int GeoModel::countEnabledArcs() const
{
	mvIMPACT_C::IPL_ULONG count;
	FeatureInquireScope scope = fisOnlyEnabled;

	if (!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_INQUIRE_FUNCTION_SCOPE, &scope))
		throwImpactError();
	if (!mvIMPACT_C::GMM_ModelCountArcs(getHandle(),&count))
		throwImpactError();

	return count;
}

inline int GeoModel::countDisabledArcs() const
{
	mvIMPACT_C::IPL_ULONG count;
	FeatureInquireScope scope = fisOnlyDisabled;

	if (!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_INQUIRE_FUNCTION_SCOPE, &scope))
		throwImpactError();
	if (!mvIMPACT_C::GMM_ModelCountArcs(getHandle(),&count))
		throwImpactError();

	return count;
}

inline int GeoModel::countAllArcs() const
{
	mvIMPACT_C::IPL_ULONG count;
	FeatureInquireScope scope = fisAll;

	if (!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_INQUIRE_FUNCTION_SCOPE, &scope))
		throwImpactError();
	if (!mvIMPACT_C::GMM_ModelCountArcs(getHandle(),&count))
		throwImpactError();

	return count;
}

inline void GeoModel::save(const std::string& fileName) const
{
	if(!mvIMPACT_C::GMM_ModelSave(getHandle(),(mvIMPACT_C::IPL_CHAR*)fileName.c_str()))
		throwImpactError();
}

inline bool GeoModel::checkValidity()const
{
	mvIMPACT_C::IPL_BOOL valid;
	if(!mvIMPACT_C::GMM_ModelCheckValidity(getHandle(),&valid))
		throwImpactError();
	return valid == IPL_TRUE;
}

inline void GeoModel::removeCorner(const Corner& corner)
{
	if(m_CornerIDMap->find(corner.getID()) == m_CornerIDMap->end())
		throwImpactException("Corner not in model");

	// remove corner in C API:
	mvIMPACT_C::GMM_FEATUREHANDLE cornerhandle =(*m_CornerIDMap)[corner.getID()];
	if(!mvIMPACT_C::GMM_ModelRemoveCorner(getHandle(), cornerhandle))
		throwImpactError();

	// remove corner form maps:
	m_CornerHandleMap->erase(cornerhandle);
	m_CornerIDMap->erase(corner.getID());
}

inline void GeoModel::addCorner(const Corner& corner,FeatureAddScope scope)
{
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(),mvIMPACT_C::MODEL_ADD_FUNCTION_SCOPE,&scope))
		throwImpactError();

	mvIMPACT_C::GMM_FEATUREHANDLE featureHandle = addCornerRaw(corner);
	(*m_CornerHandleMap)[featureHandle] = corner;
	(*m_CornerIDMap)[corner.getID()] = featureHandle;
}

inline void GeoModel::removeArc(const Arc& arc)
{
	if(m_ArcIDMap->find(arc.getID()) == m_ArcIDMap->end())
		throwImpactException("Arc not in model");

	// remove corner in C API:
	mvIMPACT_C::GMM_FEATUREHANDLE archandle =(*m_ArcIDMap)[arc.getID()];
	if(!mvIMPACT_C::GMM_ModelRemoveArc(getHandle(), archandle))
		throwImpactError();

	// remove corner form maps:
	m_ArcHandleMap->erase(archandle);
	m_ArcIDMap->erase(arc.getID());
}

inline void GeoModel::addArc(const Arc& arc, FeatureAddScope scope)
{
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(),mvIMPACT_C::MODEL_ADD_FUNCTION_SCOPE,&scope))
		throwImpactError();

	mvIMPACT_C::GMM_FEATUREHANDLE featureHandle = addArcRaw(arc);
	(*m_ArcHandleMap)[featureHandle] = arc;
	(*m_ArcIDMap)[arc.getID()] = featureHandle;
}

inline mvIMPACT_C::GMM_FEATUREHANDLE GeoModel::addCornerRaw(const Corner& corner)
{
	mvIMPACT_C::GMM_CORNER iplCorner;
	iplCorner.x =(mvIMPACT_C::IPL_FLOAT)corner.getCornerPoint().getX();
	iplCorner.y =(mvIMPACT_C::IPL_FLOAT)corner.getCornerPoint().getY();
	iplCorner.openingAngle =(mvIMPACT_C::IPL_FLOAT)corner.getOpeningAngle();
	iplCorner.bisector =(mvIMPACT_C::IPL_FLOAT)corner.getBisector();

	iplCorner.line1.x1 =(mvIMPACT_C::IPL_FLOAT)corner.getLine1().getP0().getX();
	iplCorner.line1.y1 =(mvIMPACT_C::IPL_FLOAT)corner.getLine1().getP0().getY();
	iplCorner.line1.x2 =(mvIMPACT_C::IPL_FLOAT)corner.getLine1().getP1().getX();
	iplCorner.line1.y2 =(mvIMPACT_C::IPL_FLOAT)corner.getLine1().getP1().getY();
	iplCorner.line2.x1 =(mvIMPACT_C::IPL_FLOAT)corner.getLine2().getP0().getX();
	iplCorner.line2.y1 =(mvIMPACT_C::IPL_FLOAT)corner.getLine2().getP0().getY();
	iplCorner.line2.x2 =(mvIMPACT_C::IPL_FLOAT)corner.getLine2().getP1().getX();
	iplCorner.line2.y2 =(mvIMPACT_C::IPL_FLOAT)corner.getLine2().getP1().getY();

	mvIMPACT_C::GMM_FEATUREHANDLE handle;
	if(!mvIMPACT_C::GMM_ModelAddCorner(
		getHandle(),
		&iplCorner,
		&handle))
		throwImpactError();

	if(handle==0)
		throwImpactException("Failed to add corner");

	return handle;
}


inline mvIMPACT_C::GMM_FEATUREHANDLE GeoModel::addArcRaw(const Arc& arc)
{
	mvIMPACT_C::GMM_ARC iplArc;
	DynArray<mvIMPACT_C::IPL_FLOAT> vStarts;
	DynArray<mvIMPACT_C::IPL_FLOAT> vIncs;

	// Fill C arc
	iplArc.x      =(mvIMPACT_C::IPL_FLOAT)arc.getCenter().getX();
	iplArc.y      =(mvIMPACT_C::IPL_FLOAT)arc.getCenter().getY();
	iplArc.radius =(mvIMPACT_C::IPL_FLOAT)arc.getRadius();

	if(0 != arc.getArcPieces().size())
	{
		vStarts.allocate((int)arc.getArcPieces().size());
		vIncs.allocate((int)arc.getArcPieces().size());

		std::vector<ArcPiece>::const_iterator i; int j;
		for(i = arc.getArcPieces().begin(), j = 0; 
			i != arc.getArcPieces().end(); 
			i++, j++)
		{
			vStarts.getPointer()[j] =(mvIMPACT_C::IPL_FLOAT)i->getStartAngle();
			vIncs.getPointer()[j] =(mvIMPACT_C::IPL_FLOAT)i->getIncAngle();
		}

		iplArc.subArcs       =(int)arc.getArcPieces().size();
		iplArc.pfStartAngles = vStarts.getPointer();
		iplArc.pfIncAngles   = vIncs.getPointer();
	}
	else
	{
		iplArc.subArcs       = 0;
		iplArc.pfStartAngles = NULL;
		iplArc.pfIncAngles   = NULL;
	}

	mvIMPACT_C::GMM_FEATUREHANDLE handle;
	if(!mvIMPACT_C::GMM_ModelAddArc(getHandle(), &iplArc, &handle))
		throwImpactError();

	return handle;
}

inline GeoModel& GeoModel::setMask(const Image& mask)
{
	if(!mvIMPACT_C::GMM_ModelSetMask(getHandle(), mask.getHandle()))
		throwImpactError();
	return *this;
}

inline Image GeoModel::getMask() const
{
	mvIMPACT_C::IPL_BUFHANDLE hD = -1;
	if(!mvIMPACT_C::GMM_ModelGetMask(getHandle(), &hD))
		throwImpactError();
	return Image::imageFactory(hD);
}

inline void GeoModel::getMask(Image& D) const
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if(!mvIMPACT_C::GMM_ModelGetMask(getHandle(), &hD))
		throwImpactError();
	if(hD != D.getHandle()) 
		D = Image::imageFactory(hD);
}

inline GeoModel& GeoModel::setEdgePoints(const Image& edgePoints)
{
	if(!mvIMPACT_C::GMM_ModelSetEdgePoints(getHandle(), edgePoints.getHandle()))
		throwImpactError();
	return *this;
}

inline Image GeoModel::getEdgePoints() const
{
	mvIMPACT_C::IPL_BUFHANDLE hD = -1;
	if(!mvIMPACT_C::GMM_ModelGetEdgePoints(getHandle(), &hD))
		throwImpactError();
	return Image::imageFactory(hD);
}

inline void GeoModel::getEdgePoints(Image& D) const
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if(!mvIMPACT_C::GMM_ModelGetEdgePoints(getHandle(), &hD))
		throwImpactError();
	if(hD != D.getHandle()) 
		D = Image::imageFactory(hD);
}

inline Corner GeoModel::getCornerFromHandle(mvIMPACT_C::GMM_FEATUREHANDLE featureHandle) const
{
	if(m_CornerHandleMap->find(featureHandle) == m_CornerHandleMap->end()) 
	{
		// if there is no Corner object in the map yet, retrieve it from the
		// C-Layer and add it to the map(and add this GeoModel to the map of the newly
		// created Corner object)

		mvIMPACT_C::GMM_CORNER iplCorner;
		if(!mvIMPACT_C::GMM_ModelGetCorner(getHandle(),featureHandle,&iplCorner))
			throwImpactError();

		Corner corner = iplCorner; 
		(*m_CornerHandleMap)[featureHandle] = corner;
		(*m_CornerIDMap)[corner.getID()] = featureHandle;

		return corner;		
	} 
	else 
		return(*m_CornerHandleMap)[featureHandle];
}

inline Arc GeoModel::getArcFromHandle(mvIMPACT_C::GMM_FEATUREHANDLE featureHandle) const
{
	if(m_ArcHandleMap->find(featureHandle) == m_ArcHandleMap->end()) 
	{
		// if there is no Arc object in the map yet, retrieve it from the
		// C-Layer and add it to the map(and add this GeoModel to the map of the newly
		// created Arc object)

		mvIMPACT_C::GMM_ARC iplArc;
		if(!mvIMPACT_C::GMM_ModelGetArc(getHandle(),featureHandle,&iplArc))
			throwImpactError();

		Arc arc = iplArc; // convert GMM_ARC into Arc using the conversion constructor

		// now we can free the GMM_ARC struct
		if(!mvIMPACT_C::GMM_ModelFreeArc(getHandle(), &iplArc))
			throwImpactError();

		// make entries into the maps:
		(*m_ArcHandleMap)[featureHandle] = arc;
		(*m_ArcIDMap)[arc.getID()] = featureHandle;

		return arc;
	} 
	else 
		return(*m_ArcHandleMap)[featureHandle];
}

inline GeoModel& GeoModel::setImage(const Image& S)
{
	if(!mvIMPACT_C::GMM_ModelSetImage(getHandle(), S.getHandle()))
		throwImpactError();

	if(m_pModelImage) 
	{ 
		// If the image is set already, overwrite the existing one with the new one
		(*m_pModelImage) = S; 
	}
	else 
	{ 
		// If there is no image set yet, create a new copy on the heap:
		m_pModelImage =(Image*)S.makeShallowCopy(); 
	}

	return *this;
}

inline Image GeoModel::getImage() const
{
	if(m_pModelImage) 
		return *m_pModelImage; 
	else 
		throwImpactException("No model image set");
}

inline Corner GeoModel::getCorner(int index) const
{
	DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(countCorners());

	// Retrieve all handles
	if(!mvIMPACT_C::GMM_ModelGetCornerHandles(getHandle(), vHandles.getPointer()))
		throwImpactError();

	return getCornerFromHandle(vHandles.getPointer()[index]);
}

inline std::vector<Corner> GeoModel::getCorners(FeatureInquireScope scope) const
{
	std::vector<Corner> vCorners;
	int numberOfCorners = countCorners(scope);
	if(numberOfCorners >0) 
	{
		DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(numberOfCorners);

		// Retrieve all handles
		if(!mvIMPACT_C::GMM_ModelGetCornerHandles(getHandle(), vHandles.getPointer()))
			throwImpactError();

		// Iterate over all handles and create Corner objects that are put into vCorners.
		for(int i = 0; i < vHandles.getSize(); i++) 
			vCorners.push_back(getCornerFromHandle(vHandles.getPointer()[i]));
	}

	return vCorners;
}

inline std::vector<Corner> GeoModel::getEnabledCorners() const
{
	std::vector<Corner> vCorners;
	int numberOfCorners = countEnabledCorners();
	if(numberOfCorners >0) 
	{
		DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(numberOfCorners);

		// Retrieve all handles
		if(!mvIMPACT_C::GMM_ModelGetCornerHandles(getHandle(), vHandles.getPointer()))
			throwImpactError();

		// Iterate over all handles and create Corner objects that are put into vCorners.
		for(int i = 0; i < vHandles.getSize(); i++) 
			vCorners.push_back(getCornerFromHandle(vHandles.getPointer()[i]));
	}

	return vCorners;
}

inline std::vector<Corner> GeoModel::getDisabledCorners() const
{
	std::vector<Corner> vCorners;
	int numberOfCorners = countDisabledCorners();
	if(numberOfCorners >0) 
	{
		DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(numberOfCorners);

		// Retrieve all handles
		if(!mvIMPACT_C::GMM_ModelGetCornerHandles(getHandle(), vHandles.getPointer()))
			throwImpactError();

		// Iterate over all handles and create Corner objects that are put into vCorners.
		for(int i = 0; i < vHandles.getSize(); i++) 
			vCorners.push_back(getCornerFromHandle(vHandles.getPointer()[i]));
	}

	return vCorners;
}

inline std::vector<Corner> GeoModel::getAllCorners() const
{
	std::vector<Corner> vCorners;
	int numberOfCorners = countAllCorners();
	if(numberOfCorners >0) 
	{
		DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(numberOfCorners);

		// Retrieve all handles
		if(!mvIMPACT_C::GMM_ModelGetCornerHandles(getHandle(), vHandles.getPointer()))
			throwImpactError();

		// Iterate over all handles and create Corner objects that are put into vCorners.
		for(int i = 0; i < vHandles.getSize(); i++) 
			vCorners.push_back(getCornerFromHandle(vHandles.getPointer()[i]));
	}

	return vCorners;
}

inline Arc GeoModel::getArc(int index) const
{
	DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(countArcs());

	// Retrieve all handles
	if(!mvIMPACT_C::GMM_ModelGetArcHandles(getHandle(), vHandles.getPointer()))
		throwImpactError();

	return getArcFromHandle(vHandles.getPointer()[index]);	
}

inline std::vector<Arc> GeoModel::getArcs(FeatureInquireScope scope) const
{
	std::vector<Arc> vArcs;
	int numberOfArcs = countArcs(scope);
	if(numberOfArcs >0) 
	{
		DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(numberOfArcs);

		// Retrieve all handles
		if(!mvIMPACT_C::GMM_ModelGetArcHandles(getHandle(), vHandles.getPointer()))
			throwImpactError();

		// Iterate over all handles and create Arc objects that are put into vArcs.
		for(int i = 0; i < vHandles.getSize(); i++) 
			vArcs.push_back(getArcFromHandle(vHandles.getPointer()[i]));
	}

	return vArcs;
}

inline std::vector<Arc> GeoModel::getEnabledArcs() const
{
	std::vector<Arc> vArcs;
	int numberOfArcs = countEnabledArcs();
	if(numberOfArcs >0) 
	{
		DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(numberOfArcs);

		// Retrieve all handles
		if(!mvIMPACT_C::GMM_ModelGetArcHandles(getHandle(), vHandles.getPointer()))
			throwImpactError();

		// Iterate over all handles and create Arc objects that are put into vArcs.
		for(int i = 0; i < vHandles.getSize(); i++) 
			vArcs.push_back(getArcFromHandle(vHandles.getPointer()[i]));
	}

	return vArcs;
}

inline std::vector<Arc> GeoModel::getDisabledArcs() const
{
	std::vector<Arc> vArcs;
	int numberOfArcs = countDisabledArcs();
	if(numberOfArcs >0) 
	{
		DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(numberOfArcs);

		// Retrieve all handles
		if(!mvIMPACT_C::GMM_ModelGetArcHandles(getHandle(), vHandles.getPointer()))
			throwImpactError();

		// Iterate over all handles and create Arc objects that are put into vArcs.
		for(int i = 0; i < vHandles.getSize(); i++) 
			vArcs.push_back(getArcFromHandle(vHandles.getPointer()[i]));
	}

	return vArcs;
}

inline std::vector<Arc> GeoModel::getAllArcs() const
{
	std::vector<Arc> vArcs;
	int numberOfArcs = countAllArcs();
	if(numberOfArcs >0) 
	{
		DynArray<mvIMPACT_C::GMM_FEATUREHANDLE> vHandles(numberOfArcs);

		// Retrieve all handles
		if(!mvIMPACT_C::GMM_ModelGetArcHandles(getHandle(), vHandles.getPointer()))
			throwImpactError();

		// Iterate over all handles and create Arc objects that are put into vArcs.
		for(int i = 0; i < vHandles.getSize(); i++) 
			vArcs.push_back(getArcFromHandle(vHandles.getPointer()[i]));
	}

	return vArcs;
}

inline int GeoModel::getWidth() const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_ModelInquire(getHandle(),mvIMPACT_C::MODEL_WIDTH,(void*)&result))
		throwImpactError();
	return(int)result;
}

inline int GeoModel::getHeight() const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_ModelInquire(getHandle(),mvIMPACT_C::MODEL_HEIGHT,(void*)&result))
		throwImpactError();
	return(int)result;
}


inline GeoModel& GeoModel::setScalingSteps(int scalingSteps)
{
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_SCALING_STEPS,(void*)& scalingSteps))
		throwImpactError();
	return *this;

}
inline int GeoModel::getScalingSteps () const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_ModelInquire(getHandle(),mvIMPACT_C::MODEL_SCALING_STEPS,(void*)&result))
		throwImpactError();
	return(int)result;
}


inline GeoModel& GeoModel::setRotationCenterX(double x)
{
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_ROTATION_CENTER_X,(void*)& x))
		throwImpactError();
	return *this;
}

inline double GeoModel::getRotationCenterX() const
{
	double result;
	if(!mvIMPACT_C::GMM_ModelInquire(getHandle(),mvIMPACT_C::MODEL_ROTATION_CENTER_X,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoModel& GeoModel::setRotationCenterY(double y)
{
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_ROTATION_CENTER_Y,(void*)& y))
		throwImpactError();
	return *this;
}

inline double GeoModel::getRotationCenterY() const
{
	double result;
	if(!mvIMPACT_C::GMM_ModelInquire(getHandle(),mvIMPACT_C::MODEL_ROTATION_CENTER_Y,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoModel& GeoModel::setEdgeDetectorThreshold(int threshold)
{
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_EDGE_DETECTOR_THRESHOLD,(void*)& threshold))
		throwImpactError();
	return *this;
}

inline int GeoModel::getEdgeDetectorThreshold() const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_ModelInquire(getHandle(),mvIMPACT_C::MODEL_EDGE_DETECTOR_THRESHOLD,(void*)&result))
		throwImpactError();
	return(int)result;
}

inline GeoModel& GeoModel::setUpperThreshold(int threshold)
{
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_UPPER_THRESHOLD,(void*)& threshold))
		throwImpactError();
	return *this;
}

inline int GeoModel::getUpperThreshold() const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_ModelInquire(getHandle(),mvIMPACT_C::MODEL_UPPER_THRESHOLD,(void*)&result))
		throwImpactError();
	return(int)result;
}

inline GeoModel& GeoModel::setVectorizationError(double maxError)
{
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_VECTORIZATION_ERROR,(void*)& maxError))
		throwImpactError();
	return *this;
}

inline double GeoModel::getVectorizationError() const
{
	double result;
	if(!mvIMPACT_C::GMM_ModelInquire(getHandle(),mvIMPACT_C::MODEL_VECTORIZATION_ERROR,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoModel& GeoModel::setMinRatioArc(double ratio)
{
	if(!mvIMPACT_C::GMM_ModelControl(getHandle(), mvIMPACT_C::MODEL_MIN_RATIO_ARC,(void*)& ratio))
		throwImpactError();
	return *this;
}

inline double GeoModel::getMinRatioArc() const
{
	double result;
	if(!mvIMPACT_C::GMM_ModelInquire(getHandle(),mvIMPACT_C::MODEL_MIN_RATIO_ARC,(void*)&result))
		throwImpactError();
	return result;
}

inline void GeoModel::dispose()
{
	(*m_counter)--;
	if(0 == *m_counter)
	{
		if(unassigned != m_handle) 
		{
			if(!mvIMPACT_C::GMM_ModelFree(m_handle))
				throwImpactError();
		}

		// no problem when delete is called on a NULL pointer
		delete m_ArcHandleMap;
		delete m_ArcIDMap;
		delete m_CornerHandleMap;
		delete m_CornerIDMap;
		delete m_pModelImage;

		delete m_counter;
		m_counter = 0;
	}
}

inline void GeoModel::copy(const GeoModel& src)
{
	m_handle = src.m_handle;
	m_counter = src.m_counter;
	m_pModelImage = src.m_pModelImage;
	m_ArcHandleMap = src.m_ArcHandleMap;
	m_ArcIDMap = src.m_ArcIDMap;
	m_CornerHandleMap = src.m_CornerHandleMap;
	m_CornerIDMap = src.m_CornerIDMap;

	(*m_counter)++;
}

inline double GeoMatchData::getTransX() const
{
	return m_transX;
}

inline GeoMatchData& GeoMatchData::setTransX(double transX)
{
	m_transX = transX;
	return *this;
}

inline double GeoMatchData::getTransY() const
{
	return m_transY;
}

inline GeoMatchData& GeoMatchData::setTransY(double transY)
{
	m_transY = transY;
	return *this;
}

inline double GeoMatchData::getRotation() const
{
	return m_rotation;
}

inline GeoMatchData& GeoMatchData::setRotation(double rotation)
{
	m_rotation = rotation;
	return *this;
}

inline double GeoMatchData::getScale() const
{
	return m_scale;
}

inline GeoMatchData& GeoMatchData::setScale(double scale)
{
	m_scale = scale;
	return *this;
}

inline double GeoMatchData::getScore() const
{
	return m_score;
}

inline GeoMatchData& GeoMatchData::setScore(double score)
{
	m_score = score;
	return *this;
}

inline int GeoMatchData::getFoundArcs() const
{
	return m_foundArcs;
}

inline GeoMatchData& GeoMatchData::setFoundArcs(int foundArcs)
{
	m_foundArcs = foundArcs;
	return *this;
}

inline int GeoMatchData::getFoundCorners() const
{
	return m_foundCorners;
}

inline GeoMatchData& GeoMatchData::setFoundCorners(int foundCorners)
{
	m_foundCorners = foundCorners;
	return *this;
}


inline GeoMatchResult GeoMatcher::findGeoModel(const GeoModel& model, const Image& S)
{
	if(!mvIMPACT_C::GMM_MatcherFindModel(getHandle(), model.getHandle(), S.getHandle()))
		throwImpactError();

	return GeoMatchResult(*this, model);
}

inline GeoMatcher::GeoMatcher()
	: m_handle(0), m_counter(0)
{
	if(!mvIMPACT_C::GMM_MatcherAlloc(&m_handle))
		throwImpactError();

	m_counter = new int(1);
}

inline GeoMatcher::GeoMatcher(const GeoMatcher& m)
{
	copy(m);	
}

inline GeoMatcher& GeoMatcher::operator=(const GeoMatcher& rhs)
{
	if(this != &rhs)
	{
		dispose();
		copy(rhs);
	}
	return *this;
}

inline GeoMatcher::~GeoMatcher()
{
	// a destructor must never throw
	try 
	{ 
		dispose(); 
	} 
	catch(...) 
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}	
}

inline int GeoMatcher::getUseCount() const
{ 
	return *m_counter; 
}

inline bool GeoMatcher::getIsUnique () const 
{ 
	return 1 == *m_counter; 
}

inline mvIMPACT_C::GMM_MATCHERHANDLE GeoMatcher::getHandle() const 
{ 
	return m_handle; 
}

inline GeoMatcher& GeoMatcher::setMaxRotation(double rot)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_MAX_ROTATION,(void*)& rot))
		throwImpactError();
	return *this;
}

inline double GeoMatcher::getMaxRotation() const
{
	double result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_MAX_ROTATION,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoMatcher& GeoMatcher::setLowerScalingBound(double scaling)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_LOWER_SCALING_BOUND,(void*)& scaling))
		throwImpactError();
	return *this;
}

inline double GeoMatcher::getLowerScalingBound() const
{
	double result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_LOWER_SCALING_BOUND,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoMatcher& GeoMatcher::setUpperScalingBound(double scaling)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_UPPER_SCALING_BOUND,(void*)& scaling))
		throwImpactError();
	return *this;
}

inline double GeoMatcher::getUpperScalingBound() const
{
	double result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_UPPER_SCALING_BOUND,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoMatcher& GeoMatcher::setMinScore(double score)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_MIN_SCORE,(void*)& score))
		throwImpactError();
	return *this;
}

inline double GeoMatcher::getMinScore() const
{
	double result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_MIN_SCORE,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoMatcher& GeoMatcher::setMaxNumberOfMatches(int number )
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_NUMBER_OF_MATCHES,(void*)& number))
		throwImpactError();
	return *this;
}

inline int GeoMatcher::getMaxNumberOfMatches() const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_NUMBER_OF_MATCHES,(void*)&result))
		throwImpactError();
	return(int)result;
}

inline GeoMatcher& GeoMatcher::setVerificationThreshold(int threshold)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_VERIFICATION_THRESHOLD,(void*)& threshold))
		throwImpactError();
	return *this;
}

inline int GeoMatcher::getVerificationThreshold() const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_VERIFICATION_THRESHOLD,(void*)&result))
		throwImpactError();
	return(int)result;
}

inline GeoMatcher& GeoMatcher::setMaxAllowedTime(int maxtime)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_MAX_ALLOWED_TIME,(void*)& maxtime))
		throwImpactError();
	return *this;
}

inline int GeoMatcher::getMaxAllowedTime() const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_MAX_ALLOWED_TIME,(void*)&result))
		throwImpactError();
	return(int)result;
}

inline GeoMatcher& GeoMatcher::setAngleTolerance(double angle)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_ANGLE_TOLERANCE,(void*)& angle))
		throwImpactError();
	return *this;
}

inline double GeoMatcher::getAngleTolerance() const
{
	double result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_ANGLE_TOLERANCE,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoMatcher& GeoMatcher::setRadiusTolerance(double tolerance)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_RADIUS_TOLERANCE,(void*)& tolerance))
		throwImpactError();
	return *this;
}

inline double GeoMatcher::getRadiusTolerance() const
{
	double result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_RADIUS_TOLERANCE,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoMatcher& GeoMatcher::setDistanceTolerance(double distance)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::MATCHER_DISTANCE_TOLERANCE,(void*)& distance))
		throwImpactError();
	return *this;
}

inline double GeoMatcher::getDistanceTolerance() const
{
	double result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::MATCHER_DISTANCE_TOLERANCE,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoMatcher& GeoMatcher::setEdgeDetectorThreshold(int threshold)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::SCENE_EDGE_DETECTOR_THRESHOLD,(void*)& threshold))
		throwImpactError();
	return *this;
}

inline int GeoMatcher::getEdgeDetectorThreshold() const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::SCENE_EDGE_DETECTOR_THRESHOLD,(void*)&result))
		throwImpactError();
	return(int)result;
}

inline GeoMatcher& GeoMatcher::setUpperThreshold(int threshold)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::SCENE_UPPER_THRESHOLD,(void*)& threshold))
		throwImpactError();
	return *this;
}

inline int GeoMatcher::getUpperThreshold() const
{
	mvIMPACT_C::IPL_ULONG result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::SCENE_UPPER_THRESHOLD,(void*)&result))
		throwImpactError();
	return(int)result;
}

inline GeoMatcher& GeoMatcher::setVectorizationError(double error)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::SCENE_VECTORIZATION_ERROR,(void*)& error))
		throwImpactError();
	return *this;
}

inline double GeoMatcher::getVectorizationError() const
{
	double result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::SCENE_VECTORIZATION_ERROR,(void*)&result))
		throwImpactError();
	return result;
}

inline GeoMatcher& GeoMatcher::setMinRatioArc(double ratio)
{
	if(!mvIMPACT_C::GMM_MatcherControl(getHandle(), mvIMPACT_C::SCENE_MIN_RATIO_ARC,(void*)& ratio))
		throwImpactError();
	return *this;
}

inline double GeoMatcher::getMinRatioArc() const
{
	double result;
	if(!mvIMPACT_C::GMM_MatcherInquire(getHandle(),mvIMPACT_C::SCENE_MIN_RATIO_ARC,(void*)&result))
		throwImpactError();
	return result;
}

inline void GeoMatcher::dispose()
{
	if(0 == --*m_counter)
	{
		delete m_counter;
		m_counter = 0;

		if(unassigned != m_handle)
		{
			if(!mvIMPACT_C::GMM_MatcherFree(m_handle))
				throwImpactError();
			m_handle = unassigned;
		}
	}
}

inline void GeoMatcher::copy(const GeoMatcher& src)
{
	m_handle = src.m_handle;
	m_counter = src.m_counter;

	(*m_counter)++;
}


inline GeoMatchResult::GeoMatchResult(const GeoMatcher& matcher, const GeoModel& model)
{
	m_pMatches = 0;
	m_count    = 0;

	if(!mvIMPACT_C::GMM_MatcherGetMatches 
		(matcher.getHandle(), 
		&m_pMatches, 
		&m_count))
		throwImpactError();

	m_counter = new int(1);
	m_matcher = matcher;
	m_model   = model;
}

inline GeoMatchResult::GeoMatchResult()
	: m_pMatches(0), m_count(0), m_counter(new int(1))
{ }

inline GeoMatchResult::~GeoMatchResult()
{
	// a destructor must never throw
	try 
	{ 
		dispose();
	} 
	catch(...) 
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline GeoMatchResult::GeoMatchResult(const GeoMatchResult& m)
{
	copy(m);
}

inline GeoMatchResult& GeoMatchResult::operator=(const GeoMatchResult& rhs)
{
	if(this != &rhs)
	{
		dispose();
		copy(rhs);
	}
	return *this;
}

inline int GeoMatchResult::getNumberOfMatches() const
{
	return m_count;
}

inline GeoMatchData GeoMatchResult::getMatch(int index) const
{
	if(index < 0 || index >=(int)m_count)
		throwImpactException("Index out of range");

	GeoMatchData data;
	data.setTransX(m_pMatches[index].x);
	data.setTransY(m_pMatches[index].y);
	data.setRotation(m_pMatches[index].r);
	data.setScale(m_pMatches[index].s);
	data.setScore(m_pMatches[index].score);
	data.setFoundArcs(m_pMatches[index].foundArcs);
	data.setFoundCorners(m_pMatches[index].foundCorners);
	return data;
}

inline std::vector<Corner> GeoMatchResult::getFoundCorners(int index) const
{
	if(index < 0 || index>=(int)m_count)
		throwImpactException("Index out of range");

	std::vector<Corner> vCorners;
	for(int i=0; i<(int)(m_pMatches[index].foundCorners); ++i)
		vCorners.push_back(m_model.getCornerFromHandle(m_pMatches[index].pCornerHandles[i]));

	return vCorners;
}

inline std::vector<Arc> GeoMatchResult::getFoundArcs(int index) const
{
	if(index < 0 || index>=(int)m_count)
		throwImpactException("Index out of range");

	std::vector<Arc> vArcs;
	for(int i=0; i <(int)(m_pMatches[index].foundArcs); ++i)
		vArcs.push_back(m_model.getArcFromHandle(m_pMatches[index].pArcHandles[i]));

	return vArcs;
}

inline std::vector<Corner> GeoMatchResult::getSceneCorners(int index) const
{
	if(index < 0 || index>=(int)m_count)
		throwImpactException("Index out of range");

	std::vector<Corner> vCorners;
	for(int i=0; i <(int)(m_pMatches[index].foundCorners); ++i)
	{
		Corner corner = m_pMatches[index].pSceneCorners[i];
		vCorners.push_back(corner);
	}
	return vCorners;
}

inline std::vector<Arc> GeoMatchResult::getSceneArcs(int index) const
{
	if(index < 0 || index>=(int)m_count)
		throwImpactException("Index out of range");

	std::vector<Arc> vArcs;
	for(int i=0; i <(int)(m_pMatches[index].foundArcs); ++i)
	{
		Arc arc = m_pMatches[index].pSceneArcs[i];
		vArcs.push_back(arc);
	}
	return vArcs;
}

inline std::vector<Point> GeoMatchResult::getTransformedEdgePoints(int index)const
{
	if(index < 0 || index>=(int)m_count)
		throwImpactException("Index out of range");

	std::vector<mvIMPACT::Point> vPoints;

	//get number of edge points in model
	mvIMPACT_C::IPL_ULONG countEdgePoints;
	mvIMPACT_C::GMM_MODELHANDLE handle = m_model.getHandle();
	if(!mvIMPACT_C::GMM_ModelCountEdgePoints(handle,&countEdgePoints))
		throwImpactError();

	if(countEdgePoints > 0)
	{
		//allocate memory
		mvIMPACT_C::IPL_POINT *points = new mvIMPACT_C::IPL_POINT[countEdgePoints];
		//get the transformed edge points
		if(!mvIMPACT_C::GMM_MatcherGetSceneEdgePoints(m_matcher.getHandle(),&(m_pMatches[index]),points))
			throwImpactError();

		for(mvIMPACT_C::IPL_ULONG i=0; i<countEdgePoints;i++) 
			vPoints.push_back(Point(points[i].xpos,points[i].ypos));

		//free memory
		delete[] points;
	}

	return vPoints;
}

inline void GeoMatchResult::dispose()
{
	if(0 == --*m_counter)
	{
		delete m_counter;
		m_counter = 0;

		if(0 != m_pMatches)
		{
			if(!mvIMPACT_C::GMM_MatcherFreeMatches(m_matcher.getHandle(),m_pMatches))
				throwImpactError();
			m_pMatches = 0;
		}
	}
}

inline void GeoMatchResult::copy(const GeoMatchResult& src)
{
	m_counter = src.m_counter;
	m_pMatches = src.m_pMatches;
	m_count    = src.m_count;
	m_matcher  = src.m_matcher;
	m_model    = src.m_model;

	(*m_counter)++;
}

inline std::string getVersionGMM()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::GMM_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

	} // namespace gmm
} // namespace mvIMPACT
#endif // #ifndef WRAP_ANY
#endif// DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_GMM_IMPL_H_
