#ifndef MVIMPACT_MATCH_H_
#define MVIMPACT_MATCH_H_


/// \file
///

#ifndef WRAP_PYTHON

#include <utility>

#include "mvIMPACT.h"

#endif // #ifndef WRAP_PYTHON


#ifndef  DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_pat.h>
#include <vl_autoreg.h>


#endif // #ifndef WRAP_PYTHON
#endif // DOXYGEN_SHOULD_SKIP_THIS


namespace mvIMPACT {

        /// \namespace mvIMPACT::match This namespace contains all classes and functions from mvIMPACT Match.
	namespace match {

enum PatMatchMode
{
  /// Set this value with mvIMPACT::match::PatModel::setNumberOfMatches to retrieve all matches instead 
  /// of a certain number of matches.
  pmmFindAll = PAT_FIND_ALL
};


/// \brief Different modes for finding a best model in an image. 

///This is used
/// by mvIMPACT::match::findBestModel.
enum BestModelMode
{
	/// Find best model based on maximum entropy.
	bmmEntropy  = mvIMPACT_C::PAT_ENTROPY,
	/// Find best model based on maximum contrast.
	bmmContrast	= mvIMPACT_C::PAT_CONTRAST,
	/// Find best model based on maximum gradient sum.
	bmmGradient = mvIMPACT_C::PAT_GRADIENT
};

/// \brief Supported condition codes which can be used with mvIMPACT::match::presetModelPixel.
enum PresetCondition
{
	/// If (image pixel < condValue1) then model pixel is set to "don't care".
	pcLess     = mvIMPACT_C::PAT_LESS,
	/// If (image pixel > condValue1) then model pixel is set to "don't care".
	pcGreater  = mvIMPACT_C::PAT_GREATER,
	/// If (image pixel == condValue1) then model pixel is set to "don't care".
	pcEqual    = mvIMPACT_C::PAT_EQUAL,
	/// If (image pixel < condValue1 && image pixel > condValue2) then model pixel is set to "don't care".
	pcInRange  = mvIMPACT_C::PAT_IN_RANGE,
	/// If (image pixel > condValue1 && image pixel < condValue2) then model pixel is set to "don't care".
	pcOutRange = mvIMPACT_C::PAT_OUT_RANGE
};


/// \brief Supported binning modes which can be used with mvIMPACT::match::PatModel.
enum BinningMode
{
	/// The mean value of the neighbourhood is used during binning (default).
	bmMean = 0,
	/// The minimum value of the neighbourhood is used during binning.
	bmMin = 1,
	/// The maximum value of the neighbourhood is used during binning.
	bmMax = 2
};



/// \brief This class represents a single match. 

/// This class is used by mvIMPACT::match::PatMatchResult to represent a single match.
/// A single match consists of the coordinates of the model center
/// in the target image and the match score. mvIMPACT::match::PatMatchResult contains
/// several mvIMPACT::match::MatchData objects, which can be retrieved in a vector by calling
/// mvIMPACT::match::PatMatchResult::getMatches.
/// \see mvIMPACT::match::PatMatchResult::getMatches
//-----------------------------------------------------------------------------
class MatchData
//-----------------------------------------------------------------------------
{
public:

	/// \brief Construct an MatchData object.
  
	///The default values for all parameters is 0.
	/// Valid range for \a score is [0,100]
	///
	 // \internal
	 // If you change the default initializations, you have to 
	 // change them in base.i as well in the tuple to class conversion !
	MatchData (
			/// x-coordinate of the match
			double x=0.0, 
			/// y-coordinate of the match
			double y=0.0, 
			/// quality of the match
			/// \xmlonly
			/// <mvParamRange>
			/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
			/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
			/// </mvParamRange>
			/// \endxmlonly			   	
			double score=0.0);
  




  /// \brief Retrieve the x-coordinate of the match.
  
  ///
  ///\see mvIMPACT::match::MatchData::setX
  ///\see mvIMPACT::match::MatchData::getY
  double getX() const;
  



  /// \brief Set the x-coordinate of the match.
  
  ///
  ///\see mvIMPACT::match::MatchData::getX
  ///\see mvIMPACT::match::MatchData::setY
  MatchData& setX(///the new x-coordinate of the match
		  double x);




  /// \brief Retrieve the y-coordinate of the match.
  
  ///
  ///\see mvIMPACT::match::MatchData::setY
  ///\see mvIMPACT::match::MatchData::getX
  double getY() const;





  /// \brief Set the y-coordinate of the match.
  
  ///
  ///\see mvIMPACT::match::MatchData::getY
  ///\see mvIMPACT::match::MatchData::setX
  MatchData& setY(double y);




  /// \brief Retrieve the quality of the match.

  /// The match score is in range [0,100].
  /// \see mvIMPACT::match::MatchData::setScore
  double getScore() const;


  /// \brief Set the quality of the match.

  /// Valid values are in range [0,100].
  ///\see mvIMPACT::match::MatchData::getScore
  MatchData& setScore(
			///the new quality value
			/// \xmlonly
			/// <mvParamRange>
			/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
			/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
			/// <mvParamStep>0.1</mvParamStep>
			/// </mvParamRange>
			/// \endxmlonly			   	
			double score);



private:

	/// x-coordinate of the match.
	double m_x;
	/// y-coordinate of the match.
	double m_y;
	/// Quality of the match. In the range [0,100].
	double m_score;
};

/// \brief This object defines a search model for mvIMPACT Match based on an image.

/// You must specify a rectangular window from the image that is used as the
/// model. 
/// The rectangular window by the upper left corner, its width and its height.
/// The following image shows a sample model:
/// \image html images/mvi_match/patModelWindow.png
/// Every model has a center point. When searching for model occurances in an 
/// image the returned match coordinates specify the location of the model
/// center point.
///\see mvIMPACT::match::PatMatchResult
///\see mvIMPACT::match::MatchData::getX
///\see mvIMPACT::match::MatchData::getY
//-----------------------------------------------------------------------------
class PatModel
//-----------------------------------------------------------------------------
{
public:
  ///\brief Construct an empty PatModel object.
  PatModel ();

  /// \brief Construct a PatModel object using the entire image as model.
  PatModel (///the image to be used as the model
	    const Image& source);

  /// \brief Construct a PatModel object using a rectangular window from an image as the model.
  
  /// Parameters \a x and \a y specify the upper left corner of the rectangular area. The
  /// width and the height of the window is given with parameters \a width and \a height.
  PatModel 
    (
		/// The source image.
		const Image& source, 
		/// The x-coordinate of the model window left upper corner.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-%(width)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int x, 
		/// The y-coordinate of the model window left upper corner.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-%(height)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int y, 
		/// Width of the model window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-%(x)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int width, 
		/// Height of the model window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-%(y)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int height
		);
  
private:
	void init (const Image& image, int x, int y, int w, int h);
	

public:
	/// \brief Destructor.
	~PatModel();

	/// \brief Construct a PatModel object from \a src.
	PatModel(const PatModel& src);

#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	PatModel& operator= (const PatModel& src);
#endif // #ifndef WRAP_ANY

	
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	int getUseCount () const; 
	bool getIsUnique    () const; 
	mvIMPACT_C::PAT_BUFHANDLE getHandle   () const; 
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	/// Construct a model from a handle.
	PatModel(mvIMPACT_C::PAT_BUFHANDLE  handle);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	/// The buffer handle.
	mvIMPACT_C::PAT_BUFHANDLE handle_;
	/// Unassigned buffer constant.
	enum { unassigned = -1 };

private:
	/// Pointer to the reference counter.
	mutable int* counter_;

	/// Dispose the buffer. If the reference count drops to zero
	/// and the buffer was allocated it is freed.
	void dispose();
	
	void copy(const PatModel& src);
	


public: // Properties

	/// \brief Specify the number of model occurrences to find. 

	/// In most cases you 
	/// want one match, which causes mvIMPACT Match to find the best match in the 
	/// target image. But it is also possible to search for a certain number of matches 
	/// or to find all matches above the acceptance threshold.
	/// If you call this function with PatMatchMode::pmmFindAll all model occurance will
	/// be returned.
	///\see mvIMPACT::match::PatModel::getNumberOfMatches
	PatModel& setNumberOfMatches(
					///the desired number of matches 
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
					/// </mvParamRange>
					/// \endxmlonly			   	
				     int numberOfMatches
					 );
	

	/// \brief Retrieve the maximum number of model occurances to find.

	/// If \c pmmFindAll is returned, mvIMPACT Match returns all matches above the acceptance threshold.
	///\see mvIMPACT::match::PatModel::setNumberOfMatches
	int getNumberOfMatches () const;
	

	/// \brief Set the binning method to use.

	/// During the search phase images will be subsampled using a binning method. The method to use
	/// depends on the model content. In most cases the default method (bmMean) should work fine. 
	/// However there are situations, where another method gives better results. For example, if you  
	/// have a bright model with small dark spots, the preferred method would be bmMin.
	///\see mvIMPACT::match::PatModel::getBinningMethod
	PatModel& setBinningMode(
					/// the binning method to use.
					BinningMode bMode);

	/// \brief Retrieve the binning method.

	/// During the search phase images will be subsampled using a binning method. 
	///\see mvIMPACT::match::PatModel::setBinningMethod
	BinningMode getBinningMode() const;

	/// \brief Set the initial subsampling factor.

	/// The initial subsampling factor tells mvIMPACT Match, to take only every n'th pixel
	/// for the first search. If you have good quality images, you can try to set this
	/// factor to a higher value to save processing time. However the chance to fail 
	/// the model increases.
	///\see mvIMPACT::match::PatModel::getInitialSubsampling
	PatModel& setInitialSubsampling (
					///the new subsampling factor
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
					/// </mvParamRange>
					/// \endxmlonly			   	
					int initialSubsampling);
	

	/// \brief Retrieve the initial subsampling factor.

	/// The initial subsampling factor tells mvIMPACT Match, to take only every n'th pixel
	/// for the first search. 
	///\see mvIMPACT::match::PatModel::setInitialSubsampling
	int getInitialSubsampling () const;
	

	/// \brief Set minimum score used during initial search
	///
	/// The initial score will be applied by the search module during
	/// the initial search. The initial search is the phase, when the model and the target 
	/// image are subsampled with the initial subsampling factor. 
	/// The initial score must be specified as a percentage [0..100]. If during the initial
	/// search phase a match is less than this level, it is not considered to be a match. 
	/// Only match candidates that are above this level will be included in further processing 
	/// steps.
	PatModel& setInitialAcceptanceScore(
			/// the new initial acceptance score
			/// \xmlonly
			/// <mvParamRange>
	    	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	     	/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
	     	/// <mvParamStep>0.1</mvParamStep>
	        /// </mvParamRange>
 	        /// \endxmlonly
			double initialAcceptanceScore);


	/// \brief Get minimum score used during initial search
	///
	/// The initial score will be applied by the search module during
	/// the initial search. The initial search is the phase, when the model and the target 
	/// image are subsampled with the initial subsampling factor. 
	/// The initial score must be specified as a percentage [0..100]. If during the initial
	/// search phase a match is less than this level, it is not considered to be a match. 
	/// Only match candidates that are above this level will be included in further processing 
	/// steps.
	double getInitialAcceptanceScore() const;

	/// \brief Set minimum score used during actual search
	///
	/// The final score must be reached in order to be considered as a match.
	/// The final score is calculated using the correlation between the image and the model. 
	/// It is specified as a percentage in the range [0..100]. The quality of the image, 
	/// where the model is searched influences the match score. To decide what is a typical 
	/// match score for your application, you should experiment with it.
	PatModel& setFinalAcceptanceScore(
			///the new final score
			/// \xmlonly
			/// <mvParamRange>
	    	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	     	/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
	     	/// <mvParamStep>0.1</mvParamStep>
	        /// </mvParamRange>
 	        /// \endxmlonly
			double finalAcceptanceScore);
	

	/// \brief Get minimum score used during actual search
	///
	/// The final score must be reached in order to be considered as a match.
	/// The final score is calculated using the correlation between the image and the model. 
	/// It is specified as a percentage in the range [0..100]. The quality of the image, 
	/// where the model is searched influences the match score. To decide what is a typical 
	/// match score for your application, you should experiment with it.
	double getFinalAcceptanceScore() const;

	
	/// \brief This function sets the search region within the target image, where the mvIMPACT::match::findPatModel
	/// \brief function tries to find the model. 

	/// Setting a search region is useful, when the model
	/// to be found is expected in a certain area. The smaller the search region, the faster 
	/// the search will be. If your target buffer consists of multiple dimensions 
	/// (multiple bands, slices or frames), the search window will be applied to every dimension.
	/// Please make sure, that
	/// <ul>
	/// <li>the search region fits within the target buffer</li>
	/// <li>the search model fits within the search region</li>
	/// </ul>
	/// \see mvIMPACT::match::PatModel:getSearchWindow
	PatModel& setSearchWindow (///rectangle that defines the search area 
				   const Rectangle& searchWindow );
	

	/// \brief Retrieve the window, where mvIMPACT Match searches for model occurances.

	/// Returns the area where mvIMPACT::match::findPatModel searches for model occurances.
	/// If your target buffer consists of multiple dimensions 
	/// (multiple bands, slices or frames), the window returned here will be applied to every dimension.
	/// \see mvIMPACT::match::PatModel:setSearchWindow
	Rectangle getSearchWindow () const;
	

	/// \brief This function sets the center of the pattern matching model with respect to 
	/// the model's origin (top left corner). 

	/// By default, the x-center is equal to 
	/// (model width-1)/2 and the y-center is equal to (model height-1)/2. Both are 
	/// integer values. All positional coordinates returned from mvIMPACT::match::findPatModel
	/// are calculated with respect to the model center.
	///\see mvIMPACT::match::PatModel::getModelCenter
	///\see mvIMPACT::match::findPatModel
	PatModel& setModelCenter (///the new model center 
				  const Point& modelCenter );
	

	/// \brief Retrieve the current model center.

	/// By default, the x-center is equal to 
	/// (model width-1)/2 and the y-center is equal to (model height-1)/2. Both are 
	/// integer values. All positional coordinates returned from mvIMPACT::match::findPatModel
	/// are calculated with respect to the model center.
	///\see mvIMPACT::match::PatModel::getModelCenter
	///\see mvIMPACT::match::findPatModel
	Point getModelCenter () const;
	

	/// \brief Retrieve the rectangular window of the image that the model is based on.

	/// This window cannot be changed after the model is created. To specify a
	/// different window you must create a new model.
	///\see mvIMPACT::match::PatModel
	Rectangle getModelWindow () const;
	

	/// \brief Get use preset value flag.
	/// 
	/// If this flag is set to \c true, the pixels set with presetModelPixel are
	/// ignored during the matching process.
	///
	/// \see mvIMPACT::match::presetModelPixel
	bool getUsePresetValue () const;
	
	/// \brief Set use preset value flag.
	/// 
	/// If this flag is set to \c true, the pixels set with presetModelPixel are
	/// ignored during the matching process.
	///
	/// \see mvIMPACT::match::presetModelPixel
	PatModel& setUsePresetValue(bool usePresetValue);
};




/// \brief This class represents the results of a pattern matching run. 

/// If the image where the search was conducted consists of several bands,slices
/// or frames you can query the results for every combination of band,slice and 
/// frame.
/// Objects of this class are returned by mvIMPACT::match::findPatModel
///\see mvIMPACT::match::findPatModel
//-----------------------------------------------------------------------------
class PatMatchResult
//-----------------------------------------------------------------------------
{
public:
  
  ///\brief Create an empty PatMatchResult object.
  PatMatchResult ();
  
  ///\brief Destructor.
  ~PatMatchResult();
  
  ///\brief Construct a PatMatchResult object from \a src.
  PatMatchResult(///the othe object
		 const PatMatchResult& src);
  

#ifndef WRAP_ANY
  ///\brief Overwrite current object with \a src.
  PatMatchResult& operator= (///the other object
			     const PatMatchResult& src);
#endif // #ifndef WRAP_ANY

	

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	int getUseCount () const; 
	bool getIsUnique    () const; 
	mvIMPACT_C::PAT_BUFHANDLE getHandle   () const; 
	/// Construct a match result from a handle.
	PatMatchResult (mvIMPACT_C::PAT_BUFHANDLE handle);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS


protected:
	/// The buffer handle.
	mvIMPACT_C::PAT_BUFHANDLE handle_;
	/// Unassigned buffer constant.
	enum { unassigned = -1 };

private:
	/// Pointer to the reference counter.
	mutable int* counter_;

	/// Dispose the buffer. If the reference count drops to zero
	/// and the buffer was allocated it is freed.
	void dispose();
	
	void copy(const PatMatchResult& src);


public: // Properties

	/// \brief Retrieve the number of bands of this match result.
	
	/// A match result can have multiple bands, this is because mvIMPACT::match::findPatModel searches 
	/// in all bands of the traget image.
	///\see mvIMPACT::match::PatMatchResult::getNumberOfSlices
	///\see mvIMPACT::match::PatMatchResult::getNumberOfFrames
	int getNumberOfBands () const;


	/// \brief Retrieve the number of slices of this match result.

	/// A match result can have multiple slices, this is because mvIMPACT::match::findPatModel searches 
	/// in all slices of the traget image.
	///\see mvIMPACT::match::PatMatchResult::getNumberOfBands
	///\see mvIMPACT::match::PatMatchResult::getNumberOfFrames
	int getNumberOfSlices () const;


	/// \brief Retrieve the number of frames of this match result.

	/// A match result can have multiple frames, this is because mvIMPACT::match::findPatModel searches 
	/// in all frames of the traget image.
	///\see mvIMPACT::match::PatMatchResult::getNumberOfSlices
	///\see mvIMPACT::match::PatMatchResult::getNumberOfBands
	int getNumberOfFrames () const;
	

	/// \brief Retrieve the number of matches for a specific combination of \a band,\a slice and \a frame.

	/// If the image where the search was conducted consists of several bands,slices
	/// or frames you must specify the offset of the band,slice and frame you are 
	/// interested in.
	///\see mvIMPACT::match::PatMatchResult::getNumberOfSlices
	///\see mvIMPACT::match::PatMatchResult::getNumberOfBands
	///\see mvIMPACT::match::PatMatchResult::getNumberOfFrames
	int getNumberOfMatches (
				///the desired band
				/// \xmlonly
				/// <mvParamRange>
	    		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	     		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfBands-1"/></mvParamMax>
				/// </mvParamRange>
 				/// \endxmlonly			
				int band=0, 
				///the desired slice
				/// \xmlonly
				/// <mvParamRange>
	    		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	     		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfSlices-1"/></mvParamMax>
				/// </mvParamRange>
 				/// \endxmlonly			
				int slice=0, 
				///the desired frame
				/// \xmlonly
				/// <mvParamRange>
	    		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	     		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfFrames-1"/></mvParamMax>
				/// </mvParamRange>
 				/// \endxmlonly			
				int frame=0
				) const;
	

	/// \brief Retrieve the locations in the target image where a model occurance was found.

	/// If the image where the search was conducted consists of several bands,slices
	/// or frames you must specify the offset of the band,slice and frame you are 
	/// interested in.
	/// The matches are returned in a vector containing MatchData objects.
	/// The entries in this vector are sorted in decreasing order by score.
	///\see mvIMPACT::match::PatMatchResult::getNumberOfMatches
	///\see mvIMPACT::match::PatMatchResult::getNumberOfSlices
	///\see mvIMPACT::match::PatMatchResult::getNumberOfBands
	///\see mvIMPACT::match::PatMatchResult::getNumberOfFrames
	///\see mvIMPACT::match::MatchData
	std::vector<MatchData> getMatches (
				///the desired band 
				/// \xmlonly
				/// <mvParamRange>
	    		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	     		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfBands-1"/></mvParamMax>
				/// </mvParamRange>
 				/// \endxmlonly			
				int band=0, 
				///the desired slice
				/// \xmlonly
				/// <mvParamRange>
	    		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	     		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfSlices-1"/></mvParamMax>
				/// </mvParamRange>
 				/// \endxmlonly			
				int slice=0, 
				///the desired frame
				/// \xmlonly
				/// <mvParamRange>
	    		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	     		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfFrames-1"/></mvParamMax>
				/// </mvParamRange>
 				/// \endxmlonly			
				int frame=0) const;
	
};


/// \brief This function searches for one or more model occurrences in the target image.
///
/// It respects all current search parameters.
/// There are two important assumptions made by this function:
/// <ul>
/// <li>
/// The scaling of the image buffer from which the model was extracted and the
/// scaling of the target buffer must be the same.
/// </li>
/// <li>
/// The orientation of the target image has not been changed since the model 
/// was allocated.
/// </li>
/// </ul>
//-----------------------------------------------------------------------------
PatMatchResult findPatModel (///the model to search for
			     const PatModel& patModel, 
			     /// the image where we want to search for occurances of \a patModel
			     const Image& source );



/// \brief This function converts a model to an image.
///
///This function creates an image form a model. The image can be used by any other processing
///function.
//-----------------------------------------------------------------------------
Image modelToImage (///the model to convert
		    const PatModel& patModel);




/// \brief This function converts a model to an image.
///
///This function creates an image form a model. The image can be used by any other processing
///function.
///\note The mvIMPACT::Image object passed to this function  must have the same
///\note properties as the model concerning the height, width and data type.
//-----------------------------------------------------------------------------
void modelToImage 
(
	/// The destination image.
	Image& dest, 
	/// The source model.
	const PatModel& patModel
	);



/// \brief This function matches a model with an image  and writes
/// \brief the match scores into a destination image. 
///
///The match scores are scaled to 
/// the data type of the destination image. For a 8 bit image the value 255
/// means a 100% match, whereas 0 means that the image and the template are 
/// uncorrelated. Internally, the function uses a normalized grayscale correlation 
/// algorithm. Due to the normalization, correlation coefficients between -1
/// and 1 can occur. However, only those correlation coefficients are of interest,
/// which range between [0..1]. Values smaller than 0 are clipped to 0 and will 
/// therefore be represented in the destination buffer as 0.<br>
/// \note This operation can be very time consuming if the model and 
/// \note the search area becomes large.
Image computeMatchScores (///the model that is matched with image \a source
			  const PatModel& patModel, 
			  ///the image that is matched with model \a patModel.
			  const Image& source);


/// \brief This function matches a model with an image  and writes
/// \brief the match scores into a destination image. 

///The match scores are scaled to 
/// the data type of the destination image. For a 8 bit image the value 255
/// means a 100% match, whereas 0 means that the image and the template are 
/// uncorrelated. Internally, the function uses a normalized grayscale correlation 
/// algorithm. Due to the normalization, correlation coefficients between -1
/// and 1 can occur. However, only those correlation coefficients are of interest,
/// which range between [0..1]. Values smaller than 0 are clipped to 0 and will 
/// therefore be represented in the destination buffer as 0.<br>
/// \note This operation can be very time consuming if the model and 
/// \note the search area becomes large.
//-----------------------------------------------------------------------------
void computeMatchScores 
(
	/// The destination image. The results go here
	Image& dest, 
	/// The template model that is matched with image \a source
	const PatModel& patModel, 
	/// The source image that is matched with model \a patModel
	const Image& source
	);



/// \brief This function tries to allocate a model automatically based on different 
/// \brief criteria (see \a mode parameter). 

///It starts searching the model in the 
/// source buffer from left to right and from top to bottom. You can specify
/// the step width (parameter \a stepSizeX and \a stepSizeY) by which the rectangular
/// area is moved. This function could be used for example, to register images
/// without defining a model by 'hand'.
/// \see #BestModelMode
//-----------------------------------------------------------------------------
PatModel findBestModel
(
	/// The source image.
	const Image& source,
	/// The desired model width.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(source)s.width"/></mvParamMax>
	/// </mvParamRange>
 	/// \endxmlonly			
	int width,
	/// The desired model height.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(source)s.height"/></mvParamMax>
	/// </mvParamRange>
 	/// \endxmlonly			
	int height,
	/// The horizontal step size that is used to step through the source image.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(source)s.width"/></mvParamMax>
	/// </mvParamRange>
 	/// \endxmlonly			
	int stepSizeX,
	/// The vertical step size that is used to step through the source image.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(source)s.height"/></mvParamMax>
	/// </mvParamRange>
 	/// \endxmlonly			
	int stepSizeY,
	/// The rating technique that is used to select the best model.
	BestModelMode mode,
	/// If the source image consists of several bands this parameter allows you to
	/// select the desired band.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(source)s.bands-1"/></mvParamMax>
	/// </mvParamRange>
 	/// \endxmlonly			
	int band=0,
	/// If the source image consists of several slices this parameter allows you to
	/// select the desired slice.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(source)s.slices-1"/></mvParamMax>
	/// </mvParamRange>
 	/// \endxmlonly			
	int slice=0,
	/// If the source image consists of several frames this parameter allows you to
	/// select the desired frame.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(source)s.frames-1"/></mvParamMax>
	/// </mvParamRange>
 	/// \endxmlonly			
	int frame=0
	);



/// \brief Select pixels in the model, that are not considered in the correlation process.
///
///All pixel, that are marked as "don't care",
///are not included in the correlation process. This helps you to speed up your application.
///The main purpose of this feature is, to exclude background pixels that could vary from one
///image to another image.
///In this case, set all background pixel  to "don't
///care". The size of the model buffer determines the
///area in the source image  (starting at the specified offsets \a offsetX and \a offsetY), from which to extract preset values.
///The values in the source buffer are compared the condition values \a condValue1 and/or condValue2 (depending
/// on \a mode). If the condition is true as specified with the \a mode parameter, 
/// the corresponding pixel in the model buffer will be set to "don't care". 
/// The data type of the source buffer and the model buffer must be the same.
///
///\see #PresetCondition
///
 // \internal
 // The feature of using several values for the preset value is intentionally
 // not wrapped because it's only used very rarely.
void presetModelPixel (
				///the model to be modified
				PatModel& patModel,
				///the source image whose pixels are used as input for the applied condition
				/// specified with \a mode
				const Image& source,
				///x-offset in \a source where we start testing the condition (upper left corner)		       
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(source)s.width-%(patModel)s.getModelWindow().width"/></mvParamMax>
				/// </mvParamRange>
 				/// \endxmlonly			
				int offsetX,
				///y-offset in \a source where we start testing the condition (upper left corner)
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(source)s.height-%(patModel)s.getModelWindow().height"/></mvParamMax>
				/// </mvParamRange>
 				/// \endxmlonly			
				int offsetY,
				/// Specifies the first value which is used in the condition
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(conValue2)s-1"/></mvParamMax>
				/// </mvParamRange>
 				/// \endxmlonly			
				int condValue1,
				///Specifies the second value which is used in the condition
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamComplex Python="%(condValue1)s+1"/></mvParamMin>
				/// </mvParamRange>
 				/// \endxmlonly			
				int condValue2,
				///Specifies the condition how preset values are set. You can specify conditions which
				///use two limits or conditions which use one limit. 
				PresetCondition mode
				);


/// \brief Set a single pixel in the model to be not considered in the correlation process.
///
///All pixel, that are marked as "don't care",
///are not included in the correlation process. This helps you to speed up your application.
///The main purpose of this feature is, to exclude background pixels that could vary from one
///image to another image.
///
 // \internal
 // The feature of using several values for the preset value is intentionally
 // not wrapped because it's only used very rarely.
void presetModelPixel (///the model to be modified
		       PatModel& patModel,
		       /// coordinates of the point to be set to "don't care" state in the model
		       Point point
		       );


//////////////////////////////////////////////////////////////////////////
// vl_autoreg functions



/// \brief Number of models used for registration.
enum ModelCount
{	
	/// Use one model.
	mcOneModel = mvIMPACT_C::AR_ONE_MODEL,
	/// Use three models.
	mcThreeModels = mvIMPACT_C::AR_THREE_MODELS
};

/// \brief Interpolation mode used when registering image pairs and sequences
///
/// \see mvIMPACT::match::registerSequence
/// \see mvIMPACT::match::registerPair
enum RegisterInterpolationMode
{
	/// Nearest neighbor interpolation.
	rpmNearestNeighbor = mvIMPACT_C::AR_NEARESTNEIGHBOR,
	/// Bilinear interpolation.
	rpmBilinear = mvIMPACT_C::AR_BILINEAR,
};


/// \brief Fallback mode used when registering image sequences
///
/// This mode specifies what to do if registering an image in the
/// sequence fails.
///
/// \see mvIMPACT::match::registerSequence
enum RegisterFallbackMode
{
	/// Clear non-registered frames.
	rfmClear = mvIMPACT_C::AR_CLEAR,
	/// Interpolate non-registered frames.
	rfmInterpolate = mvIMPACT_C::AR_INTERPOLATE,
};

/// \brief Dimension mode used when registering image sequences
///
/// This specifies the dimension used for determining the sequence
/// to be registered.
/// 
/// In mode rmDefault uses the following algorithm to determine
/// the dimension used:
/// If these flags are not given, the function tries to figure it out on the
/// following priority scheme:
/// Image has multiple frames: Process frames dimension
/// Image has multiple slices only: Process slices dimension
/// Image has multiple bands only: Process bands dimension
/// Image has multiple slices and bands: Process slices dimension
///
/// \see mvIMPACT::match::registerSequence
enum RegisterDimensionMode
{
	/// Automatically determine dimension (see above).
	rdmDefault = mvIMPACT_C::AR_DEFAULT,
	/// Process band dimension.
	rdmBands = mvIMPACT_C::AR_BANDS,
	/// Process z slices dimension.
	rdmSlices = mvIMPACT_C::AR_SLICES,
	/// Process time frames dimension.
	rdmFrames = mvIMPACT_C::AR_FRAMES
};


///\brief Serach for a model in an image.

/// The function extracts a model from image \a sourceReference at pixel coordinates 
/// given by \a modelRect. It then tries to find the model in image \a sourceTarget.
/// The location where the model is found in the target image is stored in the 
/// mvIMPACT::match::MatchData object, that is returned.
/// The location can then be used to register the images \a sourceReference and \a sourceTarget.
/// If the model is not found an exception is raised.
/// Only works on images with one band, one slice and one frame!
/// \see mvIMPACT::match::matchPair
/// \see mvIMPACT::match::registerPair
MatchData searchModel 
(
	/// Image that is treated as reference image.
	const Image& sourceReference,
	/// Image that is matched against the reference image.
	const Image& sourceTarget,
	/// The location of the model on the reference image.
	const Rectangle& modelRect,
	/// Horizontal model center coordinate.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(modelRect)s.width-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	double modelCenterX,
	/// Vertical model center coordinate.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(modelRect)s.height-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	double modelCenterY,
	/// Minimum score to be considered a match.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
	/// <mvParamStep>0.1</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly			
	double minScore
	);




///\brief Match two images automatically.

/// Tries to automatically match a pair of images and returns a geometric 
/// transformation matrix. This function tries to automatically match a pair 
/// of images, without any user intervention. The function separates the 
/// image \a sourceReference into \a xTilesCount x \a yTilesCount tiles. 2 x 2 tiles is the
/// minimum of tiles that are allowed. Within each tile, the model of size 
/// \a width x \a height with the highest entropy is used as the model that is searched 
/// for in the image \a sourceTarget. You may want to use square models 
/// (64 x64 is a good start) to reach maximum speed. If you have 3 x 2 tiles, 
/// you'll get 6 models to match. For each of these models, the function tries to
/// search them in the \a sourceTarget image. All models that could be matched return an 
/// x,y coordinate pair. All coordinate pairs of matches that are higher in 
/// score than \a minScore are then used to calculate a geometric affine 
/// transformation matrix. If the minimum number of required matches could not 
/// be reached, the function throws an exception. The result of this function is the 
/// affine transformation matrix, that you can use to transform \a sourceTarget to be 
/// registered with \a sourceReference.
/// Only works on images with one band, one slice and one frame!
/// \see mvIMPACT::match::searchModel
/// \see mvIMPACT::match::registerPair
//-----------------------------------------------------------------------------
MatrixDouble matchPair
(
	/// Source image that is treated as reference image.
	const Image& sourceReference,
	/// Source image that is matched against the reference image.
	const Image& sourceTarget,
	/// Number of horizontal tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int xTilesCount,
	/// Number of vertical tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int yTilesCount,
	/// Minimum score to be considered a match.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
	/// <mvParamStep>0.1</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly			
	double minScore,
	/// Model width.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.width"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int width=64,
	/// Model height.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.height"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int height=64,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(rightBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int leftBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(bottomBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int topBorderHeight = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(leftBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int rightBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(topBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int bottomBorderHeight = 0,
	/// Register using one or three models.
	ModelCount modelCount = mcOneModel
	);


/// \brief Register two images automatically.
///
///  Tries to automatically match a pair of images and geometrically registers 
/// them. 
///This function tries to automatically register a pair 
/// of images, without any user intervention. The function separates the 
/// image \a sourceReference into \a xTilesCount x \a yTilesCount tiles. 2 x 2 tiles is the
/// minimum of tiles that are allowed. Within each tile, the model of size 
/// \a width x \a height with the highest entropy is used as the model that is searched 
/// for in the image \a sourceTarget. You may want to use square models 
/// (64 x64 is a good start) to reach maximum speed. If you have 3 x 2 tiles, 
/// you'll get 6 models to match. For each of these models, the function tries to
/// search them in the \a sourceTarget image. All models that could be matched return an 
/// x,y coordinate pair. All coordinate pairs of matches that are higher in 
/// score than \a minScore are then used to register the images.
/// If the minimum number of required matches could not 
/// be reached, the function throws an exception. The result of this function is the 
/// registered image.
/// Only works on images with one band, one slice and one frame!
/// \see mvIMPACT::match::searchModel
/// \see mvIMPACT::match::matchPair
//-----------------------------------------------------------------------------
Image registerPair
(
	/// Source image that is treated as reference image.
	const Image& sourceReference,
	// Source image that is matched against the reference image.
	const Image& sourceTarget,
	/// Number of horizontal tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int xTilesCount,
	/// Number of vertical tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int yTilesCount,
	/// Minimum score to be considered a match.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
	/// <mvParamStep>0.1</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly			
	double minScore,
	/// Model width.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.width"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int width = 64,
	/// Model height.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.height"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int height = 64,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(rightBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int leftBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(bottomBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int topBorderHeight = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(leftBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int rightBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(topBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int bottomBorderHeight = 0,
	/// Register using one or three models.
	ModelCount modelCount = mcOneModel,
	/// Specifies the interpolation mode. 
	RegisterInterpolationMode registerInterpolationMode = rpmBilinear
	);




/// \brief Register two images automatically.

///  Tries to automatically match a pair of images and geometrically registers 
/// them. 
/// This function tries to automatically register a pair 
/// of images, without any user intervention. The function separates the 
/// image \a sourceReference into \a xTiles x \a yTiles tiles. 2 x 2 tiles is the
/// minimum of tiles that are allowed. Within each tile, the model of size 
/// \a width x \a height with the highest entropy is used as the model that is searched 
/// for in the image \a sourceTarget. You may want to use square models 
/// (64 x64 is a good start) to reach maximum speed. If you have 3 x 2 tiles, 
/// you'll get 6 models to match. For each of these models, the function tries to
/// search them in the \a sourceTarget image. All models that could be matched return an 
/// x,y coordinate pair. All coordinate pairs of matches that are higher in 
/// score than \a minScore are then used to register the images.
/// If the minimum number of required matches could not 
/// be reached, the function throws an exception. The result of this function is stored 
/// in image \a dest.
/// Only works on images with one band, one slice and one frame!
/// \see mvIMPACT::match::searchModel
/// \see mvIMPACT::match::matchPair
//-----------------------------------------------------------------------------
void registerPair
(
	/// The destination image.
	Image& dest,
	/// Source image that is treated as reference image.
	const Image& sourceReference,
	// Source image that is matched against the reference image.
	const Image& sourceTarget,
	/// Number of horizontal tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int xTilesCount,
	/// Number of vertical tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int yTilesCount,
	/// Minimum score to be considered a match.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
	/// <mvParamStep>0.1</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly			
	double minScore,
	/// Model width.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.width"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int width = 64,
	/// Model height.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.height"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int height = 64,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(rightBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int leftBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(bottomBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int topBorderHeight = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(leftBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int rightBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(topBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int bottomBorderHeight = 0,
	/// Register using one or three models.
	ModelCount modelCount = mcOneModel,
	/// Specifies the interpolation mode. 
	RegisterInterpolationMode registerInterpolationMode = rpmBilinear
	);





///\brief Register a sequence of images automatically

/// Tries to automatically match a sequence of images and geometrically 
/// registers them. This function uses registerPair() internally
/// that is called for all image pairs in the sequence repeatedly.
/// The image returned has the same dimensions that \a sourceSequence.
/// Specifies the operation mode used for pixel interpolation. It can 
/// \bug How is this image sequence strutured? Slices? Frames?
/// \see mvIMPACT::match::registerPair
//-----------------------------------------------------------------------------
Image registerSequence
(
	/// Image containing a number of image pairs.
	const Image& sourceSequence,
	/// Number of horizontal tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int xTilesCount,
	/// Number of vertical tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int yTilesCount,
	/// Minimum score to be considered a match.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
	/// <mvParamStep>0.1</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly			
	double minScore,
	/// Model width.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.width"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int width = 64,
	/// Model height.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.height"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int height = 64,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(rightBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int leftBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(bottomBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int topBorderHeight = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(leftBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int rightBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(topBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int bottomBorderHeight = 0,
	/// Register using one or three models.
	ModelCount modelCount = mcOneModel,
	/// Specifies the interpolation mode. 
	RegisterInterpolationMode registerInterpolationMode = rpmBilinear,
	/// Specifies the fallback mode
	RegisterFallbackMode registerFallbackMode = rfmClear,
	/// Specifies the dimension to use
	RegisterDimensionMode registerDimensionMode = rdmDefault
	);


///\brief Register a sequence of images automatically

/// Tries to automatically match a sequence of images and geometrically 
/// registers them. This function uses registerPair() internally
/// that is called for all image pairs in the sequence repeatedly.
/// The image returned has the same dimensions that \a sourceSequence.
/// Specifies the operation mode used for pixel interpolation. It can 
/// \bug How is this image sequence strutured? Slices? Frames?
/// \see mvIMPACT::match::registerPair
//-----------------------------------------------------------------------------
void registerSequence
(
	/// The destination image.
	Image& dest,
	/// Image containing a number of image pairs.
	const Image& sourceSequence,
	/// Number of horizontal tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int xTilesCount,
	/// Number of vertical tiles.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="2"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int yTilesCount,
	/// Minimum score to be considered a match.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
	/// <mvParamStep>0.1</mvParamStep>
	/// </mvParamRange>
	/// \endxmlonly			
	double minScore,
	/// Model width.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.width"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int width = 64,
	/// Model height.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceReference)s.height"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int height = 64,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(rightBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int leftBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(bottomBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int topBorderHeight = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.width-%(leftBorderWidth)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int rightBorderWidth = 0,
	/// Specifies a border that is not used in the pattern matching process.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="%(sourceTarget)s.height-%(topBorderHeight)s"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			
	int bottomBorderHeight = 0,
	/// Register using one or three models.
	ModelCount modelCount = mcOneModel,
	/// Specifies the interpolation mode. 
	RegisterInterpolationMode registerInterpolationMode = rpmBilinear,
	/// Specifies the fallback mode
	RegisterFallbackMode registerFallbackMode = rfmClear,
	/// Specifies the dimension to use
	RegisterDimensionMode registerDimensionMode = rdmDefault
	);

	/// \brief Return version number from vl_pat module.

	/// This function returns a version string. The following format is used: 
	/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
	/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
	/// Example: 6.3.1.4529
	//-----------------------------------------------------------------------------
	std::string getVersionMatch();

	} // namespace match
} // namespace mvIMPACT

#include "mvIMPACT_match_impl.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::match;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

#endif // MVIMPACT_MATCH_H_

