#ifndef MVIMPACT_MATCH_IMPL_H_
#define MVIMPACT_MATCH_IMPL_H_


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	namespace match {



inline MatchData::MatchData (double x, double y, double score)
		: m_x(x), m_y(y), m_score(score)
	{ }

inline double MatchData::getX() const
{
	return m_x;
}

inline MatchData& MatchData::setX(double x)
{
	m_x = x;
	return *this;
}

inline double MatchData::getY() const
{
	return m_y;
}

inline MatchData& MatchData::setY(double y)
{
	m_y = y;
	return *this;
}

inline double MatchData::getScore() const
{
	return m_score;
}

inline MatchData& MatchData::setScore(double score)
{
	m_score = score;
	return *this;
}


inline PatModel::PatModel ()
		: handle_(unassigned), counter_(new int(1))
	{ }

inline PatModel::PatModel (const Image& image)
		: handle_(unassigned), counter_(new int(1))
	{
		init (image, 0, 0, image.getWidth(),image.getHeight());
	}

inline PatModel::PatModel 
	(
		const Image& image, 
		int x, 
		int y, 
		int width, 
		int height
	)
		: handle_(unassigned), counter_(new int(1))
	{
		init (image, x, y, width, height);
	}

inline void PatModel::init (const Image& image, int x, int y, int w, int h)
	{
		if (!mvIMPACT_C::PAT_BufAlloc (
				&handle_,
				image.getHandle(),
				x,y,w,h,
				mvIMPACT_C::PAT_MODEL))
			throwImpactError();
	}

inline PatModel::~PatModel()
	{
		// a destructor must never throw
		try { 
			dispose(); 
		} catch(...) {
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}	
	}

inline PatModel::PatModel(const PatModel& m)
	{
		copy(m);
	}

inline PatModel& PatModel::operator= (const PatModel& rhs)
	{
		if (this != &rhs)
		{
			dispose();
			copy(rhs);
		}
		return *this;
	}

inline int PatModel::getUseCount () const { return *counter_; }

inline bool PatModel::getIsUnique    () const { return 1 == *counter_; }

inline mvIMPACT_C::PAT_BUFHANDLE PatModel::getHandle   () const { return handle_; }

inline void PatModel::dispose()
	{
		if (0 == --*counter_)
		{
			delete counter_;
			counter_ = 0;
		
			if (unassigned != handle_)
			{
				if (!mvIMPACT_C::PAT_BufFree(handle_))
					throwImpactError();
				handle_ = unassigned;
			}
		}
	}

inline void PatModel::copy(const PatModel& src)
{
	counter_ = src.counter_;
	handle_ = src.handle_;
	
	(*counter_)++;
}


inline PatModel::PatModel(mvIMPACT_C::PAT_BUFHANDLE handle)
		: handle_(handle), counter_(new int(1))
	{
	}

inline PatModel& PatModel::setNumberOfMatches( int number )
	{
		if(!mvIMPACT_C::PAT_SetControl(getHandle(), mvIMPACT_C::PAT_NB_MATCHES, (void*)& number))
		{ throwImpactError(); }
		return *this;
	}

inline int PatModel::getNumberOfMatches () const
	{
		mvIMPACT_C::IPL_ULONG iplResult;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_NB_MATCHES,(void*)&iplResult))
		{ throwImpactError(); }

		return(int)iplResult;
	}

inline PatModel& PatModel::setInitialSubsampling ( int spacing )
	{
		double d = spacing;
		if(!mvIMPACT_C::PAT_SetControl(getHandle(), mvIMPACT_C::PAT_FIRST_LEVEL, (void*)& d))
		{ throwImpactError(); }
		return *this;
	}

inline int PatModel::getInitialSubsampling () const
	{
		mvIMPACT_C::IPL_DOUBLE iplResult;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_FIRST_LEVEL,(void*)&iplResult))
		{ throwImpactError(); }

		return(int)iplResult;
	}

inline PatModel& PatModel::setBinningMode ( BinningMode bMode )
	{
		mvIMPACT_C::IPL_LONG bmode = bMode;
		if(!mvIMPACT_C::PAT_SetControl(getHandle(), mvIMPACT_C::PAT_BINNING_METHOD, (void*)& bmode))
		{ throwImpactError(); }
		return *this;
	}

inline BinningMode PatModel::getBinningMode () const
	{
		mvIMPACT_C::IPL_LONG bmode = 0;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_BINNING_METHOD,(void*)&bmode))
		{ throwImpactError(); }

		return(BinningMode)bmode;
	}


inline PatModel& PatModel::setInitialAcceptanceScore(double initialAcceptanceScore)
{
	if(!mvIMPACT_C::PAT_SetControl(
		getHandle(), 
		mvIMPACT_C::PAT_CERTAINTY_LEVEL, 
		(void*)&  initialAcceptanceScore))
	{ throwImpactError(); }
	return *this;
}

inline double PatModel::getInitialAcceptanceScore() const
{
		mvIMPACT_C::IPL_DOUBLE iplResult;
		if (!mvIMPACT_C::PAT_BufInquire(
				getHandle(),
				mvIMPACT_C::PAT_CERTAINTY_LEVEL,
				(void*)&iplResult))
		{ throwImpactError(); }
			
		return iplResult;
}

inline PatModel& PatModel::setFinalAcceptanceScore(double finalAcceptanceScore)
{
		if(!mvIMPACT_C::PAT_SetControl(getHandle(), mvIMPACT_C::PAT_ACCEPTANCE_LEVEL, (void*)& finalAcceptanceScore))
		{ throwImpactError(); }
		return *this;
}	

inline double PatModel::getFinalAcceptanceScore() const
{
		mvIMPACT_C::IPL_DOUBLE iplResult;
		if (!mvIMPACT_C::PAT_BufInquire(
				getHandle(),
				mvIMPACT_C::PAT_ACCEPTANCE_LEVEL,
				(void*)&iplResult))
		{ throwImpactError(); }
			
		return iplResult;
}


inline PatModel& PatModel::setSearchWindow ( const Rectangle& searchRegion )
	{
		int x,y,w,h;

		x = searchRegion.getLeft();
		y = searchRegion.getTop();
		w = searchRegion.getWidth();
		h = searchRegion.getHeight();

		if (!mvIMPACT_C::PAT_SetWindow(
				getHandle(),x,y,w,h))
			throwImpactError();
		return *this;
	}

inline Rectangle PatModel::getSearchWindow () const
	{
		int x,y,w,h;

		mvIMPACT_C::IPL_LONG iplx;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_XPOS_SEARCH_REGION,(void*)&iplx))
		{ throwImpactError(); }
		x = (int)iplx;
		mvIMPACT_C::IPL_LONG iply;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_YPOS_SEARCH_REGION,(void*)&iply))
		{ throwImpactError(); }
		y = (int)iply;
		mvIMPACT_C::IPL_LONG iplw;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_XSIZE_SEARCH_REGION,(void*)&iplw))
		{ throwImpactError(); }
		w = (int)iplw;
		mvIMPACT_C::IPL_LONG iplh;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_YSIZE_SEARCH_REGION,(void*)&iplh))
		{ throwImpactError(); }
		h = (int)iplh;
		
		return Rectangle (x,y,x+w-1,y+h-1);
	}

inline PatModel& PatModel::setModelCenter ( const Point& center )
	{
		if (!mvIMPACT_C::PAT_SetModelCenter(
			getHandle(), center.getX(), center.getY())
			)
		{ throwImpactError(); }
		return *this;
	}

inline Point PatModel::getModelCenter () const
	{
		Point result;
		
		mvIMPACT_C::IPL_DOUBLE iplResult;

		if (!mvIMPACT_C::PAT_BufInquire(
				getHandle(),
				mvIMPACT_C::PAT_XCEN_MODEL,
				(void*)&iplResult))
		{ throwImpactError(); }
		result.setX((int)iplResult);

		if (!mvIMPACT_C::PAT_BufInquire(
				getHandle(),
				mvIMPACT_C::PAT_YCEN_MODEL,
				(void*)&iplResult))
		{ throwImpactError(); }
		result.setY((int)iplResult);

		return result;
	}

inline Rectangle PatModel::getModelWindow () const
	{
		int x; mvIMPACT_C::IPL_LONG iplx;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_XOFF_MODEL,(void*)&iplx))
		{ throwImpactError(); }
		x = (int)iplx;
		int y; mvIMPACT_C::IPL_LONG iply;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_YOFF_MODEL,(void*)&iply))
		{ throwImpactError(); }
		y = (int)iply;
		int w; mvIMPACT_C::IPL_LONG iplw;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_XSIZE_MODEL,(void*)&iplw))
		{ throwImpactError(); }
		w = (int)iplw;
		int h; mvIMPACT_C::IPL_LONG iplh;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_YSIZE_MODEL,(void*)&iplh))
		{ throwImpactError(); }
		h = (int)iplh;

		return Rectangle(x,y,x+w-1,y+h-1);
	}

inline bool PatModel::getUsePresetValue () const
	{
		mvIMPACT_C::IPL_BOOL iplResult;
		if (!mvIMPACT_C::PAT_BufInquire(
				getHandle(),
				mvIMPACT_C::PAT_USE_PRESET_VALUE,
				(void*)&iplResult))
		{ throwImpactError(); }
		
		return (0 != iplResult);
	}

inline PatModel& PatModel::setUsePresetValue(bool val)
{
	mvIMPACT_C::IPL_BOOL iplVal = val ? IPL_TRUE : IPL_FALSE;
	if(!mvIMPACT_C::PAT_SetControl(getHandle(), mvIMPACT_C::PAT_USE_PRESET_VALUE, (void*)&iplVal))
	{ throwImpactError(); }
	return *this;
}


inline PatMatchResult::PatMatchResult ()
	: handle_(unassigned), counter_(new int(1))
	{ }

inline PatMatchResult::~PatMatchResult()
	{
		// a destructor must never throw
		try { 
			dispose(); 
		} catch(...) {
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}	
	}

inline PatMatchResult::PatMatchResult(const PatMatchResult& m)
	{
		copy(m);
	}

inline PatMatchResult& PatMatchResult::operator= (const PatMatchResult& rhs)
	{
		if (this != &rhs)
		{
			dispose();
			copy(rhs);
		}
		return *this;
	}

	inline int PatMatchResult::getUseCount () const { return *counter_; }

inline bool PatMatchResult::getIsUnique    () const { return 1 == *counter_; }

inline mvIMPACT_C::PAT_BUFHANDLE PatMatchResult::getHandle   () const { return handle_; }




inline void PatMatchResult::dispose()
	{
		if (0 == --*counter_)
		{
			delete counter_;
			counter_ = 0;
		
			if (unassigned != handle_)
			{
				if (!mvIMPACT_C::PAT_BufFree(handle_))
					throwImpactError();
				handle_ = unassigned;
			}
		}
	}

inline void PatMatchResult::copy(const PatMatchResult& src)
{
	handle_ = src.handle_;
	counter_ = src.counter_;
	
	(*counter_)++;
}

inline PatMatchResult::PatMatchResult (mvIMPACT_C::PAT_BUFHANDLE handle)
		: handle_(handle), counter_(new int(1))
	{
	}

inline int PatMatchResult::getNumberOfBands () const
	{
		mvIMPACT_C::IPL_LONG iplResult;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_NB_BANDS,(void*)&iplResult))
		{ throwImpactError(); }

		return(int)iplResult;
	}

inline int PatMatchResult::getNumberOfSlices () const
	{
		mvIMPACT_C::IPL_LONG iplResult;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_NB_ZSLICES,(void*)&iplResult))
		{ throwImpactError(); }

		return(int)iplResult;
	}

inline int PatMatchResult::getNumberOfFrames () const
	{
		mvIMPACT_C::IPL_LONG iplResult;
		if(!mvIMPACT_C::PAT_BufInquire(getHandle(),mvIMPACT_C::PAT_NB_TSLICES,(void*)&iplResult))
		{ throwImpactError(); }

		return(int)iplResult;
	}

inline int PatMatchResult::getNumberOfMatches 
	(
		int bandOffset, 
		int sliceOffset, 
		int frameOffset
	) const
	{
		mvIMPACT_C::IPL_ULONG n = 0;
		if (!mvIMPACT_C::PAT_GetResultBufData(
				getHandle(),
				bandOffset,sliceOffset,frameOffset,
				0,
				mvIMPACT_C::PAT_NB_MATCHES_FOUND,
				&n))
			throwImpactError();
		return (int)n;
	}

inline std::vector<MatchData> PatMatchResult::getMatches (int bandOffset, int sliceOffset, int frameOffset) const
	{
		std::vector<MatchData> vResult;

		int n = getNumberOfMatches(bandOffset,sliceOffset,frameOffset);
		
		// Query the details of all matches and return them 
		// in a vector.
		double x,y,score;
		for (int i=0; i<n; ++i)
		{
			if (!mvIMPACT_C::PAT_GetResultBufData(
					getHandle(),
					bandOffset,sliceOffset,frameOffset,
					i,
					mvIMPACT_C::PAT_XPOS,
					&x))
				throwImpactError();

			if (!mvIMPACT_C::PAT_GetResultBufData(
					getHandle(),
					bandOffset,sliceOffset,frameOffset,
					i,
					mvIMPACT_C::PAT_YPOS,
					&y))
				throwImpactError();

			if (!mvIMPACT_C::PAT_GetResultBufData(
					getHandle(),
					bandOffset,sliceOffset,frameOffset,
					i,
					mvIMPACT_C::PAT_SCORE,
					&score))
				throwImpactError();

			vResult.push_back(MatchData(x,y,score));
		}

		return vResult;
	}


inline PatMatchResult findPatModel (const PatModel& model, const Image& S )
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::PAT_BUFHANDLE result = -1;
	if (!mvIMPACT_C::PAT_SearchModel(S.getHandle(), model.getHandle(), &result))
		throwImpactError();
	return PatMatchResult(result);
}


inline Image modelToImage (const PatModel& model)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = -1;
	if (!mvIMPACT_C::PAT_CopyModel (model.getHandle(), &hD))
		throwImpactError();
	return Image::imageFactory(hD);
}


inline void modelToImage 
(
	Image& D, 
	const PatModel& model
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::PAT_CopyModel (model.getHandle(), &hD))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}


inline Image computeMatchScores (const PatModel& model, const Image& S)
//-----------------------------------------------------------------------------
{
	Image D = createCompatibleImage(S);
	if (!mvIMPACT_C::PAT_MatchTemplate(S.getHandle(),D.getHandle(),model.getHandle()))
		throwImpactError();
	return D;
}


inline void computeMatchScores 
(
	Image& D, 
	const PatModel& model, 
	const Image& S
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::PAT_MatchTemplate(S.getHandle(),D.getHandle(),model.getHandle()))
		throwImpactError();
}


inline PatModel findBestModel
(
	const Image& S,
	int width,
	int height,
	int horizStep,
	int vertStep,
	BestModelMode mode,
	int bandOffset,
	int sliceOffset,
	int frameOffset
)
//-----------------------------------------------------------------------------
{
	// 1. Create a child buffer for S that contains only the requested band,slice,frame.
	Image child(S,0,0,bandOffset,sliceOffset,frameOffset,S.getWidth(),S.getHeight(),1,1,1);

	// 2. Search best model in the child image.
	mvIMPACT_C::PAT_MODEL_RESULT result;
	if (!mvIMPACT_C::PAT_FindBestModel(
			&result,
			child.getHandle(),
			width,
			height,
			horizStep,
			vertStep,
			mvIMPACT_C::PAT_BESTMODEL(mode)))
		throwImpactError();

	// 3. Create a PatModel object and return it.
	return PatModel(child,result.XPos,result.YPos,width,height);
}


inline void presetModelPixel 
(
	PatModel& model,
	const Image& S,
	int xOff,
	int yOff,
	int condValue1,
	int condValue2,
	PresetCondition mode
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::PAT_PresetModelPixel(
			model.getHandle(),
			S.getHandle(),
			xOff,
			yOff,
			1, 
			condValue1,
			condValue2,
			mvIMPACT_C::PAT_PRESET_CONDITION(mode)))
		throwImpactError();
}

inline void presetModelPixel (
		       PatModel& model,
		       Point point
		       )
{
	if (!mvIMPACT_C::PAT_PresetModelPixel(
			model.getHandle(),
			PAT_DONT_CARE,
			point.getX(),
			point.getY(),
			1, 
			0, // ignored, as source buffer is set to PAT_DONT_CARE
			0, // ignored, as source buffer is set to PAT_DONT_CARE
			mvIMPACT_C::PAT_PRESET_CONDITION(pcLess)))
				// ignored, as source buffer is set to PAT_DONT_CARE
	{ throwImpactError(); }
}





inline MatchData searchModel 
(
	const Image& reference,
	const Image& target,
	const Rectangle& modelArea,
	double modelCenterX,
	double modelCenterY,
	double minScore
)
//-----------------------------------------------------------------------------
{
	MatchData match(0.0,0.0,0.0);

	mvIMPACT_C::IPL_RECT iplModelArea = modelArea;

	double x = 0.;
	double y = 0.;
	double score = 0.;
	if (!mvIMPACT_C::AR_SearchModel(
			reference.getHandle(),
			target.getHandle(),
			&iplModelArea,
			modelCenterX,
			modelCenterY,
			minScore,
			&score,
			&x,
			&y))
		throwImpactError();

	match.setX(x); match.setY(y); match.setScore(score);

	// Check if the returned score is higher than minScore. Otherwise there is no
	// match and an exception is raised.
	if (match.getScore() < minScore)
	{
		throwImpactException("Model not found");
	}

	return match;
}





inline MatrixDouble matchPair
(
	const Image& reference,
	const Image& target,
	int xTiles,
	int yTiles,
	double minScore,
	int width,
	int height,
	int leftBorder ,
	int topBorder ,
	int rightBorder ,
	int bottomBorder ,
	ModelCount models 
)
//-----------------------------------------------------------------------------
{
	MatrixDouble result(3,3);
	
	if (!mvIMPACT_C::AR_MatchPair(
			reference.getHandle(),
			target.getHandle(),
			leftBorder,
			topBorder,
			rightBorder,
			bottomBorder,
			xTiles,
			yTiles,
			mvIMPACT_C::AR_MODEL_COUNT(models),
			width,
			height,
			minScore,
			result.data()))
		throwImpactError();
	
	return result;
}





inline Image registerPair
(
	const Image& reference,
	// Source image that is matched against the reference image.
	const Image& target,
	int xTiles,
	int yTiles,
	double minScore,
	int width ,
	int height ,
	int leftBorder ,
	int topBorder ,
	int rightBorder ,
	int bottomBorder ,
	ModelCount models ,
	RegisterInterpolationMode interpolationMode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = -1;

	if (!mvIMPACT_C::AR_RegisterPair(
			reference.getHandle(),
			target.getHandle(),
			&hD,
			leftBorder,
			topBorder,
			rightBorder,
			bottomBorder,
			xTiles,
			yTiles,
			mvIMPACT_C::AR_MODEL_COUNT(models),
			width,
			height,
			minScore,
			mvIMPACT_C::AR_REGISTER_MODE(interpolationMode)))
		throwImpactError();

	return Image::imageFactory(hD);
}

inline void registerPair
(
	Image& D,
	const Image& reference,
	const Image& target,
	int xTiles,
	int yTiles,
	double minScore,
	int width,
	int height,
	int leftBorder,
	int topBorder,
	int rightBorder,
	int bottomBorder,
	ModelCount models,
	RegisterInterpolationMode interpolationMode)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::AR_RegisterPair(
			reference.getHandle(),
			target.getHandle(),
			&hD,
			leftBorder,
			topBorder,
			rightBorder,
			bottomBorder,
			xTiles,
			yTiles,
			mvIMPACT_C::AR_MODEL_COUNT(models),
			width,
			height,
			minScore,
			mvIMPACT_C::AR_REGISTER_MODE(interpolationMode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}


inline Image registerSequence
(
	const Image& sequence,
	int xTiles,
	int yTiles,
	double minScore,
	int width ,
	int height ,
	int leftBorder ,
	int topBorder ,
	int rightBorder ,
	int bottomBorder ,
	ModelCount models ,
	RegisterInterpolationMode interpolationMode,
	RegisterFallbackMode fallbackMode,
	RegisterDimensionMode dimensionMode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = -1;

	if (!mvIMPACT_C::AR_RegisterSequence(
			sequence.getHandle(),
			&hD,
			leftBorder,
			topBorder,
			rightBorder,
			bottomBorder,
			xTiles,
			yTiles,
			mvIMPACT_C::AR_MODEL_COUNT(models),
			width,
			height,
			minScore,
			(mvIMPACT_C::AR_REGISTER_MODE)((int)interpolationMode | (int)fallbackMode | (int)dimensionMode )))
		throwImpactError();

	return Image::imageFactory(hD);
}


inline void registerSequence
(
	Image& D,
	const Image& sequence,
	int xTiles,
	int yTiles,
	double minScore,
	int width,
	int height,
	int leftBorder,
	int topBorder,
	int rightBorder,
	int bottomBorder,
	ModelCount models,
	RegisterInterpolationMode interpolationMode,
	RegisterFallbackMode fallbackMode,
	RegisterDimensionMode dimensionMode)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::AR_RegisterSequence(
			sequence.getHandle(),
			&hD,
			leftBorder,
			topBorder,
			rightBorder,
			bottomBorder,
			xTiles,
			yTiles,
			mvIMPACT_C::AR_MODEL_COUNT(models),
			width,
			height,
			minScore,
			(mvIMPACT_C::AR_REGISTER_MODE)((int)interpolationMode | (int)fallbackMode | (int)dimensionMode )))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline std::string getVersionMatch()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::PAT_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}
	} // namespace match
} // namespace mvIMPACT
#endif // #ifndef WRAP_ANY
#endif// DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_MATCH_IMPL_H_
