#ifndef MVIMPACT_MEASURE_IMPL_H_
#define MVIMPACT_MEASURE_IMPL_H_


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	namespace measure {



inline MeasureBase::~MeasureBase()
	{
		// a destructor must never throw
		try { 
			dispose(); 
		} catch(...) {
			mvIMPACT_C::IPL_Trace(" caught exception in destructor.");
		}	
	}

inline MeasureBase::MeasureBase()
		: handle_(unassigned), counter_(new int(1))
	{
		if (!mvIMPACT_C::GAU_BufAlloc (&handle_))
			throwImpactError();
	}

inline MeasureBase::MeasureBase(const MeasureBase& m)
{
	copy(m);	
}

inline MeasureBase& MeasureBase::operator= (const MeasureBase& rhs)
{
	if (this != &rhs)
	{
		dispose();
		copy(rhs);
	}
	return *this;
}

inline int MeasureBase::getUseCount() const { return *counter_; }

inline bool MeasureBase::getIsUnique   () const { return 1 == *counter_; }

inline mvIMPACT_C::GAU_BUFHANDLE MeasureBase::getHandle  () const { return handle_; }


inline void MeasureBase::dispose()
	{
		if (0 == --*counter_)
		{
			delete counter_;
			counter_ = 0;
		
			if (unassigned != handle_)
			{
				if (!mvIMPACT_C::GAU_BufFree(handle_))
					throwImpactError();
				handle_ = unassigned;
			}
		}
	}

inline void MeasureBase::copy(const MeasureBase& src)
{
	handle_ = src.handle_;
	counter_ = src.counter_;
	
	(*counter_)++;
}

inline MeasureBase& MeasureBase::setCoordinateSystem 
	(
		double originX, 
		double originY, 
		double scaleX, 
		double scaleY, 
		double angle
	)
	{
		if (!mvIMPACT_C::GAU_SetCrdSystem(getHandle(),originX,originY,scaleX,scaleY,angle))
			throwImpactError();
		return *this;
	}


inline MeasureBase& MeasureBase::setCoordinateSystem(Position pos)
{
	if (!mvIMPACT_C::GAU_SetCrdSystem(
		getHandle(),
		pos.getX(),
		pos.getY(),
		pos.getScaleX(),
		pos.getScaleY(),
		pos.getAngle()))
	{ throwImpactError(); }
	return *this;
}

inline void MeasureBase::getCoordinateSystem
	(
		double& originX, 
		double& originY, 
		double& scaleX, 
		double& scaleY, 
		double& angle
	) const
	{
		if (!mvIMPACT_C::GAU_GetCrdSystem(getHandle(),&originX,&originY,&scaleX,&scaleY,&angle))
			throwImpactError();
	}

inline Position MeasureBase::getCoordinateSystem() const
{
	double originX;
	double originY; 
	double scaleX;
	double scaleY; 
	double angle;

	if (!mvIMPACT_C::GAU_GetCrdSystem(
		getHandle(),
		&originX,
		&originY,
		&scaleX,
		&scaleY,
		&angle))
	{ throwImpactError(); }
	
	return Position(originX, originY, scaleX, scaleY, angle);
}

inline MeasureEdgesBase::MeasureEdgesBase()
		: MeasureBase()
	{
	}

inline MeasureEdgesBase::MeasureEdgesBase(const MeasureEdgesBase& m)
		: MeasureBase()
	{
		*this = m;
	}

inline MeasureEdgesBase& MeasureEdgesBase::operator= (const MeasureEdgesBase& rhs)
	{
		MeasureBase::operator= (rhs);
		return *this;
	}

inline MeasureEdgesBase& MeasureEdgesBase::setMinScore (double score)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_SCORE, (void*)& score))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgesBase::getMinScore() const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_SCORE, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgesBase& MeasureEdgesBase::setInterpolationMode (InterpolationMode mode)
	{
		mvIMPACT_C::GAU_INTERPOLATION_MODE iplMode(static_cast<mvIMPACT_C::GAU_INTERPOLATION_MODE>(mode));
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_INT_MODE, (void*)& iplMode))
		{ throwImpactError(); }
		return *this;
	}

inline InterpolationMode MeasureEdgesBase::getInterpolationMode () const
	{
		mvIMPACT_C::GAU_INTERPOLATION_MODE result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_INT_MODE, (void*)&result))
		{ throwImpactError(); }
		return InterpolationMode(result);
	}

inline MeasureEdgesBase& MeasureEdgesBase::setAutomaticContrast (bool automatic)
	{
		 mvIMPACT_C::IPL_BOOL iplAutomatic = automatic ? IPL_TRUE : IPL_FALSE;
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_CONTRAST_AUTO, (void*)& iplAutomatic))
		{ throwImpactError(); }
		return *this;
	}

inline bool MeasureEdgesBase::getAutomaticContrast () const
	{
		mvIMPACT_C::IPL_BOOL result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_CONTRAST_AUTO, (void*)&result))
		{ throwImpactError(); }
		
		return (0 != result);
	}

inline MeasureEdgesBase& MeasureEdgesBase::setTypicalContrast (double contrast)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_CONTRAST_TYPICAL, (void*)& contrast))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgesBase::getTypicalContrast () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_CONTRAST_TYPICAL, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgesBase& MeasureEdgesBase::setAutomaticSteepness (bool automatic)
	{
		 mvIMPACT_C::IPL_BOOL iplAutomatic = automatic ? IPL_TRUE : IPL_FALSE;
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_STEEPNESS_AUTO, (void*)& iplAutomatic))
		{ throwImpactError(); }
		return *this;
	}

inline bool MeasureEdgesBase::getAutomaticSteepness () const
	{
		mvIMPACT_C::IPL_BOOL result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_STEEPNESS_AUTO, (void*)&result))
		{ throwImpactError(); }
		
		return (0 != result);
	}

inline MeasureEdgesBase& MeasureEdgesBase::setTypicalSteepness (double steepness)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_STEEPNESS_TYPICAL, (void*)& steepness))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgesBase::getTypicalSteepness () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_STEEPNESS_TYPICAL, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgesBase& MeasureEdgesBase::setContrastWeight (double weight)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_CONTRAST_WEIGHT, (void*)& weight))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgesBase::getContrastWeight () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_CONTRAST_WEIGHT, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgesBase& MeasureEdgesBase::setSteepnessWeight (double weight)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_STEEPNESS_WEIGHT, (void*)& weight))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgesBase::getSteepnessWeight () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_STEEPNESS_WEIGHT, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgesBase& MeasureEdgesBase::setPolarityWeight (double weight)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_POLARITY_WEIGHT, (void*)& weight))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgesBase::getPolarityWeight () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_POLARITY_WEIGHT, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgesBase& MeasureEdgesBase::setBestMatchOnly (bool onlybest)
	{
		 mvIMPACT_C::IPL_BOOL iplOnlyBest = onlybest ? IPL_TRUE : IPL_FALSE;
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_BEST_MATCH_ONLY, (void*)& iplOnlyBest))
		{ throwImpactError(); }
		return *this;
	}

inline bool MeasureEdgesBase::getBestMatchOnly () const
	{
		mvIMPACT_C::IPL_BOOL result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_BEST_MATCH_ONLY, (void*)&result))
		{ throwImpactError(); }
		
		return (0 != result);
	}

inline MeasureEdgesBase& MeasureEdgesBase::setKernelType (KernelType kernelType)
	{
		mvIMPACT_C::GAU_KERNEL_TYPE_MODE iplKernelType(static_cast<mvIMPACT_C::GAU_KERNEL_TYPE_MODE>(kernelType));
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_KERNEL_TYPE, (void*)& iplKernelType))
		{ throwImpactError(); }
		return *this;
	}

inline KernelType MeasureEdgesBase::getKernelType () const
	{
		mvIMPACT_C::GAU_KERNEL_TYPE_MODE result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_KERNEL_TYPE, (void*)&result))
		{ throwImpactError(); }
		return KernelType(result);
	}

inline MeasureEdgesBase& MeasureEdgesBase::setSmoothPasses (int count)
	{
		 mvIMPACT_C::IPL_LONG iplCount = count;
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_SMOOTH_PASSES, (void*)& iplCount))
		{ throwImpactError(); }
		return *this;
	}

inline int MeasureEdgesBase::getSmoothPasses () const
	{
		mvIMPACT_C::IPL_LONG result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_SMOOTH_PASSES, (void*)&result))
		{ throwImpactError(); }
		
		return (int)result;
	}


inline MeasureEdges::MeasureEdges()
		: MeasureEdgesBase()
	{
		// Search for single edges
	 mvIMPACT_C::GAU_EDGE_PAIR_MODE mode = mvIMPACT_C::GAU_SINGLE;
		if (!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_EDGE_MODE, (void*)&mode))
			throwImpactError();
	}

inline MeasureEdges::MeasureEdges(const MeasureEdges& m)
		: MeasureEdgesBase()
	{
		*this = m;
	}

inline MeasureEdges& MeasureEdges::operator= (const MeasureEdges& rhs)
	{
		MeasureEdgesBase::operator= (rhs);
		return *this;
	}

	inline void MeasureEdges::addFeature (EdgeFeature feature)
	{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_AddFeature(getHandle(), (mvIMPACT_C::GAU_FEATURE)feature, 0))
			throwImpactError();
	}

inline void MeasureEdges::find 
	(
		const Image& S, 
		int x1, 
		int y1, 
		int x2, 
		int y2, 
		int width
	)
	{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindEdge(S.getHandle(),getHandle(),x1,y1,x2,y2,width))
			throwImpactError();
	}

inline void MeasureEdges::find(const Image& source, Line line, int width)
{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindEdge(
				source.getHandle(),
				getHandle(),
				line.getP0().getX(),
				line.getP0().getY(),
				line.getP1().getX(),
				line.getP1().getY(),
				width))
		{ throwImpactError(); }
}


inline int MeasureEdges::count 
	(
		int band, 
		int slice, 
		int frame
	) const
	{
		return (int) getFeature ( (EdgeFeature)mvIMPACT_C::GAU_EDGE_NUMBER,0,band,slice,frame );
	}

inline int MeasureEdges::getBands() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_BANDS, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline int MeasureEdges::getSlices() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_SLICES, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline int MeasureEdges::getFrames() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_FRAMES, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline double MeasureEdges::getFeature
	(
		EdgeFeature feature, 
		int index, 
		int band , 
		int slice, 
		int frame
	) const
	{
		assert(unassigned != getHandle());

		double v;
		if (!mvIMPACT_C::GAU_GetResults(
				getHandle(),
				index,
				(mvIMPACT_C::GAU_FEATURE)feature,
				band,slice,frame,
				&v))
			throwImpactError();

		return v;
	}

inline MeasureEdges& MeasureEdges::setPolarity( EdgePolarityMode mode )
	{
		mvIMPACT_C::GAU_POLARITY_MODE iplMode(static_cast<mvIMPACT_C::GAU_POLARITY_MODE>(mode));
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_POLARITY, (void*)& iplMode))
		{ throwImpactError(); }
		return *this;
	}

inline EdgePolarityMode MeasureEdges::getPolarity() const
	{
		 mvIMPACT_C::GAU_POLARITY_MODE iplResult;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_POLARITY, (void*)&iplResult))
		{ throwImpactError(); }
		return EdgePolarityMode(iplResult);
	}



inline MeasureEdgePairs::MeasureEdgePairs( PairMode mode )
		: MeasureEdgesBase()
	{
		// Search for edge pairs
		setPairMode (mode);
	}

inline MeasureEdgePairs::MeasureEdgePairs(const MeasureEdgePairs& m)
		: MeasureEdgesBase()
	{
		*this = m;
	}

inline MeasureEdgePairs& MeasureEdgePairs::operator= (const MeasureEdgePairs& rhs)
	{
		MeasureEdgesBase::operator= (rhs);
		return *this;
	}

	inline void MeasureEdgePairs::addFeature (EdgePairFeature feature)
	{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_AddFeature(getHandle(), (mvIMPACT_C::GAU_FEATURE)feature, 0))
			throwImpactError();
	}

inline void MeasureEdgePairs::find 
	(
		const Image& S, 
		int x1, 
		int y1, 
		int x2, 
		int y2, 
		int width
	)
	{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindEdge(S.getHandle(),getHandle(),x1,y1,x2,y2,width))
			throwImpactError();
	}

inline void MeasureEdgePairs::find(const Image& source, Line line, int width)
{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindEdge(
				source.getHandle(),
				getHandle(),
				line.getP0().getX(),
				line.getP0().getY(),
				line.getP1().getX(),
				line.getP1().getY(),
				width))
			throwImpactError();
}


inline int MeasureEdgePairs::count 
	(
		int band, 
		int slice, 
		int frame
	) const
	{
		return (int) getFeature ( (EdgePairFeature)mvIMPACT_C::GAU_EDGE_NUMBER,0,band,slice,frame );
	}

inline int MeasureEdgePairs::getBands() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_BANDS, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline int MeasureEdgePairs::getSlices() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_SLICES, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline int MeasureEdgePairs::getFrames() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_FRAMES, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}


inline double MeasureEdgePairs::getFeature 
	(
		EdgePairFeature feature, 
		int index, 
		int band , 
		int slice, 
		int frame
	) const
	{
		assert(unassigned != getHandle());

		double v;
		if (!mvIMPACT_C::GAU_GetResults(
				getHandle(),
				index,
				(mvIMPACT_C::GAU_FEATURE)feature,
				band,slice,frame,
				&v))
			throwImpactError();

		return v;
	}

inline MeasureEdgePairs& MeasureEdgePairs::setPairMode( PairMode mode )
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_EDGE_MODE, (void*)& mode))
		{ throwImpactError(); }
		return *this;
	}

inline PairMode MeasureEdgePairs::getPairMode() const
	{
		PairMode result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_EDGE_MODE, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgePairs& MeasureEdgePairs::setMinScore2 (double score)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_SCORE_2, (void*)& score))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgePairs::getMinScore2 () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_SCORE_2, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgePairs& MeasureEdgePairs::setContrastWeight2 (double weight)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_CONTRAST_2_WEIGHT, (void*)& weight))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgePairs::getContrastWeight2 () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_CONTRAST_2_WEIGHT, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgePairs& MeasureEdgePairs::setSteepnessWeight2 (double weight)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_STEEPNESS_2_WEIGHT, (void*)& weight))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgePairs::getSteepnessWeight2 () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_STEEPNESS_2_WEIGHT, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgePairs& MeasureEdgePairs::setPolarityWeight2 (double weight)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_POLARITY_2_WEIGHT, (void*)& weight))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgePairs::getPolarityWeight2 () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_POLARITY_2_WEIGHT, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgePairs& MeasureEdgePairs::setTypicalContrast2 (double weight)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_CONTRAST_2_TYPICAL, (void*)& weight))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgePairs::getTypicalContrast2 () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_CONTRAST_2_TYPICAL, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureEdgePairs& MeasureEdgePairs::setTypicalSteepness2 (double weight)
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_STEEPNESS_2_TYPICAL, (void*)& weight))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureEdgePairs::getTypicalSteepness2 () const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_STEEPNESS_2_TYPICAL, (void*)&result))
		{ throwImpactError(); }
		return result;
	}



inline MeasureCircleBase::MeasureCircleBase()
		: MeasureBase()
	{
	}

inline MeasureCircleBase::MeasureCircleBase( const MeasureCircleBase& b )
		: MeasureBase (b)
	{
	}

inline MeasureCircleBase& MeasureCircleBase::operator= ( const MeasureCircleBase& b )
	{
		MeasureBase::operator= (b);
		return *this;
	}

inline MeasureCircleBase& MeasureCircleBase::setRemovalIterations( int iterations )
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_ITERATIONS, (void*)& iterations))
		{ throwImpactError(); }
		return *this;
	}

inline int MeasureCircleBase::getRemovalIterations() const
	{
		mvIMPACT_C::IPL_LONG result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_ITERATIONS, (void*)&result))
		{ throwImpactError(); }
		
		return (int)result;
	}

inline MeasureCircleBase& MeasureCircleBase::setDistanceTolerance( double tolerance )
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_DIST_TOLERANCE, (void*)& tolerance))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureCircleBase::getDistanceTolerance() const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_DIST_TOLERANCE, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureCircleBase& MeasureCircleBase::setMaxIterations( int iterations )
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_MAX_ITERATIONS, (void*)&iterations))
		{ throwImpactError(); }
		return *this;
	}

inline int MeasureCircleBase::getMaxIterations() const
	{
		mvIMPACT_C::IPL_LONG result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_MAX_ITERATIONS, (void*)&result))
		{ throwImpactError(); }
		
		return (int)result;
	}

inline MeasureCircleBase& MeasureCircleBase::setTolerance( double tolerance )
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_TOLERANCE, (void*)& tolerance))
		{ throwImpactError(); }
		return *this;
	}

inline double MeasureCircleBase::getTolerance() const
	{
		double result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_TOLERANCE, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline MeasureCircleBase& MeasureCircleBase::setGradientThreshold( int threshold )
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_GRAD_THRESHOLD, (void*)&threshold))
		{ throwImpactError(); }
		return *this;
	}

inline int MeasureCircleBase::getGradientThreshold() const
	{
		mvIMPACT_C::IPL_LONG result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_GRAD_THRESHOLD, (void*)&result))
		{ throwImpactError(); }
		
		return (int)result;
	}

inline MeasureCircleBase& MeasureCircleBase::setMeasurementDirection( CircleMeasurementDirection direction )
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_MEASUREMENT_DIRECTION, (void*)&direction))
		{ throwImpactError(); }
		return *this;
	}

inline CircleMeasurementDirection MeasureCircleBase::getMeasurementDirection() const
	{
		CircleMeasurementDirection result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_MEASUREMENT_DIRECTION, (void*)&result))
		{ throwImpactError(); }
		return result;
	}




inline MeasureCircles::MeasureCircles()
		: MeasureCircleBase()
	{
		// Search for circles
		mvIMPACT_C::GAU_EDGE_PAIR_MODE mode = mvIMPACT_C::GAU_SINGLE;
		if (!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_EDGE_MODE, (void*)&mode))
			throwImpactError();
	}

inline MeasureCircles::MeasureCircles(const MeasureCircles& m)
		: MeasureCircleBase()
	{
		*this = m;
	}

inline MeasureCircles& MeasureCircles::operator= (const MeasureCircles& rhs)
	{
		MeasureCircleBase::operator= (rhs);
		return *this;
	}

inline void MeasureCircles::addFeature (CircleFeature feature)
	{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_AddFeature(getHandle(), (mvIMPACT_C::GAU_FEATURE)feature, 0))
			throwImpactError();
	}

inline void MeasureCircles::find 
	(
		const Image& S, 
		int x, 
		int y, 
		int width, 
		int height
	)
	{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindCircle(S.getHandle(),getHandle(),x,y,width,height))
			throwImpactError();
	}

inline void MeasureCircles::find
	(
		const Image& source, 
		Point & point, 
		int width, 
		int height
	)
{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindCircle(
				source.getHandle(),
				getHandle(),
				point.getX(),
				point.getY(),
				width,
				height))
			throwImpactError();
}
	
inline void MeasureCircles::find
	(
		const Image& source, 
		Rectangle & rectangle
	)
{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindCircle(
				source.getHandle(),
				getHandle(),
				rectangle.getLeft(),
				rectangle.getTop(),
				rectangle.getWidth(),
				rectangle.getHeight()))
			throwImpactError();
}

inline void MeasureCircles::find
	(
		const Image& source, 
		const std::vector<MeasureCircles>& windows,
		const std::vector<Rectangle>& rectangles
	)
{
		if (windows.size() != rectangles.size())
			throw ImpactException(mvIMPACT_C::IPL_SIZEMISMATCH_ERROR);

		std::vector<mvIMPACT_C::GAU_BUFHANDLE> vHandles;
		for (std::vector<MeasureCircles>::const_iterator i = windows.begin(); i != windows.end(); ++i)
			vHandles.push_back(i->getHandle());
		DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplHandles(vHandles);

		DynArray<mvIMPACT_C::IPL_RECT, Rectangle> iplRectangles(rectangles);

		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindCircleEx(
				source.getHandle(),
				getHandle(),
				iplHandles.getPointer(),
				iplRectangles.getPointer(),
				iplHandles.getSize()))
			throwImpactError();
}


inline int MeasureCircles::count 
	(
		int band, 
		int slice, 
		// If the source image consists of only one frame, set this parameter to 0.
		int frame
	) const
	{
		return (int) getFeature ( (CircleFeature)mvIMPACT_C::GAU_CIRCLE_NUMBER,0,band,slice,frame );
	}

inline int MeasureCircles::getBands() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_BANDS, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline int MeasureCircles::getSlices() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_SLICES, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline int MeasureCircles::getFrames() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_FRAMES, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}


inline double MeasureCircles::getFeature
	(
		CircleFeature feature, 
		int index, 
		int band, 
		int slice, 
		int frame
	) const
	{
		assert(unassigned != getHandle());

		double v;
		if (!mvIMPACT_C::GAU_GetResults(
				getHandle(),
				index,
				(mvIMPACT_C::GAU_FEATURE)feature,
				band,slice,frame,
				&v))
			throwImpactError();

		return v;
	}

inline MeasureCircles& MeasureCircles::setPolarity( EdgePolarityMode mode )
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_POLARITY, (void*)& mode))
		{ throwImpactError(); }
		return *this;
	}

inline EdgePolarityMode MeasureCircles::getPolarity() const
	{
		EdgePolarityMode result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_POLARITY, (void*)&result))
		{ throwImpactError(); }
		return result;
	}



inline MeasureCirclePair::MeasureCirclePair( PairMode mode )
		: MeasureCircleBase()
	{
		// Search for a circle pair
		mvIMPACT_C::GAU_EDGE_PAIR_MODE cmode = (mvIMPACT_C::GAU_EDGE_PAIR_MODE)mode;
		if (!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_EDGE_MODE, (void*)&cmode))
			throwImpactError();
	}

inline MeasureCirclePair::MeasureCirclePair(const MeasureCirclePair& m)
		: MeasureCircleBase()
	{
		*this = m;
	}

inline MeasureCirclePair& MeasureCirclePair::operator= (const MeasureCirclePair& rhs)
	{
		MeasureCircleBase::operator= (rhs);
		return *this;
	}

	inline void MeasureCirclePair::addFeature (CircleFeature feature)
	{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_AddFeature(getHandle(), (mvIMPACT_C::GAU_FEATURE)feature, 0))
			throwImpactError();
	}

inline void MeasureCirclePair::find 
	(
		const Image& S, 
		int x, 
		int y, 
		int width, 
		int height
	)
	{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindCircle(S.getHandle(),getHandle(),x,y,width,height))
			throwImpactError();
	}

inline void MeasureCirclePair::find 
	(
		const Image& S, 
		Point point,
		int width, 
		int height
	)
	{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindCircle(
				S.getHandle(),
				getHandle(),
				point.getX(),
				point.getY(),
				width,
				height))
			throwImpactError();
	}

inline void MeasureCirclePair::find
	(
		const Image& source, 
		Rectangle & rectangle
	)
{
		assert(unassigned != getHandle());
		if (!mvIMPACT_C::GAU_FindCircle(
				source.getHandle(),
				getHandle(),
				rectangle.getLeft(),
				rectangle.getTop(),
				rectangle.getWidth(),
				rectangle.getHeight()))
			throwImpactError();
}




inline int MeasureCirclePair::getBands() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_BANDS, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline int MeasureCirclePair::getSlices() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_SLICES, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline int MeasureCirclePair::getFrames() const
	{
		double v = 0;
		if (!mvIMPACT_C::GAU_GetResults(getHandle(), 0, mvIMPACT_C::GAU_NB_FRAMES, 0, 0, 0, &v))
			throwImpactError();
		return (int)v;
	}

inline double MeasureCirclePair::getFeature 
	(
		CircleFeature feature,
		int index, 
		int band , 
		int slice, 
		int frame
	) const
	{
		assert(unassigned != getHandle());

		double v;
		if (!mvIMPACT_C::GAU_GetResults(
				getHandle(),
				index,
				(mvIMPACT_C::GAU_FEATURE)feature,
				band,slice,frame,
				&v))
			throwImpactError();

		return v;
	}

inline MeasureCirclePair& MeasureCirclePair::setPairMode( PairMode mode )
	{
		if(!mvIMPACT_C::GAU_BufControl(getHandle(), mvIMPACT_C::GAU_EDGE_MODE, (void*)& mode))
		{ throwImpactError(); }
		return *this;
	}

inline PairMode MeasureCirclePair::getPairMode() const
	{
		PairMode result;
		if(!mvIMPACT_C::GAU_BufInquire(getHandle(), mvIMPACT_C::GAU_EDGE_MODE, (void*)&result))
		{ throwImpactError(); }
		return result;
	}

inline std::string getVersionMeasure()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::GAU_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}


	} // namespace measure
} // namespace mvIMPACT
#endif // #ifndef WRAP_ANY
#endif// DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_MEASURE_IMPL_H_
