#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page aoicallback_page AOI callback example (Python)

\section aoicallback_section_purpose Purpose
The aoicallback example shows, how callbacks can be used with AOIs.

\section aoicallback_section_intro What it does
An Area of Interest (AOI) is used to define a child region of an image interactively.
This child region is shown in another window.

\section aoicallback_section_description How it works
To react on events triggered by an AOI, one must overwrite the corresponding event
handlers of class Window. This is done by deriving from Window. In the constructor, the
member variable childWindow is set which is used to refer to the second window which shows
the selected Region. Note that the constructor of the base class has to be called. Otherwise
the derived class won't work properly.

\code
#this class handles the track events
class MyWindow(Window):

    #overload constructor
    def __init__(self, image, childWindow):
        Window.__init__(self, image)
        self.childWindow = childWindow
\endcode

The method onOoiTracked() is called whenever the AOI has been moved
or resized. So it is overwritten here. As the onOoiTracked method
delivers only an object of type ObjectOfInterest, the base class for all
types of Objects of Interest, the object has to be converted into an AreaOfInterest
object first.

\code
    #implementation of the tracked method
    def onOoiTracked(self, ooi):
        #convert Ooi to Aoi
        aoi = AreaOfInterest.castFromObjectOfInterest(ooi)
\endcode

Now, the position and dimensions of the AOI can be retrieved.

\code
        #get bounds of the aoi
        left = aoi.rectangle.left
        top = aoi.rectangle.top
        width = aoi.rectangle.width
        height = aoi.rectangle.height
\endcode

After clipping the area to a valid image region, the child window
is updated to correspond to the region. Then the second (small) window is redrawn.

\code
        # clipping
        # [...]

        #translate existing child image of the area that is covered by the aoi
        if width > 0 and height > 0:
            childImage = self.childWindow.buffer
            childImage.translate(left, top, width, height)

        #redraw window
        self.childWindow.redraw()
\endcode

In the "main program" first an image is loaded. Then the "child" window is created. Afterwards,
the derived class MyWindow can be instantiated. Finally, the AOI is created in \c wnd.

\code
#main program
#load and display image
img = loadImage("screw.bmp")

#create another window 100x100 and copy actual aoi
child = Image(
    img,
    0,
    0,
    0,
    100,
    100,
    img.bands)
wnd2 = Window(child)

wnd = MyWindow(img, wnd2)

#create an aoi
aoi = AreaOfInterest(wnd, (0, 0, 99, 99))

\endcode

For the windowing system to work correctly, it is required to call the global function
runMessageLoop() to get the user interface to work. This function blocks until space key is
pressed in one of the application's windows.

\code
#keep demo alive
runMessageLoop()
\endcode

Finally, the two window objects are destroyed, so the corresponding windows are closed. Note
that this is not neccessary here, because the application ends and the Python interpreter destroys the
objects anyway.

\code

del wnd
del wnd2

\endcode

\section aoicallback_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *

#this class handles the track events
class MyWindow(Window):

    #overload constructor
    def __init__(self, image, childWindow):
        Window.__init__(self, image)
        self.childWindow = childWindow

    #implementation of the tracked method
    def onOoiTracked(self, ooi):
        #convert Ooi to Aoi
        aoi = AreaOfInterest.castFromObjectOfInterest(ooi)

        #get bounds of the aoi
        left = aoi.rectangle.left
        top = aoi.rectangle.top
        width = aoi.rectangle.width
        height = aoi.rectangle.height

        #clipping
        if left < 0:
            width = width + left
            left = 0

        if top < 0:
            height = height + top
            top = 0

        if (left + width) > self.buffer.width:
            width = self.buffer.width - left
        if (top + height) > self.buffer.height:
            height = self.buffer.height - top

        #translate existing child image of the area that is covered by the aoi
        if width > 0 and height > 0:
            childImage = self.childWindow.buffer
            childImage.translate(left, top, width, height)

        #redraw window
        self.childWindow.redraw()

#main program
#load and display image
img = loadImage("screw.bmp")

#create another window 100x100 and copy actual aoi
child = Image(
    img,
    0,
    0,
    0,
    100,
    100,
    img.bands)
wnd2 = Window(child)

wnd = MyWindow(img, wnd2)

#create an aoi
aoi = AreaOfInterest(wnd, (0, 0, 99, 99))

#keep demo alive
runMessageLoop()

del wnd
del wnd2

##</matrixdoc>
