#! /usr/bin/python
# -*- coding: iso-8859-1 -*-

"""
/**
\\xmlonly
mvIMPACT Measure
\endxmlonly

\page autogauge_page Autogauge example (Python)

\section autogauge_section_purpose Purpose
This example shows how to use the MeasureEdges class to get information about a single edge.

\section autogauge_section_intro What it does
The example shows an image and a LineOfInterest. Every time the LineOfInterest is moved or
resized the mvIMPACT Measure module is used to calculate a line, that represents the detected edge.

\section autogauge_section_description How it works
First of all the mouse handler has to be implemented. This is done by subclassing base.Window and
implementing the onMousebuttonUp method. This method will be called each time a mouse button is released
and starts the measurement process. The class also stores the results and the condition, if an edge
could be detected at all, in member variables, which are used by the drawing handler method onDraw to
draw a line representing the edge.

The constructor of the subclass first calls the corresponding constructor of the parent class base.Window.
If this is not done, no window will be opened. Then the member variables representing the measurement are initialized.

\code

class MyWindow(Window):

    def __init__(self, image):
        Window.__init__(self, image)
        self.edgeX0 = 0
        self.edgeY0 = 0
        self.edgeX1 = 0
        self.edgeY1 = 0
        self.lineIsValid = False

\endcode

The buttonUp method reads the coordinates of the LineOfInterest.

\code
    def onMouseButtonUp(self, button, point):

        #get the coordinates of the line
        line = loi.line
        x0 = line.p0.x
        y0 = line.p0.y
        x1 = line.p1.x
        y1 = line.p1.y

        #get width of the line to one side and multiply
        #by 2 to calculate the width of the measuring window
        width = loi.width * 2

        print x0,"\t", y0,"\t",x1,"\t", y1,"\t", width

\endcode

 Then a MeasureEdges object is created. This object is used to do the actual measuring. In the
 example we want to calculate a line between the two intersection points of the edge with the
 image border. Thus we have to add the corresponding features. In the example these are:
 efX0Interset, efY0Intersect, efX1Intersect and efY1Intersect. The next instructions tell the
 MeasureEdges object to ignore edges with a match score below 40 and to deliver only the edge, that
 has the maximum match score. Thus, there will be no or exactly one result.

\code
        #perform measurement for single edges
        measure = MeasureEdges()

        #add requested features
        measure.addFeature(efX0Intersect)
        measure.addFeature(efY0Intersect)
        measure.addFeature(efX1Intersect)
        measure.addFeature(efY1Intersect)

        #set some other properties
        measure.minScore = 40
        measure.bestMatchOnly = True

\endcode

 In the next step the measurement process is started. This is done by calling
 measure.MeasureEdges.find. In the example the center line and the width of the
 measurement window is defined by the LineOfInterest.
 The call is surrounded by a try-except clause, because the function throws an exception
 if it does not find a circle. The number of results is then set to 0.
 If the call was successful the number of results is queried by calling
 measure.MeasureEdges.count.

\code
        #define measurement window and search for edges
        try:
            measure.find(self.buffer.asImage(),
                         x0, y0, x1, y1, width)
            #get number of results
            edges = measure.count()
        except ImpactException:
            edges = 0

\endcode

 If there are any results they are queried by calling meausre.MeasureEdges.get
 and passing the corresponding feature. The four values defining the line are retrieved and the
 member variables are set appropriately, so that they can be displayed. The drawing of the line is done
 by a separate method.

\code
        if edges > 0:
            #get results
            self.edgeX0 = measure.getFeature(efX0Intersect)
            self.edgeY0 = measure.getFeature(efY0Intersect)
            self.edgeX1 = measure.getFeature(efX1Intersect)
            self.edgeY1 = measure.getFeature(efY1Intersect)
            self.lineIsValid = True
        else:
            self.lineIsValid = False
\endcode

In the main program the derived class MyWindow is instantiated and the LineOfInterest object is created.
Afterwards a short help message is printed.
\code
#open window and display the image
window = MyWindow(img)

#create a LineOfInterest
loi = LineOfInterest(window, (100, 100, 170, 170))
loi.color = (0, 0, 255)
loi.showArrow = True
loi.width = 20
loi.fillVisible = True
loi.fillTransparent = True
loi.dragWidth = True

#output message
print "Drag the LOI over an edge to be measured."
print "The LOI should be placed perpendicular to the edge."
print "Releasing the left mouse button starts the measurement."
print "(The coordinates x0, y0, x1, y1 and the width of the line are listed.)\n"
\endcode

Finally the message loop is run and the window is destroyed after the user has pressed space.

\code
#wait until the user presses space
runMessageLoop()
\endcode

\section autogauge_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.measure import *

#this class handles the track events
class MyWindow(Window):

    def __init__(self, image):
        Window.__init__(self, image)
        self.edgeX0 = 0
        self.edgeY0 = 0
        self.edgeX1 = 0
        self.edgeY1 = 0
        self.lineIsValid = False

    #implementation of the buttonUp method
    def onMouseButtonUp(self, button, point):

        #get the coordinates of the line
        line = loi.line
        x0 = line.p0.x
        y0 = line.p0.y
        x1 = line.p1.x
        y1 = line.p1.y

        #get width of the line to one side and multiply
        #by 2 to calculate the width of the measuring window
        width = loi.width * 2

        print x0,"\t", y0,"\t",x1,"\t", y1,"\t", width
        #perform measurement for single edges
        measure = MeasureEdges()

        #add requested features
        measure.addFeature(efX0Intersect)
        measure.addFeature(efY0Intersect)
        measure.addFeature(efX1Intersect)
        measure.addFeature(efY1Intersect)

        #set some other properties
        measure.minScore = 40
        measure.bestMatchOnly = True

        #define measurement window and search for edges
        try:
            measure.find(self.buffer.asImage(),
                         x0, y0, x1, y1, width)
            #get number of results
            edges = measure.count()
        except ImpactException:
            edges = 0

        if edges > 0:
            #get results
            self.edgeX0 = measure.getFeature(efX0Intersect)
            self.edgeY0 = measure.getFeature(efY0Intersect)
            self.edgeX1 = measure.getFeature(efX1Intersect)
            self.edgeY1 = measure.getFeature(efY1Intersect)
            self.lineIsValid = True
        else:
            self.lineIsValid = False

        self.redraw()
        return True

    def onDraw(self, dc, sourceRect, destRect):
          if self.lineIsValid:
              spx = self.scrollPosX
              spy = self.scrollPosY

              dc.drawLine(Line(
                      int(self.edgeX0-spx),
                      int(self.edgeY0-spy),
                      int(self.edgeX1-spx),
                      int(self.edgeY1-spy)),
                      RGBColor(0, 0, 255))

#main program

#load an image
img = loadImage("stanz.bmp")

#open window and display the image
window = MyWindow(img)

#create a LineOfInterest
loi = LineOfInterest(window, (100, 100, 170, 170))
loi.color = (0, 0, 255)
loi.showArrow = True
loi.width = 20
loi.fillVisible = True
loi.fillTransparent = True
loi.dragWidth = True

#output message
print "Drag the LOI over an edge to be measured."
print "The LOI should be placed perpendicular to the edge."
print "Releasing the left mouse button starts the measurement."
print "(The coordinates x0, y0, x1, y1 and the width of the line are listed.)\n"

#wait until the user presses space
runMessageLoop()

del window

#</matrixdoc>
