#! /usr/bin/python
# -*- coding: iso-8859-1 -*-

"""
/**
\\xmlonly
mvIMPACT Measure
\endxmlonly

\page autogaugecircle_page AutogaugeCircle example (Python)

\section autogaugecircle_section_purpose Purpose
This example shows how to use the MeasureCircles class to get information about a single circle.

\section autogaugecircle_section_intro What it does
The example shows an image and an AreaOfInterest. Every time the AreaOfInterest is moved or resized the mvIMPACT Measure
module is used to determine the radius and the center point of the underlying circle. The detected circle is drawn onto
the image and the radius and the coordinates of the center point are displayed.

\section autogaugecircle_section_description How it works
First of all the mouse handler has to be implemented. This is done by creating a subclass of class Window and
implementing the onMouseButtonUp method. This method will be called each time a mouse button is released
and starts the measurement process. The class also stores the results and the condition, if a circle
could be detected at all, in member variables. These are used by the overloaded method onDraw to draw the circle into
the window.

In the constructor of the subclass first the corresponding constructor of class Window is called. Then
the member variables for storing the results and the AreaOfInterest object for displaying
the results are initialized.

\code
class MyWindow(Window):

    def __init__(self, image):
        Window.__init__(self, image)
        self.radius = 0.
        self.centX = 0.
        self.centY = 0.
        self.circleIsValid = False

        self.dispAoi = AreaOfInterest(
                                self,
                                Rectangle((320, 10), (320, 10))
                                )
        self.dispAoi.textColor = (255, 0, 0)
        self.dispAoi.visible = True
        self.dispAoi.uIActive = False
\endcode

The buttonUp method reads the coordinates of the AreaOfInterest.

\code
    def onMouseButtonUp (self, button, point):
        #get the coordinates of the rectangle
        rectangle = aoi.rectangle
        left = rectangle.left
        top = rectangle.top
        right = rectangle.right
        bottom = rectangle.bottom

\endcode

Then a MeasureCircles object is created. This object is used to do the actual measuring. In the
example we want to calculate the radius and the center point of the detected circle.
Thus we have to add the corresponding features. In the example these are:
cfRadius, cfXCenter and cfYCenter.

\code
        #perform measurement for single circles
        measure = MeasureCircles()

        #add requested features
        measure.addFeature(cfRadius)
        measure.addFeature(cfXCenter)
        measure.addFeature(cfYCenter)
\endcode

 In the next step the measurment process is started. This is done by calling
 measure.MeasureCircles.find. In the example the measurement window is
 defined by the AreaOfInterest. The call is surrounded by a try-except clause, because the function throws an exception
 if it does not find a circle. The number of results is then set to 0.
 If the call was successful the number of results is queried by calling measure.MeasureCircles.count.

\code
        try:
            #define measurement window and search for circles
            measure.find(self.myImage, left, top, abs(right-left), abs(bottom-top))
            #get number of results
            edges = measure.count()
        except:
            edges = 0
\endcode

 If there are any results they are queried by calling measure.MeasureCircles.get
 and passing the corresponding feature. The radius and the center point are retrieved and the
 member variables are set appropriately, so that they can be displayed. The drawing of the circle is done
 by the overloaded method onDraw.

\code
        if edges > 0:
            #get results
            self.radius = measure.getFeature(cfRadius)
            self.centX = measure.getFeature(cfXCenter)
            self.centY = measure.getFeature(cfYCenter)
            self.circleIsValid = True
            message = (
                "Radius:%.3f, x-center:%.3f, y-center:%.3f"
                   % ( self.radius, self.centX, self.centY )
                )
        else:
            self.circleIsValid = False
            message = "measurement failed"

\endcode

 In the main program an instance of the new class MyWindow is created and
 the AreaOfInterest object for specifying the measurement region is created.
 Afterwards a short help message is printed.

\code
#open window and display the image
window = MyWindow(img)

#create a AreaOfInterest
aoi = AreaOfInterest(
    window,
    (10, 10, 110, 110)
    )

#output message
print "Drag the AOI over a circle or circular arc to be measured.\n"
print "Releasing the left mouse button starts the measurement.\n\n"

\endcode

Finally the message loop is run and after the user has pressed space, the
window is destroyed.

\code
#wait until the user presses space
runMessageLoop()

del window
\endcode

\section autogaugecircle_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.measure import *

#this class handles the track events
class MyWindow(Window):

    def __init__(self, image):
        Window.__init__(self, image)
        self.radius = 0.
        self.centX = 0.
        self.centY = 0.
        self.circleIsValid = False

        self.dispAoi = AreaOfInterest(
                                self,
                                Rectangle((320, 10), (320, 10))
                                )
        self.dispAoi.textColor = (255, 0, 0)
        self.dispAoi.visible = True
        self.dispAoi.uIActive = False

    #implementation of the onMouseButtonUp method
    def onMouseButtonUp (self, button, point):
        #get the coordinates of the rectangle
        rectangle = aoi.rectangle
        left = rectangle.left
        top = rectangle.top
        right = rectangle.right
        bottom = rectangle.bottom

        print left, top, right, bottom, "\n"

        #perform measurement for single circles
        measure = MeasureCircles()

        #add requested features
        measure.addFeature(cfRadius)
        measure.addFeature(cfXCenter)
        measure.addFeature(cfYCenter)

        try:
            #define measurement window and search for circles
            measure.find(self.buffer.asImage(),
                         left, top, abs(right-left), abs(bottom-top))
            #get number of results
            edges = measure.count()
        except ImpactException:
            edges = 0

        if edges > 0:
            #get results
            self.radius = measure.getFeature(cfRadius)
            self.centX = measure.getFeature(cfXCenter)
            self.centY = measure.getFeature(cfYCenter)
            self.circleIsValid = True
            message = (
                "Radius:%.3f, x-center:%.3f, y-center:%.3f"
                   % ( self.radius, self.centX, self.centY )
                )
        else:
            self.circleIsValid = False
            message = "measurement failed"

        #draw results
        self.dispAoi.text = message

        self.redraw()
        return True

    def onDraw(self, dc, sourceRect, destRect):
          if self.circleIsValid:
              spx = self.scrollPosX
              spy = self.scrollPosY
              dc.drawEllipse(
                    Rectangle(
                        int(self.centX-self.radius-spx), int(self.centY-self.radius-spy),
                        int(self.centX+self.radius-spx), int(self.centY+self.radius-spy)
                    ),
                    (100, 100, 100)
                )

#main program

#load an image
img = loadImage("stanz.bmp")

#open window and display the image
window = MyWindow(img)

#create a AreaOfInterest
aoi = AreaOfInterest(
    window,
    (10, 10, 110, 110)
    )

#output message
print "Drag the AOI over a circle or circular arc to be measured.\n"
print "Releasing the left mouse button starts the measurement.\n\n"

#wait until the user presses space
runMessageLoop()

del window

#</matrixdoc>
