#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Barcode
\endxmlonly

\page barcode_page Barcode example (Python)

\section barcode_section_purpose Purpose
Barcodes are a machine-readable representation of information (as bars and gaps) on a surface.
This example shows how to extract these information from an image.

\section barcode_section_intro What it does
The sample extracts from a UPC/EAN code the information using nearly the default barcode settings.

\section barcode_section_description How it works
First of all the sample loads an image and displays it in a window.

\code
#load and display an image
image = loadImage("BarCode16.bmp")
window = Window(image)
\endcode

Then the barcode settings are initialised and the MinGaps are set to "11". That means, the barcode
must have minimum 11 spaces (gaps). The last line of the code fragment below enables the barcode
print quality inspection according to ANSI X3.182. Without enabling this switch no print quality
inspection values are calculated.

\code

#setup bar code configuration
config = BarcodeConfiguration()
config.contrast = 10
config.minGaps = 10
config.noiseThreshold = 40
config.checkPrintQuality = True

\endcode

In the next step the barcode will be decoded with the given configuration.

\code
#decode the barcode with the given configuration
results = decodeBC(image, config)
\endcode

After that the program iterates through all barcodes and outputs parameters such as code type,
code data, checksum, location, direction and the overprint factor. This information can always be
inquired, that means it does not depend on the flag that enables or disables print quality inspection.

\code
if candidates > 0:
    total_time = results.totalDecodingTime
    print "Total decoding time: ", total_time

    #iterate through all candidates
    for candCounter in range(candidates):
        print "Candidate: ", candCounter
        codes = results.getNumberOfCodes(candCounter)

        #iterate through all codes
        for codeCounter in range(codes):
            print "Code: ", codeCounter

            #get the code type
            codeType = results.getCodeType(candCounter, codeCounter)
            print "Decoder: ", codeType

            #get the code data
            codeData = results.getCode(candCounter, codeCounter)
            print "Code: ", codeData

            #get the checksum
            checksum = results.getChecksum(candCounter, codeCounter)
            print "Ckecksum: ", checksum

            #get the location
            location = results.getLocation(candCounter)
            print "Location: %s %s %s %s" % tuple([(a.x, a.y) for a in location])

            #overprint
            overprint = results.getOverprint(candCounter)
            print "Overprint: ", overprint

            #barcode direction
            direction = results.getDirection(candCounter)
            if direction:
                print "Direction: forward"
            else:
                print "Direction: backward"

            #decoding time
            time = results.getDecodingTime(candCounter)
            print "Decoding time: ", time

\endcode

In the next step the print quality inspection values according to ANSI X3.182 are printed
on the console. These values can only be inquired if the print quality inspection was enabled before
decoding the barcode.

\code

            #reflectance minimum
            rmin = results.getReflectanceMinimum(candCounter)
            print "X3.182 Reflectance minimum: ", rmin

            #reflectance maximum
            rmax = results.getReflectanceMaximum(candCounter)
            print "X3.182 Reflectance maximum: ", rmax

            #symbol contrast
            sc = results.getSymbolContrast(candCounter)
            print "X3.182 Symbol contrast: ", sc

            #global threshold
            gt = results.getGlobalThreshold(candCounter)
            print "X3.182 Global threshold: ", gt

            #minimum edge contrast
            minec = results.getMinimumEdgeContrast(candCounter)
            print "X3.182 Minimum edge contrast: ", minec

            #modulation
            mod = results.getModulation(candCounter)
            print "X3.182 Modulation: ", mod

            #maximum non-uniformity
            mnu = results.getMaximumNonUniformity(candCounter)
            print "X3.182 Maximum non-uniformity: ", mnu

            #edge determination test
            edt = results.getEdgeDeterminationTest(candCounter)
            if edt:
                print "X3.182 Edge determination test: passed"
            else:
                print "X3.182 Edge determination test: failed"

            #minimum reflectance test
            mrt = results.getMinimumReflectanceTest(candCounter)
            if mrt:
                print "X3.182 Miniumm reflectance test: passed"
            else:
                print "X3.182 Miniumm reflectance test: failed"

            #minimum edge contrast test
            mect = results.getMinimumEdgeContrastTest(candCounter)
            if mect:
                print "X3.182 Miniumm edge contrast test: passed"
            else:
                print "X3.182 Miniumm edge contrast test: failed"

            #symbol contrast grade
            scg = results.getSymbolContrastGrade(candCounter)
            if scg == 4:
                scg_str = "(A)"
            elif scg == 3:
                scg_str = "(B)"
            elif scg == 2:
                scg_str = "(C)"
            elif scg == 1:
                scg_str = "(D)"
            elif scg == 0:
                scg_str = "(F)"
            print "X3.182 Symbol contrast grade: " + str(scg)  + " " + scg_str

            #modulation grade
            mg = results.getModulationGrade(candCounter)
            if mg == 4:
                mg_str = "(A)"
            elif mg == 3:
                mg_str = "(B)"
            elif mg == 2:
                mg_str = "(C)"
            elif mg == 1:
                mg_str = "(D)"
            elif mg == 0:
                mg_str = "(F)"
            print "X3.182 Modulation grade: " + str(mg) + " " +  mg_str

            #defects grade
            dg = results.getDefectsGrade(candCounter)
            if dg == 4:
                dg_str = "(A)"
            elif dg == 3:
                dg_str = "(B)"
            elif dg == 2:
                dg_str = "(C)"
            elif dg == 1:
                dg_str = "(D)"
            elif dg == 0:
                dg_str = "(F)"
            print "X3.182 Defects grade: " + str(dg) + " " + dg_str

            #decodability grade
            decg = results.getDecodabilityGrade(candCounter)
            if decg == 4:
                decg_str = "(A)"
            elif decg == 3:
                decg_str = "(B)"
            elif decg == 2:
                decg_str = "(C)"
            elif decg == 1:
                decg_str = "(D)"
            elif decg == 0:
                decg_str = "(F)"
            print "X3.182 Decodability grade: " + str(decg) + " " + decg_str

            #scan grade
            sg = results.getScanGrade(candCounter)
            if sg == 4:
                sg_str = "(A)"
            elif sg == 3:
                sg_str = "(B)"
            elif sg == 2:
                sg_str = "(C)"
            elif sg == 1:
                sg_str = "(D)"
            elif sg == 0:
                sg_str = "(F)"
            print "X3.182 Scan grade: " + str(sg) + " " + sg_str

\endcode

\section barcode_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.barcode import *

#load and display an image
image = loadImage("BarCode16.bmp")
window = Window(image)

#setup bar code configuration
config = BarcodeConfiguration()
config.contrast = 10
config.minGaps = 10
config.noiseThreshold = 40
config.checkPrintQuality = True

#decode the barcode with the given configuration
results = decodeBC(image, config)

candidates = results.numberOfCandidates
print "NumberOfCandidates: ", candidates
if candidates > 0:
    total_time = results.totalDecodingTime
    print "Total decoding time: ", total_time

    #iterate through all candidates
    for candCounter in range(candidates):
        print "Candidate: ", candCounter
        codes = results.getNumberOfCodes(candCounter)

        #iterate through all codes
        for codeCounter in range(codes):
            print "Code: ", codeCounter

            #get the code type
            codeType = results.getCodeType(candCounter, codeCounter)
            print "Decoder: ", codeType

            #get the code data
            codeData = results.getCode(candCounter, codeCounter)
            print "Code: ", codeData

            #get the checksum
            checksum = results.getChecksum(candCounter, codeCounter)
            print "Ckecksum: ", checksum

            #get the location
            location = results.getLocation(candCounter)
            print "Location: %s %s %s %s" % tuple([(a.x, a.y) for a in location])

            #overprint
            overprint = results.getOverprint(candCounter)
            print "Overprint: ", overprint

            #barcode direction
            direction = results.getDirection(candCounter)
            if direction:
                print "Direction: forward"
            else:
                print "Direction: backward"

            #decoding time
            time = results.getDecodingTime(candCounter)
            print "Decoding time: ", time

            #reflectance minimum
            rmin = results.getReflectanceMinimum(candCounter)
            print "X3.182 Reflectance minimum: ", rmin

            #reflectance maximum
            rmax = results.getReflectanceMaximum(candCounter)
            print "X3.182 Reflectance maximum: ", rmax

            #symbol contrast
            sc = results.getSymbolContrast(candCounter)
            print "X3.182 Symbol contrast: ", sc

            #global threshold
            gt = results.getGlobalThreshold(candCounter)
            print "X3.182 Global threshold: ", gt

            #minimum edge contrast
            minec = results.getMinimumEdgeContrast(candCounter)
            print "X3.182 Minimum edge contrast: ", minec

            #modulation
            mod = results.getModulation(candCounter)
            print "X3.182 Modulation: ", mod

            #maximum non-uniformity
            mnu = results.getMaximumNonUniformity(candCounter)
            print "X3.182 Maximum non-uniformity: ", mnu

            #edge determination test
            edt = results.getEdgeDeterminationTest(candCounter)
            if edt:
                print "X3.182 Edge determination test: passed"
            else:
                print "X3.182 Edge determination test: failed"

            #minimum reflectance test
            mrt = results.getMinimumReflectanceTest(candCounter)
            if mrt:
                print "X3.182 Miniumm reflectance test: passed"
            else:
                print "X3.182 Miniumm reflectance test: failed"

            #minimum edge contrast test
            mect = results.getMinimumEdgeContrastTest(candCounter)
            if mect:
                print "X3.182 Miniumm edge contrast test: passed"
            else:
                print "X3.182 Miniumm edge contrast test: failed"

            #symbol contrast grade
            scg = results.getSymbolContrastGrade(candCounter)
            if scg == 4:
                scg_str = "(A)"
            elif scg == 3:
                scg_str = "(B)"
            elif scg == 2:
                scg_str = "(C)"
            elif scg == 1:
                scg_str = "(D)"
            elif scg == 0:
                scg_str = "(F)"
            print "X3.182 Symbol contrast grade: " + str(scg)  + " " + scg_str

            #modulation grade
            mg = results.getModulationGrade(candCounter)
            if mg == 4:
                mg_str = "(A)"
            elif mg == 3:
                mg_str = "(B)"
            elif mg == 2:
                mg_str = "(C)"
            elif mg == 1:
                mg_str = "(D)"
            elif mg == 0:
                mg_str = "(F)"
            print "X3.182 Modulation grade: " + str(mg) + " " +  mg_str

            #defects grade
            dg = results.getDefectsGrade(candCounter)
            if dg == 4:
                dg_str = "(A)"
            elif dg == 3:
                dg_str = "(B)"
            elif dg == 2:
                dg_str = "(C)"
            elif dg == 1:
                dg_str = "(D)"
            elif dg == 0:
                dg_str = "(F)"
            print "X3.182 Defects grade: " + str(dg) + " " + dg_str

            #decodability grade
            decg = results.getDecodabilityGrade(candCounter)
            if decg == 4:
                decg_str = "(A)"
            elif decg == 3:
                decg_str = "(B)"
            elif decg == 2:
                decg_str = "(C)"
            elif decg == 1:
                decg_str = "(D)"
            elif decg == 0:
                decg_str = "(F)"
            print "X3.182 Decodability grade: " + str(decg) + " " + decg_str

            #scan grade
            sg = results.getScanGrade(candCounter)
            if sg == 4:
                sg_str = "(A)"
            elif sg == 3:
                sg_str = "(B)"
            elif sg == 2:
                sg_str = "(C)"
            elif sg == 1:
                sg_str = "(D)"
            elif sg == 0:
                sg_str = "(F)"
            print "X3.182 Scan grade: " + str(sg) + " " + sg_str

runMessageLoop()

del window

#</matrixdoc>
