#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT GMM
\endxmlonly

\page gmm_page Gmm example (Python)

\section gmm_section_purpose Purpose
This example shows how to use the Geometrical Model Matcher of mvIMPACT.

\section gmm_section_intro What it does
A model image is analyzed, a subset of the model is constructed and saved to disk and
afterwards the original model is searched for in a scene image.

\section gmm_section_description How it works

First two tool functions are defined. The first one prints the features of a model. The second one
prints information about the matches in an object of class GeoMatchResult.

\code
def printSelectedFeatures(model):
    sys.stderr.write("\nFeatures that are used for pattern matching:\n")

    ## Corners
    nCountCorners = model.enabledCornersCount

    sys.stderr.write("\nNumber of corners %d\n" % nCountCorners)

    if nCountCorners > 0:

        corners = model.enabledCorners

        for i in range(nCountCorners):
            corner = corners[i]
            sys.stderr.write("Corner %3d: Corner point: %3d/%3d, opening angle: %3d degrees\n" %
                (i, int(corner.cornerPoint.x), int(corner.cornerPoint.y), int(corner.openingAngle/math.pi*180.0)))

    ## Arcs
    nCountArcs = model.enabledArcsCount

    sys.stderr.write("\nNumber of arcs %d\n" % nCountArcs)

    if nCountArcs > 0:
        arcs = model.arcs

        for i in range(nCountArcs):
            arc = arcs[i]
            sys.stderr.write("Arc %3d: Center point: %d/%d, radius: %d\n" %
                (i, int(arc.center.x), int(arc.center.y), int(arc.radius)))

def printMatches(matchResult):
    nCountMatches = matchResult.numberOfMatches

    sys.stderr.write("\nNumber of matches: %d\n" % nCountMatches)

    for i in range(nCountMatches):
        currMatch = matchResult.getMatch(i)
        sys.stderr.write("\nMatch %d:\n" % i)
        sys.stderr.write("x translation: %3.2f\n" % currMatch.transX)
        sys.stderr.write("y translation: %3.2f\n" % currMatch.transY)
        sys.stderr.write("rotation:      %3.2f degrees\n" % (currMatch.rotation/math.pi*180.0))
        sys.stderr.write("scaling:       %3.2f\n" % currMatch.scale)
        sys.stderr.write("score:         %3.2f\n" % currMatch.score)

\endcode

Then a helper class is defined for displaying a list of corners in a window. The corners
are displayed as a cross consisting of two lines of interest (LOIs).

\code
class CornerDisplay:
    def __init__(self, window):
        self.m_window = window
        self.m_corners = []
        self.m_loi1 = []
        self.m_loi2 = []

    def clear(self):
        self.m_corners = []
        self.m_loi1 = []
        self.m_loi2 = []

    def addCorners(self, corners):
        for currCorner in corners:

            self.m_corners.append(currCorner)

            newLoi1 = LineOfInterest(
                self.m_window,
                Line(
                    Point(int(currCorner.cornerPoint.x-5), int(currCorner.cornerPoint.y)),
                    Point(int(currCorner.cornerPoint.x+5), int(currCorner.cornerPoint.y))))
            newLoi1.color = (255, 0, 0)
            self.m_loi1.append(newLoi1)

            newLoi2 = LineOfInterest(
                self.m_window,
                Line(
                    Point(int(currCorner.cornerPoint.x), int(currCorner.cornerPoint.y-5)),
                    Point(int(currCorner.cornerPoint.x), int(currCorner.cornerPoint.y+5))))
            newLoi2.color = (255, 0, 0)
            self.m_loi2.append(newLoi2)
\endcode

Then another helper class is defined to display the matches represented by a
GeoMatchResult object in a given window. The positions of the matches as well as
a line of text additionally describing the rotation and scaling of the matches are
displayed in the given window which should display the scene image.

\code
class MatchDisplay:

    def __init__(self, window):
        self.m_window = window
        self.m_gmr = GeoMatchResult()
        self.m_loi1 = []
        self.m_loi2 = []
        self.m_tloi = []
        self.m_window = window
        self.m_center = Point(0, 0)

    def clear(self):
        self.m_gmr = GeoMatchResult()
        self.m_loi1 = []
        self.m_loi2 = []
        self.m_tloi = []

    def setGeoMatchResult(self, r, center):
        self.clear()

        self.m_gmr = r
        self.m_center = center

        for i in range(self.m_gmr.numberOfMatches):
            currMatch = self.m_gmr.getMatch(i)

            newLoi1 = LineOfInterest(
                self.m_window,
                Line(
                    Point(
                        int(center.x + currMatch.transX - 5),
                        int(center.y + currMatch.transY)),
                    Point(
                        int(center.x + currMatch.transX + 5),
                        int(center.y + currMatch.transY))))

            newLoi1.color = (0, 255, 0)
            self.m_loi1.append(newLoi1)

            newLoi2 = LineOfInterest(
                self.m_window,
                Line(
                    Point(
                        int(center.x + currMatch.transX),
                        int(center.y + currMatch.transY - 5)),
                    Point(
                        int(center.x + currMatch.transX),
                        int(center.y + currMatch.transY + 5))))
            newLoi2.color = (0, 255, 0)

            newLoi2.text = "%d" % i
            newLoi2.textColor = (0, 0, 255)
            self.m_loi2.append(newLoi2)

            pt = Point(220, 20*i + 20)
            newTLoi = LineOfInterest(
                self.m_window,
                Line(pt, pt))

            text = "%d - tx: %3.2f ty:  %3.2f rot: %3.2f scale: %3.2f score: %3.2f " % (
                            i , currMatch.transX, currMatch.transY, currMatch.rotation/math.pi*180.0,
                                currMatch.scale*100., currMatch.score*100.
                            )
            newTLoi.text = text
            newTLoi.textColor = (0, 255, 255)
            self.m_tloi.append(newTLoi)
\endcode

In the main program, first an image is loaded and displayed in a window.
\code
## main()

## Load model image
modelImage = loadImage("model.bmp")

## display the loaded image
window = Window(modelImage)
runMessageLoop("The Model! Click the space bar to continue\n")
\endcode

Then a GeoModel object is constructed. During construction the feature extraction
process is done. The resulting model shall contain a maximum of 7 features and
a threshold of 30 is used to choose edge pixels for model verification.

\code
## Model
geoModel = GeoModel(modelImage, 7, 30)
\endcode

Afterwards the model features are printed on the terminal and displayed in the
window using the helper class CornerDisplay.
\code
## Print all model features:
printSelectedFeatures(geoModel)
cd = CornerDisplay(window)
cd.addCorners(geoModel.enabledCorners)

window.redraw()
runMessageLoop("The 8 corners! Click the space bar to continue\n")
cd.clear()
\endcode

Now the edge points used for model verification are displayed in the window. Afterwards
the window is changed to show again the model image.
\code
## Read edge points of model
edgePointImage = geoModel.edgePoints

window.buffer = edgePointImage
window.redraw()
runMessageLoop("The Edge Points! Click the space bar to continue\n")
window.buffer = modelImage
window.redraw()
\endcode

Now a new empty model is created using the same width and height as the existing one.
Then at most 5 features are copied from the existing model to the newly created one, i.e.
afterwards it is a "reduced version" of the original model.
\code
myModel = GeoModel(geoModel.width, geoModel.height)

myModel.edgePoints = geoModel.edgePoints
nCountCorners = geoModel.enabledCornersCount
nCountCopyCorners = nCountCorners

if nCountCopyCorners > 5:
    nCountCopyCorners = 5
corners = geoModel.enabledCorners[:nCountCopyCorners]

for corner in corners:
    myModel.addCorner(corner)
\endcode

Afterwards the new reduced model is displayed in the window.

\code
## Print selected 5 model features from copied model:
printSelectedFeatures(myModel)
cd.addCorners(myModel.enabledCorners)

window.redraw()
runMessageLoop("The 5 corners! Click the space bar to continue\n")
cd.clear()
\endcode

Then the reduced model is saved on disk.

\code
## Save copied model to file "copiedmodel.gmm"
myModel.save("copiedmodel.gmm")
\endcode

Then the rotation center of the model is set. This point is used to specify the
translation position of a model in the scene as well.

\code
## Set rotation center to the middle of the model image
dx = geoModel.width/2.0
geoModel.rotationCenterX = dx
dy = geoModel.height/2.0
geoModel.rotationCenterY = dy
\endcode

Then the object representing the matcher is created and some configuration
settings are set.

\code
## generate Matcher object
matcher = GeoMatcher()

## Set some options for pattern matching. It is not obligatory to set options.
## You can also use default parameters.
matcher.minScore = 0.8
matcher.maxNumberOfMatches = mmMatcherFindAll
\endcode

Afterwards the scene image is loaded from disk and the pattern matching process
is started using this scene.

\code
## Load scene image
sceneImage = loadImage("scene.bmp")

## Start pattern matching
matchResult = matcher.findGeoModel(geoModel, sceneImage)
\endcode

Finally the results of the matching process are displayed on the terminal as
well as in a newly created window. Afterwards the helper objects are destroyed as well
as the two windows.

\code
sceneWindow = Window( sceneImage )
printMatches(matchResult)
md = MatchDisplay(sceneWindow)
md.setGeoMatchResult(matchResult, Point(int(dx), int(dy)))

## display the scene image
runMessageLoop("The Scene! Click the space bar to continue\n")

del md
del cd
del window
del sceneWindow
\endcode

\section gmm_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.gmm import *

import sys
import math

def printSelectedFeatures(model):
    sys.stderr.write("\nFeatures that are used for pattern matching:\n")

    ## Corners
    nCountCorners = model.enabledCornersCount

    sys.stderr.write("\nNumber of corners %d\n" % nCountCorners)

    if nCountCorners > 0:

        corners = model.enabledCorners

        for i in range(nCountCorners):
            corner = corners[i]
            sys.stderr.write("Corner %3d: Corner point: %3d/%3d, opening angle: %3d degrees\n" %
                (i, int(corner.cornerPoint.x), int(corner.cornerPoint.y), int(corner.openingAngle/math.pi*180.0)))

    ## Arcs
    nCountArcs = model.enabledArcsCount

    sys.stderr.write("\nNumber of arcs %d\n" % nCountArcs)

    if nCountArcs > 0:
        arcs = model.arcs

        for i in range(nCountArcs):
            arc = arcs[i]
            sys.stderr.write("Arc %3d: Center point: %d/%d, radius: %d\n" %
                (i, int(arc.center.x), int(arc.center.y), int(arc.radius)))

def printMatches(matchResult):
    nCountMatches = matchResult.numberOfMatches

    sys.stderr.write("\nNumber of matches: %d\n" % nCountMatches)

    for i in range(nCountMatches):
        currMatch = matchResult.getMatch(i)
        sys.stderr.write("\nMatch %d:\n" % i)
        sys.stderr.write("x translation: %3.2f\n" % currMatch.transX)
        sys.stderr.write("y translation: %3.2f\n" % currMatch.transY)
        sys.stderr.write("rotation:      %3.2f degrees\n" % (currMatch.rotation/math.pi*180.0))
        sys.stderr.write("scaling:       %3.2f\n" % currMatch.scale)
        sys.stderr.write("score:         %3.2f\n" % currMatch.score)

class CornerDisplay:
    def __init__(self, window):
        self.m_window = window
        self.m_corners = []
        self.m_loi1 = []
        self.m_loi2 = []

    def clear(self):
        self.m_corners = []
        self.m_loi1 = []
        self.m_loi2 = []

    def addCorners(self, corners):
        for currCorner in corners:
            self.m_corners.append(currCorner)

            newLoi1 = LineOfInterest(
                self.m_window,
                Line(
                    Point(int(currCorner.cornerPoint.x-5), int(currCorner.cornerPoint.y)),
                    Point(int(currCorner.cornerPoint.x+5), int(currCorner.cornerPoint.y))))
            newLoi1.color = (255, 0, 0)
            self.m_loi1.append(newLoi1)

            newLoi2 = LineOfInterest(
                self.m_window,
                Line(
                    Point(int(currCorner.cornerPoint.x), int(currCorner.cornerPoint.y-5)),
                    Point(int(currCorner.cornerPoint.x), int(currCorner.cornerPoint.y+5))))
            newLoi2.color = (255, 0, 0)
            self.m_loi2.append(newLoi2)

class MatchDisplay:

    def __init__(self, window):
        self.m_window = window
        self.m_gmr = GeoMatchResult()
        self.m_loi1 = []
        self.m_loi2 = []
        self.m_tloi = []
        self.m_window = window
        self.m_center = Point(0, 0)

    def clear(self):
        self.m_gmr = GeoMatchResult()
        self.m_loi1 = []
        self.m_loi2 = []
        self.m_tloi = []

    def setGeoMatchResult(self, r, center):
        self.clear()

        self.m_gmr = r
        self.m_center = center

        for i in range(self.m_gmr.numberOfMatches):
            currMatch = self.m_gmr.getMatch(i)

            newLoi1 = LineOfInterest(
                self.m_window,
                Line(
                    Point(
                        int(center.x + currMatch.transX - 5),
                        int(center.y + currMatch.transY)),
                    Point(
                        int(center.x + currMatch.transX + 5),
                        int(center.y + currMatch.transY))))

            newLoi1.color = (0, 255, 0)
            self.m_loi1.append(newLoi1)

            newLoi2 = LineOfInterest(
                self.m_window,
                Line(
                    Point(
                        int(center.x + currMatch.transX),
                        int(center.y + currMatch.transY - 5)),
                    Point(
                        int(center.x + currMatch.transX),
                        int(center.y + currMatch.transY + 5))))
            newLoi2.color = (0, 255, 0)

            newLoi2.text = "%d" % i
            newLoi2.textColor = (0, 0, 255)
            self.m_loi2.append(newLoi2)

            pt = Point(220, 20*i + 20)
            newTLoi = LineOfInterest(
                self.m_window,
                Line(pt, pt))

            text = "%d - tx: %3.2f ty:  %3.2f rot: %3.2f scale: %3.2f score: %3.2f " % (
                            i , currMatch.transX, currMatch.transY, currMatch.rotation/math.pi*180.0,
                                currMatch.scale*100., currMatch.score*100.
                            )
            newTLoi.text = text
            newTLoi.textColor = (0, 255, 255)
            self.m_tloi.append(newTLoi)

## main()

## Load model image
modelImage = loadImage("model.bmp")

## display the loaded image
window = Window(modelImage)
runMessageLoop("The Model! Click the space bar to continue\n")

## Model
geoModel = GeoModel(modelImage, 7, 30)

## Print all model features:
printSelectedFeatures(geoModel)
cd = CornerDisplay(window)
cd.addCorners(geoModel.enabledCorners)

window.redraw()
runMessageLoop("The 7 corners! Click the space bar to continue\n")
cd.clear()

## Read edge points of model
edgePointImage = geoModel.edgePoints

window.buffer = edgePointImage
window.redraw()
runMessageLoop("The Edge Points! Click the space bar to continue\n")
window.buffer = modelImage
window.redraw()

myModel = GeoModel(geoModel.width, geoModel.height)

myModel.edgePoints = geoModel.edgePoints
nCountCorners = geoModel.enabledCornersCount
nCountCopyCorners = nCountCorners

if nCountCopyCorners > 5:
    nCountCopyCorners = 5
corners = geoModel.enabledCorners[:nCountCopyCorners]

for corner in corners:
    myModel.addCorner(corner)

## Print selected 5 model features from copied model:
printSelectedFeatures(myModel)
cd.addCorners(myModel.enabledCorners)

window.redraw()
runMessageLoop("The 5 corners! Click the space bar to continue\n")
cd.clear()

## Save copied model to file "copiedmodel.gmm"
myModel.save("copiedmodel.gmm")

## Set rotation center to the middle of the model image
dx = geoModel.width/2.0
geoModel.rotationCenterX = dx
dy = geoModel.height/2.0
geoModel.rotationCenterY = dy

## generate Matcher object
matcher = GeoMatcher()

## Set some options for pattern matching. It is not obligatory to set options.
## You can also use default parameters.
matcher.minScore = 0.8
matcher.maxNumberOfMatches = mmMatcherFindAll

## Load scene image
sceneImage = loadImage("scene.bmp")

## Start pattern matching
matchResult = matcher.findGeoModel(geoModel, sceneImage)

sceneWindow = Window( sceneImage )
printMatches(matchResult)
md = MatchDisplay(sceneWindow)
md.setGeoMatchResult(matchResult, Point(int(dx), int(dy)))

## display the scene image
runMessageLoop("The Scene! Click the space bar to continue\n")

del md
del cd
del window
del sceneWindow

##</matrixdoc>
