#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page multithreshold_page Multiple thresholds example (Python)

\section multithreshold_section_purpose Purpose
This sample shows how to apply multiple thresholds simultaneously to an image.

\section multithreshold_section_intro What it does
This sample applies two thresholds to the input image and shows the resulting image
in different colors depending on the results.

\section multithreshold_section_description How it works

First the source image is loaded and displayed.

\code
#load and display image
image = loadImage("phone.bmp")
window = Window(image)
\endcode

Then a tuple of thresholds is created. The condition of the first one is true if
a pixel's value is less than 50; the condition of the second one is true if a pixel's value
is greater than 180.
\code
#setup threshold values
thresholds = (Threshold(50, pmLess), Threshold(180, pmGreater))
\endcode

Then the palette, i.e. the mapping, is generated from the thresholds. The mapping sets bit 0 of
the resulting image according to the condition of first threshold in the tuple, bit 1 according to
the second one and so on. Here there are only two thresholds and thus only the two least significant bits
are concerned. Actually the value 3 corresponding to the binary coding 11 does never occur, because it is
impossible for a pixel to be less than 50 as well as greater than 180.

\code
#calculate new palette and map through it
thresholdPal = generateThresholdPalette(thresholds, bdUChar)
\endcode

Afterwards, the mapping is applied to the image.

\code
image2 = mapLut(image, thresholdPal)
\endcode

Then there is an image where only three values occur: 0, 1 and 2. Thus with a normal grey palette it's impossible
to distinguish between the three values. So a color palette is created to display the values. The first entry
of the palette is set to black, the second to red and the third one to green. That means that values
which are neither less than 50 nor greater than 180 are displayed black, values which are less than 50 in the source
image are displayed red and values which are greater than 180 in the source image are displayed green.

Note that the three lists represent all (three) values for one single channel.
\code
#setup new palette for displaying the results
pal = Palette(256, 3)
setConstant(pal, 0)

values = ((0, 255, 0), # red   channel values for the three concerning entries
          (0, 0, 255), # green channel values for the three concerning entries
          (0, 0, 0  )) # blue  channel values for the three concerning entries

for band, rgb in zip(xrange(3), values):
    for index, value in zip(xrange(3), rgb):
        pal.setEntry(index, band, 0, 0, value)
\endcode

Finally the resulting window is displayed with the color palette constructed before. Afterwards
space is expected to be pressed by the user and the windows are destroyed.

\code
#display thresholded image and set the new palette
window2 = Window(image2)
window2.colorPalette = pal

#keep sample alive
runMessageLoop()

del window
del window2
\endcode

\section multithreshold_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.color import *

#load and display image
image = loadImage("phone.bmp")
window = Window(image)

#setup threshold values
thresholds = (Threshold(50, pmLess), Threshold(180, pmGreater))

#calculate new palette and map through it
thresholdPal = generateThresholdPalette(thresholds, bdUChar)
image2 = mapLut(image, thresholdPal)

#setup new palette for displaying the results
pal = Palette(256, 3)
setConstant(pal, 0)

values = ((0, 255, 0), # red   channel values for the three concerning entries
          (0, 0, 255), # green channel values for the three concerning entries
          (0, 0, 0  )) # blue  channel values for the three concerning entries

for band, rgb in zip(xrange(3), values):
    for index, value in zip(xrange(3), rgb):
        pal.setEntry(index, band, 0, 0, value)

#display thresholded image and set the new palette
window2 = Window(image2)
window2.colorPalette = pal

#keep sample alive
runMessageLoop()

del window
del window2

##</matrixdoc>
