#! /usr/bin/python

"""
/**
\\xmlonly
mvIMPACT OCR
\endxmlonly

\page ocr2_page Advanced OCR example (Python)

\section ocr2_section_purpose Purpose
This example illustrates how to recognize dotted characters from a can.

\section ocr2_section_intro What it does
The example shows the bottom of a can with the expiration date. At the beginning the sample makes some
processings to make the characters readable. Then it divides the text (including the characters) into
segments. If the segmentation was successfully, by using an OCR font database the OCR is executed on the image.

\section ocr2_section_description How it works

A class derived from Window is defined. In the constructor, the parent's constructor is called and
the member variable, i.e. the SegmentList object is initialized. Then a simple setter function for
the segment list is defined.

\code
class MyWindow(Window):

    #overwrite constructor
    def __init__(self, image):
        Window.__init__(self, image)
        self.mylist = None

    #used to update segment list
    def setSegmentList(self, list):
        self.mylist = list
        self.redraw()
\endcode

Afterwards the onDraw method is overwritten to draw the segments from the list into the window.

\code
    #called each time the window will be drawn
    def onDraw (self, dc, sourceRect, destRect):
        if self.mylist is not None:
            #draw the segments from the segment list
            color = RGBColor(0, 255, 0)
            for counter in range(self.mylist.size):
                spx = self.scrollPosX
                spy = self.scrollPosY
                rect = self.mylist.getSegment(counter)
                p0 = Point(
                    rect.left + xOffset - spx,
                    rect.top + yOffset - spy)
                p1 = Point(
                    rect.right + xOffset - spx,
                    rect.top + yOffset - spy)
                p2 = Point(
                    rect.right + xOffset - spx,
                    rect.bottom + yOffset - spy)
                p3 = Point(
                    rect.left + xOffset - spx,
                    rect.bottom + yOffset - spy)

                dc.drawLine(p0, p1, color)
                dc.drawLine(p1, p2, color)
                dc.drawLine(p2, p3, color)
                dc.drawLine(p3, p0, color)
\endcode

In the main program the input image is loaded and displayed in a window.

\code
#main program
#load and display image
image = loadImage("can.bmp")
window = MyWindow(image)
\endcode

The dotted characters of the can need a preprocessing. Therefore a child image and a user defined structuring
element representing are defined. After this the child image is eroded and the window is redrawn to display
the result of the preprocessing operation.

\code
#create child image offset:200/200 w/h:240/160
child = Image(image, xOffset, yOffset, 240, 160)

#create user defined structuring element for erosion:
se = StructuringElement(3, 3, 1, 1, 1, bdLong)

#set relevant pixels in structuring element to 1:
se.setValues(9 * [1.])

#perform erosion (only on child area)
erode(child, child, se)

#update window
window.redraw()
\endcode

The OCR font database is loaded and the properties of the segmentation are defined (these
settings can be used to optimize the speed of the complete OCR process).

\code
#load OCR font database
if os.getenv("VL_FILE_DIR"):
    ocrFontDB = OcrFontDatabase(os.getenv("VL_FILE_DIR") + "pattern.ocr")
else:
    ocrFontDB = OcrFontDatabase("pattern.ocr")

#set some properties for the segmentation
ocrFontDB.classificationType = ocCorrelation
ocrFontDB.threshold = 127
ocrFontDB.minCharWidth = 15
ocrFontDB.minCharHeight = 25
ocrFontDB.maxCharWidth = 30
ocrFontDB.maxCharHeight = 40

\endcode

Then a new SegmentList object is created and the image is segmented.

\code
#segment the child area
segmentList = SegmentList(256)
window.setSegmentList(segmentList)

ocrFontDB.segment(child, segmentList)

#force window update
window.redraw()
\endcode

The next step optimizes the segments by splitting, merging and sorting the characters.

\code
#wait for space bar
runMessageLoop("Press <space> to split\n")

#split large objects
segmentList.split(ocrFontDB, child)

#force window update
window.redraw()

#wait for space bar
runMessageLoop("Press <space> to merge\n")

#merge small objects
segmentList.merge(ocrFontDB, child)

#force window update
window.redraw()

#wait for space bar
runMessageLoop("Press <space> to sort\n")

#sort objects from left to right and from up to down
segmentList.sort(ocrFontDB, child)

#update reference in onDrawHandler
window.segmentList = segmentList

\endcode

After this the result is printed:

\code
#wait for space bar
runMessageLoop("Press <space> to read\n")

for rectCounter in range(segmentList.size):
    rect = segmentList.getSegment(rectCounter)
    result = ocrFontDB.readChar(child, rect)
    sys.stdout.write(chr(result.ascii))

#start new line
print
\endcode

Finally the message loop is run and the window is destroyed after the user has pressed space.

\code
#keep demo alive
runMessageLoop()

del window
\endcode

\section ocr2_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.ocr import *

import os
import sys

xOffset = 200
yOffset = 200

class MyWindow(Window):

    #overwrite constructor
    def __init__(self, image):
        Window.__init__(self, image)
        self.mylist = None

    #used to update segment list
    def setSegmentList(self, list):
        self.mylist = list
        self.redraw()

    #called each time the window will be drawn
    def onDraw (self, dc, sourceRect, destRect):
        if self.mylist is not None:
            #draw the segments from the segment list
            color = RGBColor(0, 255, 0)
            for counter in range(self.mylist.size):
                spx = self.scrollPosX
                spy = self.scrollPosY
                rect = self.mylist.getSegment(counter)
                p0 = Point(
                    rect.left + xOffset - spx,
                    rect.top + yOffset - spy)
                p1 = Point(
                    rect.right + xOffset - spx,
                    rect.top + yOffset - spy)
                p2 = Point(
                    rect.right + xOffset - spx,
                    rect.bottom + yOffset - spy)
                p3 = Point(
                    rect.left + xOffset - spx,
                    rect.bottom + yOffset - spy)

                dc.drawLine(p0, p1, color)
                dc.drawLine(p1, p2, color)
                dc.drawLine(p2, p3, color)
                dc.drawLine(p3, p0, color)

#main program
#load and display image
image = loadImage("can.bmp")
window = MyWindow(image)

#wait for key
runMessageLoop("Press <space> to preprocess image\n")

#create child image offset:200/200 w/h:240/160
child = Image(image, xOffset, yOffset, 240, 160)

#create user defined structuring element for erosion:
se = StructuringElement(3, 3)

#set relevant pixels in structuring element to 1:
se.setValues(9 * [1.])

#perform erosion (only on child area)
erode(child, child, se)

#update window
window.redraw()

#wait for space key
runMessageLoop("Press <space> to segment image\n")

#load OCR font database
if os.getenv("VL_FILE_DIR"):
    ocrFontDB = OcrFontDatabase(os.getenv("VL_FILE_DIR") + "pattern.ocr")
else:
    ocrFontDB = OcrFontDatabase("pattern.ocr")

#set some properties for the segmentation
ocrFontDB.classificationType = ocCorrelation
ocrFontDB.threshold = 127
ocrFontDB.minCharWidth = 15
ocrFontDB.minCharHeight = 25
ocrFontDB.maxCharWidth = 30
ocrFontDB.maxCharHeight = 40

#segment the child area
segmentList = SegmentList(256)
window.setSegmentList(segmentList)

ocrFontDB.segment(child, segmentList)

#force window update
window.redraw()

#wait for space bar
runMessageLoop("Press <space> to split\n")

#split large objects
segmentList.split(ocrFontDB, child)

#force window update
window.redraw()

#wait for space bar
runMessageLoop("Press <space> to merge\n")

#merge small objects
segmentList.merge(ocrFontDB, child)

#force window update
window.redraw()

#wait for space bar
runMessageLoop("Press <space> to sort\n")

#sort objects from left to right and from up to down
segmentList.sort(ocrFontDB, child)

#update reference in onDrawHandler
window.segmentList = segmentList

#wait for space bar
runMessageLoop("Press <space> to read\n")

for rectCounter in range(segmentList.size):
    rect = segmentList.getSegment(rectCounter)
    result = ocrFontDB.readChar(child, rect)
    sys.stdout.write(chr(result.ascii))

#start new line
print

#keep demo alive
runMessageLoop()

del window

#</matrixdoc>
