#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Match
\endxmlonly

\page patmatch_page Pattern matching example (Python)

\section patmatch_section_purpose Purpose
This example shows how to use the match module.

\section patmatch_section_intro What it does
Two rotation invariant models are defined in a model image. Then these models are searched for
in a scene image. Afterwards the two sets of points in the model image and in the scene image are
used to calculate a transformation from the scene image to the model image. This transformation is finally
applied to the scene image.

\section patmatch_section_description How it works

First of all some "constants" are set regarding the dimensions of the two models.

\code
model1Width = 42
model1Height = 45
model2Width = 56
model2Height = 54
\endcode

Then a helper function is defined which is used to sort the matches. This is required for
the final transformation to be calculated. The function sorts the given points by using their
distance to a reference point.

\code
#This function sorts the points given in vector "matches". The sorting
#criteria is the distance to the point given with parameter "measurePoint".
#The point nearest to "measurepoint" will be the first point in the result vector.
#Parameter:
#    matches:        the vector containing the unsorted points
#    measurePoint:    reference point for distance calculation
#
def sortPoints(matches, measurePoint):

    distance = []
    result = []
    #step through each point in the unsorted vector
    for point in matches:

        #get point coordinates and calculate distance to the reference point
        diffx = measurePoint.x - point.x
        diffy = measurePoint.y - point.y
        mydist = math.sqrt(diffx*diffx + diffy*diffy)

        #insert point into result vector (sorting by insertion)
        position = 0
        foundPosition = False
        if point == matches[0]:
            foundPosition = True

        #search for position to insert
        while foundPosition == False and position < len(distance):
            if mydist <= distance[position]:
                foundPosition = True
            else:
                position = position + 1

        distance.insert(position, mydist)
        result.insert(position, PointDouble(point.x, point.y))

    return result

\endcode

In the main program, first the reference image and the scene image are loaded from disk and
displayed each in its own window.

\code
#main program
#load reference image
referenceImage = loadImage("stanz.bmp")
window1 = Window(referenceImage)

#load distorted image
image2 = loadImage("stanz1.bmp")
window2 = Window(image2)
\endcode

Then the two models are defined each as a part of the reference image.

\code
#setup first model
model1 = PatModel(referenceImage, 314, 45, model1Width, model1Height)
model1.initialAcceptanceScore = 80
model1.finalAcceptanceScore = 70
model1.numberOfMatches = pmmFindAll

#setup second model
model2 = PatModel(referenceImage, 285, 385, model2Width, model2Height)
model2.initialAcceptanceScore = 80
model2.finalAcceptanceScore = 70
model2.numberOfMatches = 1

\endcode

Afterwards the first model is searched for in the reference image after the user
pressed space. All occurences of the model are marked with an AOI.

\code
#wait for space bar
runMessageLoop("Press <space> to search for model1 in reference image\n")

#search for model
matchResult = findPatModel(model1, referenceImage)
matches1 = matchResult.getMatches(0, 0, 0)

aoi1 = []
#mark all occurences of model1
for counter, aMatch in enumerate(matches1):
    aoi = AreaOfInterest(
            window1,
            Rectangle(
                Point(
                    long(aMatch.x - model1Width/2),
                    long(aMatch.y - model1Height/2)),
                Point(
                    long(aMatch.x + model1Width/2),
                    long(aMatch.y + model1Height/2))
                )
            )
    aoi.linesVisible = True
    aoi.textColor = (0, 255, 0)
    aoi.text = "Model 1: %d" % counter
    aoi1.append(aoi)

#redraw window and wait for space bar
window1.redraw()
\endcode

Afterwards the second model is searched for in the reference image. Again, all
occurences are marked with an AOI.

\code
runMessageLoop("Press <space> to start searching for model2 in reference image\n")

#search for model2 in reference image
matchResult = findPatModel(model2, referenceImage)
matches2 = matchResult.getMatches(0, 0, 0)

aoi2 = []
#mark all occurences of model2
for counter, aMatch in enumerate(matches2):
    aoi = AreaOfInterest(
            window1,
            Rectangle(
                Point(
                    long(aMatch.x - model2Width/2),
                    long(aMatch.y - model2Height/2)),
                Point(
                    long(aMatch.x + model2Width/2),
                    long(aMatch.y + model2Height/2))
                )
            )
    aoi.linesVisible = True
    aoi.textColor = (0, 255, 0)
    aoi.text = "Model 2: %d" % counter
    aoi2.append(aoi)

#redraw window and wait for space bar
window1.redraw()

\endcode

Then the first model is searched for in the scene image.

\code
runMessageLoop("Press <space> to search for model1 in distorted image\n")

#search for model1 in distorted image
matchResult = findPatModel(model1, image2)
matches3 = matchResult.getMatches(0, 0, 0)

aoi3 = []
#mark all occurences of model1
for counter, aMatch in enumerate(matches3):
    aoi = AreaOfInterest(
            window2,
            Rectangle(
                Point(
                    long(aMatch.x - model1Width/2),
                    long(aMatch.y - model1Height/2)),
                Point(
                    long(aMatch.x + model1Width/2),
                    long(aMatch.y + model1Height/2))
                )
            )
    aoi.linesVisible = True
    aoi.textColor = (0, 255, 0)
    aoi.text = "Model 1: %d" % counter
    aoi3.append(aoi)

#redraw window and wait for space bar
window2.redraw()

\endcode

Now the second model is searched for in the scene image.

\code
runMessageLoop("Press <space> to search for model2 in distorted image\n")

#search for model2 in distorted image
matchResult = findPatModel(model2, image2)
matches4 = matchResult.getMatches(0, 0, 0)

aoi4 = []
#mark all occurences of model2
for counter, aMatch in enumerate(matches4):
    aoi = AreaOfInterest(
            window2,
            Rectangle(
                Point(
                    long(aMatch.x - model2Width/2),
                    long(aMatch.y - model2Height/2)),
                Point(
                    long(aMatch.x + model2Width/2),
                    long(aMatch.y + model2Height/2))
                )
            )
    aoi.linesVisible = True
    aoi.textColor = (0, 255, 0)
    aoi.text = "Model 2: %d" % counter
    aoi4.append(aoi)

#redraw window and wait for space bar
window2.redraw()

\endcode

Afterwards the two point vectors representing occurences of the first model are sorted using the
helper function and the first (and only) occurence of the second model in the corresponding image as
the reference point for the sorting. Then the transformation matrix is calculated and the
scene image is transformed to be congruent to the reference image (apart from border effects).

\code
runMessageLoop("Press <space> to rotate image2\n")

#sort the points
referencePoints = sortPoints(matches1, matches2[0])
distortedPoints = sortPoints(matches3, matches4[0])

#calculate matrix
rotationMatrix = getMatrixPassPoint(referencePoints, distortedPoints)

#rotate and display image
rotatedImage = matrix(image2, rotationMatrix, giBilinear)
window3 = Window(rotatedImage)
\endcode

Finally the user has to press space to let the program destroy the windows.

\code
#keep sample alive
runMessageLoop()

del window1
del window2
del window3
\endcode

\section patmatch_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.match import *

import math

model1Width = 42
model1Height = 45
model2Width = 56
model2Height = 54

#This function sorts the points given in vector "matches". The sorting
#criteria is the distance to the point given with parameter "measurePoint".
#The point nearest to "measurepoint" will be the first point in the result vector.
#Parameter:
#    matches:        the vector containing the unsortet points
#    measurePoint:    reference point for distance calculation
#
def sortPoints(matches, measurePoint):

    distance = []
    result = []
    #step through each point in the unsortet vector
    for point in matches:

        #get point coordinates and calculate distance to the reference point
        diffx = measurePoint.x - point.x
        diffy = measurePoint.y - point.y
        mydist = math.sqrt(diffx*diffx + diffy*diffy)

        #insert point into result vector (sorting by insertion)
        position = 0
        foundPosition = False
        if point == matches[0]:
            foundPosition = True

        #search for position to insert
        while foundPosition == False and position < len(distance):
            if mydist <= distance[position]:
                foundPosition = True
            else:
                position = position + 1

        distance.insert(position, mydist)
        result.insert(position, PointDouble(point.x, point.y))

    return result

#main program
#load reference image
referenceImage = loadImage("stanz.bmp")
window1 = Window(referenceImage)

#load distorted image
image2 = loadImage("stanz1.bmp")
window2 = Window(image2)

#setup first model
model1 = PatModel(referenceImage, 314, 45, model1Width, model1Height)
model1.initialAcceptanceScore = 80
model1.finalAcceptanceScore = 70
model1.numberOfMatches = pmmFindAll

#setup second model
model2 = PatModel(referenceImage, 285, 385, model2Width, model2Height)
model2.initialAcceptanceScore = 80
model2.finalAcceptanceScore = 70
model2.numberOfMatches = 1

#wait for space bar
runMessageLoop("Press <space> to search for model1 in reference image\n")

#search for model
matchResult = findPatModel(model1, referenceImage)
matches1 = matchResult.getMatches(0, 0, 0)

aoi1 = []
#mark all occurences of model1
for counter, aMatch in enumerate(matches1):
    aoi = AreaOfInterest(
            window1,
            Rectangle(
                Point(
                    long(aMatch.x - model1Width/2),
                    long(aMatch.y - model1Height/2)),
                Point(
                    long(aMatch.x + model1Width/2),
                    long(aMatch.y + model1Height/2))
                )
            )
    aoi.linesVisible = True
    aoi.textColor = (0, 255, 0)
    aoi.text = "Model 1: %d" % counter
    aoi1.append(aoi)

#redraw window and wait for space bar
window1.redraw()
runMessageLoop("Press <space> to start searching for model2 in reference image\n")

#search for model2 in reference image
matchResult = findPatModel(model2, referenceImage)
matches2 = matchResult.getMatches(0, 0, 0)

aoi2 = []
#mark all occurences of model2
for counter, aMatch in enumerate(matches2):
    aoi = AreaOfInterest(
            window1,
            Rectangle(
                Point(
                    long(aMatch.x - model2Width/2),
                    long(aMatch.y - model2Height/2)),
                Point(
                    long(aMatch.x + model2Width/2),
                    long(aMatch.y + model2Height/2))
                )
            )
    aoi.linesVisible = True
    aoi.textColor = (0, 255, 0)
    aoi.text = "Model 2: %d" % counter
    aoi2.append(aoi)

#redraw window and wait for space bar
window1.redraw()
runMessageLoop("Press <space> to search for model1 in distorted image\n")

#search for model1 in distorted image
matchResult = findPatModel(model1, image2)
matches3 = matchResult.getMatches(0, 0, 0)

aoi3 = []
#mark all occurences of model1
for counter, aMatch in enumerate(matches3):
    aoi = AreaOfInterest(
            window2,
            Rectangle(
                Point(
                    long(aMatch.x - model1Width/2),
                    long(aMatch.y - model1Height/2)),
                Point(
                    long(aMatch.x + model1Width/2),
                    long(aMatch.y + model1Height/2))
                )
            )
    aoi.linesVisible = True
    aoi.textColor = (0, 255, 0)
    aoi.text = "Model 1: %d" % counter
    aoi3.append(aoi)

#redraw window and wait for space bar
window2.redraw()
runMessageLoop("Press <space> to search for model2 in distorted image\n")

#search for model2 in distorted image
matchResult = findPatModel(model2, image2)
matches4 = matchResult.getMatches(0, 0, 0)

aoi4 = []
#mark all occurences of model2
for counter, aMatch in enumerate(matches4):
    aoi = AreaOfInterest(
            window2,
            Rectangle(
                Point(
                    long(aMatch.x - model2Width/2),
                    long(aMatch.y - model2Height/2)),
                Point(
                    long(aMatch.x + model2Width/2),
                    long(aMatch.y + model2Height/2))
                )
            )
    aoi.linesVisible = True
    aoi.textColor = (0, 255, 0)
    aoi.text = "Model 2: %d" % counter
    aoi4.append(aoi)

#redraw window and wait for space bar
window2.redraw()
runMessageLoop("Press <space> to rotate image2\n")

#sort the points
referencePoints = sortPoints(matches1, matches2[0])
distortedPoints = sortPoints(matches3, matches4[0])

#calculate matrix
rotationMatrix = getMatrixPassPoint(referencePoints, distortedPoints)

#rotate and display image
rotatedImage = matrix(image2, rotationMatrix, giBilinear)
window3 = Window(rotatedImage)

#keep sample alive
runMessageLoop()

del window1
del window2
del window3

#</matrixdoc>
