#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page presetimg_page Preset image example (Python)

\section presetimg_section_purpose Purpose
This example shows how to preset and image with various patterns.

\section presetimg_section_intro What it does
This example demonstrates the patterns which can be used to preset an image.
The image is set to a constant value, a checker pattern, a wedge pattern, a ring pattern
and random noise.

\section presetimg_section_description How it works

First an empty (uninitialized) image is created and displayed.

\code
image = Image(512, 512)
wnd = Window(image)
\endcode

Then the image is presetted with constant values from 0 to 255 with a step width of
16.

\code
print "Presetting with constant value (ranging from 0 to 255)\n"

for i in range(0, 256, 16):
    presetConstantImage(image, i)
    wnd.redraw()
    dispatchMessages()

\endcode

Then the image is preset with a checkerboard pattern from 0 to 255 with a step width of
16. The other half of the board is initialized with <code>255-i</code> where \c i is the
value of the first half.

\code
print "Presetting with checkerboard pattern (ranging from 0 to 255)\n"

for i in range(0, 256, 16):
    presetCheckerImage(
                image,
                PresetColors(i, 255-i),
                PresetTransformation(32, 32))
    wnd.redraw()
    dispatchMessages()
\endcode

Then the image is preset with a checkerboard pattern at different roation angles ranging from
0 to 90 degrees at a step with of 6.

\code
print "Presetting with checkerboard pattern (rotating)\n"

for i in range(0, 90, 6):
    presetCheckerImage(
                image,
                PresetColors(0, 255),
                PresetTransformation(32, 32, 256, 256, i*math.pi/360))
    wnd.redraw()
    dispatchMessages()

\endcode

Afterwards the image is preset with a wedge pattern set with values
i between 0 and 256 and a step width of 16 for the first color and
255-i for the second color.

\code
print "Presetting with wedge pattern (ranging from 0 to 255)\n"

for i in range(0, 256, 16):
    presetWedgeImage(
                image,
                PresetColors(i, 255-i),
                PresetTransformation(64, 1))
    wnd.redraw()
    dispatchMessages()

\endcode

Afterwards the image is set with gaussian ring patterns ranging again from
0 to 256 with a step with of 16.

\code
print "Presetting with gaussian ring pattern (ranging from 0 to 255)\n"

for i in range(0, 256, 16):
    presetRingImage(
                image,
                PresetColors(i, 255-i),
                PresetTransformation(64, 64, 256, 256),
                pctPresetGauss)
    wnd.redraw()
    dispatchMessages()

\endcode

Then, a noise pattern is used as preset.

\code
print "Presetting with noise pattern\n"

for i in range(0, 256, 16):
    presetNoiseImage(
                image,
                PresetColors(0, 255),
                i,
                pctPresetGauss)
    wnd.redraw()
    dispatchMessages()
\endcode

Finally the a new color image is created and three child images each consisting of
one band of their parent. These images are then set with three different kinds of
preset values. The red channel is preset with a checker pattern, the green channel
with a wedge pattern and the blue channel with a noise pattern.

\code
color = Image(512, 512, 3)
wnd.buffer = color
red = Image(color, 0, 0, 0, color.width, color.height, 1)
green = Image(color, 0, 0, 1, color.width, color.height, 1)
blue = Image(color, 0, 0, 2, color.width, color.height, 1)

print "Now doing a bit of everything in a color image\n"

for i in range(0, 360, 18):
    presetCheckerImage(
                red,
                PresetColors(0, 255),
                PresetTransformation(32, 32, 256, 256, i*math.pi/360))
    presetWedgeImage(
                green,
                PresetColors(0, 255),
                PresetTransformation(64, 1, 256, 256, (360-i)*math.pi/360))
    presetNoiseImage(
                blue,
                PresetColors(0, 255),
                i,
                pctPresetGauss)

    wnd.redraw()
    dispatchMessages()
\endcode

Finally the user is requested to press space and the window is destroyed.

\code
## keep sample alive until console window is closed
## or space bar is pressed
runMessageLoop()

del wnd
\endcode

\section presetimg_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *

import math

image = Image(512, 512)
wnd = Window(image)

print "Presetting with constant value (ranging from 0 to 255)\n"

for i in range(0, 256, 16):
    presetConstantImage(image, i)
    wnd.redraw()
    dispatchMessages()

print "Presetting with checkerboard pattern (ranging from 0 to 255)\n"

for i in range(0, 256, 16):
    presetCheckerImage(
                image,
                PresetColors(i, 255-i),
                PresetTransformation(32, 32))
    wnd.redraw()
    dispatchMessages()

print "Presetting with checkerboard pattern (rotating)\n"

for i in range(0, 360, 18):
    presetCheckerImage(
                image,
                PresetColors(0, 255),
                PresetTransformation(32, 32, 256, 256, i*math.pi/360))
    wnd.redraw()
    dispatchMessages()

print "Presetting with wedge pattern (ranging from 0 to 255)\n"

for i in range(0, 256, 16):
    presetWedgeImage(
                image,
                PresetColors(i, 255-i),
                PresetTransformation(64, 1))
    wnd.redraw()
    dispatchMessages()

print "Presetting with gaussian ring pattern (ranging from 0 to 255)\n"

for i in range(0, 256, 16):
    presetRingImage(
                image,
                PresetColors(i, 255-i),
                PresetTransformation(64, 64, 256, 256),
                pctPresetGauss)
    wnd.redraw()
    dispatchMessages()

print "Presetting with noise pattern\n"

for i in range(0, 256, 16):
    presetNoiseImage(
                image,
                PresetColors(0, 255),
                i,
                pctPresetGauss)
    wnd.redraw()
    dispatchMessages()

color = Image(512, 512, 3)
wnd.buffer = color
red = Image(color, 0, 0, 0, color.width, color.height, 1)
green = Image(color, 0, 0, 1, color.width, color.height, 1)
blue = Image(color, 0, 0, 2, color.width, color.height, 1)

print "Now doing a bit of everything in a color image\n"

for i in range(0, 360, 18):
    presetCheckerImage(
                red,
                PresetColors(0, 255),
                PresetTransformation(32, 32, 256, 256, i*math.pi/360))
    presetWedgeImage(
                green,
                PresetColors(0, 255),
                PresetTransformation(64, 1, 256, 256, (360-i)*math.pi/360))
    presetNoiseImage(
                blue,
                PresetColors(0, 255),
                i,
                pctPresetGauss)

    wnd.redraw()
    dispatchMessages()

## keep sample alive until console window is closed
## or space bar is pressed
runMessageLoop()

del wnd

#</matrixdoc>
