#include <stdio.h>

#include <vl_base.h>
#include <vl_file.h>
#include <vl_gmm.h>

void translateError        (IPL_FRESULT err);
void printSelectedFeatures (GMM_MODELHANDLE hModel);
void setOptions            (GMM_MATCHERHANDLE hMather);
void printMatches          (GMM_MATCH* pMatches, IPL_ULONG nCountMatches);

#define CHECK_RESULT(X) \
	if ((X) == IPL_FALSE) \
	{ \
		printf("Statement failed:" #X "\n"); \
		translateError(IPL_GetErrorCode()); \
		abort(); \
	}
	
/*
* This is a sample how to use the GMM module.
* It demonstrates all basic functions using two images "model.bmp" and "scene.bmp".
*
* 1. Load model image
* 2. Load scene image
* 3. Create model description
* 4. Show selected corners and arcs
* 5. Set options for pattern matching
* 6. Run pattern matching
* 7. Show resulting transformations
*
*/
//------------------------------------------------------------------------------
int main()
//------------------------------------------------------------------------------
{
	GMM_MODELHANDLE   hModel;
	GMM_MATCHERHANDLE hMatcher;
	IPL_BUFHANDLE     hModelImage;
	IPL_BUFHANDLE     hSceneImage;

	// Load model image
	hModelImage = IPL_DONT_CARE;
	CHECK_RESULT (IPL_FlImport (&hModelImage, NULL, FL_DONT_CARE, "model.bmp"));
	
	// Load scene image
	hSceneImage = IPL_DONT_CARE;
	CHECK_RESULT (IPL_FlImport (&hSceneImage, NULL, FL_DONT_CARE, "scene.bmp"));
	
	// Model
	CHECK_RESULT (GMM_ModelAlloc(&hModel));
	CHECK_RESULT (GMM_ModelCreateFromImage(hModel, hModelImage, 8, 30));

	// Print model features used for pattern matching
	printSelectedFeatures(hModel);
	
	// ---
	// This is a good place to check what features and edge points 
	// were selected automatically. If you think the choice is
	// not optimal, you can modify the selection manually.
	// ---
	
	// Set rotation center to the middle of the model image
	IPL_DOUBLE d;
	
	// Define x coordinate of the rotation center (Default 0)
	IPL_ULONG nModelWidth;
	GMM_ModelInquire(hModel, MODEL_WIDTH, &nModelWidth);
	d = (IPL_DOUBLE)nModelWidth/2.0;
	CHECK_RESULT (GMM_ModelControl(hModel, MODEL_ROTATION_CENTER_X, &d));
	
	// Define y coordinate of the rotation center (Default 0)
	IPL_ULONG nModelHeight;
	GMM_ModelInquire(hModel, MODEL_HEIGHT, &nModelHeight);
	d = (IPL_DOUBLE)nModelHeight/2.0;
	CHECK_RESULT (GMM_ModelControl(hModel, MODEL_ROTATION_CENTER_Y, &d));
	
	// Get a new pattern matcher handle
	CHECK_RESULT (GMM_MatcherAlloc(&hMatcher));
	
	// Set options for pattern matching. It is not obligatory to set options.
	// You can also use default parameters.
	setOptions(hMatcher);
	
	// Start pattern matching
	CHECK_RESULT (GMM_MatcherFindModel(hMatcher, hModel, hSceneImage));

	// Get matches
	GMM_MATCH* pMatches;
	IPL_ULONG  nCountMatches;
	CHECK_RESULT(GMM_MatcherGetMatches(hMatcher, &pMatches, &nCountMatches));

	// Print matches
	printMatches(pMatches, nCountMatches);
	
	// Free
	CHECK_RESULT (GMM_ModelFree(hModel));
	if (nCountMatches > 0)
	{
		CHECK_RESULT (GMM_MatcherFreeMatches(hMatcher, pMatches));
	}
	CHECK_RESULT (GMM_MatcherFree(hMatcher));
	
	CHECK_RESULT (IPL_BufFree(hSceneImage));
	CHECK_RESULT (IPL_BufFree(hModelImage));

	printf("\nPress any key to quit\n");
	IPL_CheckForInput(NULL, IPL_TRUE, IPL_FALSE, NULL);
}

//------------------------------------------------------------------------------
void printSelectedFeatures(GMM_MODELHANDLE hModel)
//------------------------------------------------------------------------------
{
	printf("\nFeatures that are used for pattern matching:\n");
	
	// ----- Corners ----- //
	IPL_ULONG          nCountCorners;
	GMM_FEATUREHANDLE* pCornerList;
	
	CHECK_RESULT (GMM_ModelCountCorners(hModel, &nCountCorners));
	printf("\nNumber of corners %ld\n", nCountCorners);
	
	if (nCountCorners > 0)
	{
		pCornerList = new GMM_FEATUREHANDLE[nCountCorners];
		CHECK_RESULT (GMM_ModelGetCornerHandles(hModel, pCornerList));
		
		IPL_ULONG   i;
		GMM_CORNER  corner;
		IPL_DOUBLE   PI = 3.14159265358979323844f;
		
		for (i = 0; i < nCountCorners; i++)
		{
			CHECK_RESULT (GMM_ModelGetCorner(hModel, pCornerList[i], &corner));
			printf ("Corner %ld: \n", i);
			printf ("Corner point: %d/%d, opening angle: %d\n", (int)corner.x, (int)corner.y, (int)(corner.openingAngle/PI*180.0f));
		}
		
		delete[] pCornerList;
	}
	
	// ----- Arcs ----- //
	IPL_ULONG          nCountArcs;
	GMM_FEATUREHANDLE* pArcList;
	
	CHECK_RESULT (GMM_ModelCountArcs(hModel, &nCountArcs));
	printf("\nNumber of arcs %ld\n", nCountArcs);
	
	if (nCountArcs > 0)
	{
		pArcList = new GMM_FEATUREHANDLE[nCountArcs];
		CHECK_RESULT (GMM_ModelGetArcHandles(hModel, pArcList));
		
		IPL_ULONG   i;
		GMM_ARC     arc;
		
		for (i = 0; i < nCountArcs; i++)
		{
			CHECK_RESULT (GMM_ModelGetArc(hModel, pArcList[i], &arc));
			printf ("Arc %ld: \n", i);
			printf ("Center point: %d/%d, radius: %d\n", (int)arc.x, (int)arc.y, (int)arc.radius);
		}

		delete[] pArcList;
	}
}

//------------------------------------------------------------------------------
void setOptions(GMM_MATCHERHANDLE hMatcher)
//------------------------------------------------------------------------------
{
	IPL_DOUBLE f;
	IPL_LONG  n;

	// Minimum score to accept a match (Default 0.7)
	// You should reduce it the image quality is bad or
	// if the searched object is occluded.
	f = 0.8f;
	CHECK_RESULT (GMM_MatcherControl(hMatcher, MATCHER_MIN_SCORE,  &f));
	
	n = GMM_MATCHER_FIND_ALL;
	CHECK_RESULT (GMM_MatcherControl(hMatcher, MATCHER_NUMBER_OF_MATCHES, &n));
}

//------------------------------------------------------------------------------
void printMatches
(
	GMM_MATCH*        pMatches, 
	IPL_ULONG         nCountMatches
)
//------------------------------------------------------------------------------
{
	printf("\nNumber of matches: %ld\n", nCountMatches);

	IPL_ULONG i;
	IPL_DOUBLE PI = 3.14159265358979323844f;
	for (i = 0; i < nCountMatches; i++)
	{
		printf("\nMatch %ld: \n", i);
		printf("x translation: %3.2f\n", pMatches[i].x);
		printf("y translation: %3.2f\n", pMatches[i].y);
		printf("rotation:      %3.2f\n", (pMatches[i].r/PI*180.0f));
		printf("scaling:       %3.2f\n", pMatches[i].s);
		printf("score:         %3.2f\n", pMatches[i].score);
	}
}

#define WRITE_CASE(CONSTANT) \
	case CONSTANT: printf("Error: " #CONSTANT " (%d)\n", (int)err); break

//------------------------------------------------------------------------------
void translateError (IPL_FRESULT err)
//------------------------------------------------------------------------------
{
	switch (err)
	{
		WRITE_CASE(IPL_NOT_SPECIFIED);
		WRITE_CASE(IPL_MEMORY_ERROR);
		WRITE_CASE(IPL_ALLOCATION_ERROR);
		WRITE_CASE(IPL_HANDLE_ERROR);
		WRITE_CASE(IPL_INVALID_PARAM_ERROR);
		WRITE_CASE(GMM_WRONG_IMAGE_SIZE);
		WRITE_CASE(GMM_OUT_OF_BOUNDS);
		WRITE_CASE(GMM_INTERNAL_ERROR);
		WRITE_CASE(GMM_NO_EIGHT_BIT_IMAGE);
		WRITE_CASE(GMM_NO_MODEL_IMAGE);
		WRITE_CASE(GMM_MODEL_NOT_READY);
		WRITE_CASE(GMM_FILE_NOT_FOUND);
		WRITE_CASE(GMM_FEATURE_NOT_FOUND);
		WRITE_CASE(GMM_IMAGE_TOO_SMALL);
		WRITE_CASE(GMM_MODEL_DESCRIPTION_CORRUPT);
		WRITE_CASE(GMM_BAD_OPTION_KEY);
		WRITE_CASE(GMM_MODEL_NOT_CREATED);
		WRITE_CASE(GMM_INVALID_CONTROL_PARAM);
		WRITE_CASE(GMM_READ_ONLY_PARAMETER);
		default: printf("Unknown error: %d\n", (int)err); break;
	}
}
