#include <system.h>

#include <stdio.h>
#include <math.h>

#include <vl_base.h>
#include <vl_file.h>
#include <vl_display.h>
#include <vl_pat.h>
#include <vl_ipl.h>

#include <samples.h>
// Functionality:
//
// It loads two images: one reference image (first image) and one distorted image (second image). The
// distorted image is displaced (rotated and at another position) with respect to the reference image.  
// The goal is, to rotate and shift the distorted image in such a way, that it is located at the same
// position as the reference image. To achieve this, a pattern matching mechanism is used.
//
// The example works as follows:
//
// - Loads two images, one reference image and one distorted image.
// - Defines two model types: one model representing small circles the other one representing the circle 
//                            with the next bigger diameter (master model)
// - Searches for the model with the smaller circles in the reference image (3 hits expected)
// - Searches for the two defined models in the distorted image (3 hits expected for the smaller circles
//   and one hit expected for the bigger circle).
// - The positions of the models represent points,that are used
//   to do a pass point transformation. We select 3 points (master model and two other models, that 
//   have the greatest distance to the master model) from each image.
// - once we have the matrix from the previous step, we transform the distorted image.
//

int main( int /*argc*/, char * /*argv[]*/ )
{

	// load any supported file format, into image
	IPL_BUFHANDLE image1 = IPL_DONT_CARE;
	CHECKSDKCALL(IPL_FlImport(&image1, NULL, FL_DONT_CARE, "stanz.bmp"));

	IPL_BUFHANDLE image2 = IPL_DONT_CARE;
	CHECKSDKCALL(IPL_FlImport(&image2, NULL, FL_DONT_CARE, "stanz1.bmp"));

	// open window to display image
	IPL_HWND hwnd = 0;
	CHECKSDKCALL(DISP_OpenWindow(image1, &hwnd, 0, NULL));

	//allocate model buffer
	IPL_BUFHANDLE model;
	IPL_LONG widthmodel = 42;
	IPL_LONG heightmodel = 45;
    CHECKSDKCALL(PAT_BufAlloc(&model, image1, 314, 45, widthmodel, heightmodel, PAT_MODEL));

    //set search parameters
    IPL_DOUBLE acceptancelevel = 70.0;
    CHECKSDKCALL(PAT_SetControl(model, PAT_ACCEPTANCE_LEVEL, &acceptancelevel));
    IPL_DOUBLE certaintylevel = 80.0;
    CHECKSDKCALL(PAT_SetControl(model, PAT_CERTAINTY_LEVEL, &certaintylevel));
    IPL_LONG expectedmatches = 3;
    CHECKSDKCALL(PAT_SetControl(model, PAT_NB_MATCHES, &expectedmatches));

	//allocate a second model buffer (master model)
	IPL_BUFHANDLE model2;
	IPL_LONG widthmodel2 = 56;
	IPL_LONG heightmodel2 = 54;
	IPL_LONG xposmodel2 = 285;
	IPL_LONG yposmodel2 = 385;
    CHECKSDKCALL(PAT_BufAlloc(&model2, image1, xposmodel2, yposmodel2, widthmodel2, heightmodel2, PAT_MODEL));

    //set search parameters
    IPL_DOUBLE acceptancelevel2 = 70.0;
    CHECKSDKCALL(PAT_SetControl(model2, PAT_ACCEPTANCE_LEVEL, &acceptancelevel2));
    IPL_DOUBLE certaintylevel2 = 80.0;
    CHECKSDKCALL(PAT_SetControl(model2, PAT_CERTAINTY_LEVEL, &certaintylevel2));
    IPL_LONG expectedmatches2 = 1;
    CHECKSDKCALL(PAT_SetControl(model2, PAT_NB_MATCHES, &expectedmatches2));

    //console message 
	CHECKSDKCALL(DISP_KeepRunning("Hit space bar to start searching for models in reference buffer.\n"));

    //start search in reference image and get result buffer handle
    PAT_BUFHANDLE resultbuf = PAT_DONT_CARE;
    CHECKSDKCALL(PAT_SearchModel(image1, model, &resultbuf));

    //get number of actual matches in reference image
	IPL_LONG nbmatches=0;
    CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, 0, PAT_NB_MATCHES_FOUND, &nbmatches));

	// create an ooiset
	DISP_OOISETHANDLE oois = DISP_DONT_CARE;
	CHECKSDKCALL(DISP_OoiSetAlloc(&oois));

    //get results from matches in reference image
	IPL_POINT_DOUBLE pointsref[3];
    IPL_POINT aoipoints[5];
    DISP_OOIHANDLE aoi;

	if (expectedmatches == nbmatches)
	{
		for (long m=0; m < nbmatches; m++)
		{
		 IPL_DOUBLE score=0,xpos=0,ypos=0;
		 CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, m, PAT_SCORE, &score));
		 CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, m, PAT_XPOS, &xpos));
		 CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, m, PAT_YPOS, &ypos));

		 //save reference points
		 pointsref[m].xpos = xpos;
		 pointsref[m].ypos = ypos;

		 //generate aoi point set
		 aoipoints[0].xpos = (IPL_LONG)xpos-widthmodel/2;
		 aoipoints[0].ypos = (IPL_LONG)ypos-heightmodel/2;
		 aoipoints[1].xpos = (IPL_LONG)xpos+widthmodel/2;
		 aoipoints[1].ypos = (IPL_LONG)ypos-heightmodel/2;
		 aoipoints[2].xpos = (IPL_LONG)xpos+widthmodel/2;
		 aoipoints[2].ypos = (IPL_LONG)ypos+heightmodel/2;
		 aoipoints[3].xpos = (IPL_LONG)xpos-widthmodel/2;
		 aoipoints[3].ypos = (IPL_LONG)ypos+heightmodel/2;
		 aoipoints[4].xpos = (IPL_LONG)xpos-widthmodel/2;
		 aoipoints[4].ypos = (IPL_LONG)ypos-heightmodel/2;

		 //create ooi 
		 aoi = DISP_DONT_CARE;
		 CHECKSDKCALL(DISP_OoiAlloc(oois, &aoi, DISP_AOI_TOOL, aoipoints, sizeof(aoipoints)/sizeof(aoipoints[0]),NULL));

		 // set some ooi properties
		 COLORREF color = PALETTERGB(0, 255, 0);
		 CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_COLOR, &color));
		 char text[32];
		 sprintf(text,"M %ld",m+1);
		 CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT, text));
		 COLORREF textcolor = PALETTERGB(255, 0, 0);
		 CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT_COLOR, &textcolor));
		}
	}

    //free result buffer
    CHECKSDKCALL(PAT_BufFree(resultbuf));

	// associate the ooiset with the thread (and all windows created thereafter)
	CHECKSDKCALL(DISP_ControlWindow(DISP_OOISET, hwnd, &oois));
	CHECKSDKCALL(DISP_UpdateWindow(hwnd, DISP_INVALIDATE));

    //console message
	CHECKSDKCALL(DISP_KeepRunning("Hit space bar to start searching for models in distorted buffer.\n"));

	//open second window
	IPL_HWND hwnd2 = 0;
	CHECKSDKCALL(DISP_OpenWindow(image2, &hwnd2, 0, NULL));

    //start search in second image and get result buffer handle
    resultbuf = PAT_DONT_CARE;
    CHECKSDKCALL(PAT_SearchModel(image2, model, &resultbuf));

    //get number of actual matches in second image
	nbmatches=0;
    CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, 0, PAT_NB_MATCHES_FOUND, &nbmatches));

	// create an ooiset
	DISP_OOISETHANDLE oois2 = DISP_DONT_CARE;
	CHECKSDKCALL(DISP_OoiSetAlloc(&oois2));

    //get results
	IPL_POINT_DOUBLE pointsdist[3];
	if (expectedmatches == nbmatches)
	{
		for (long m=0; m < nbmatches; m++)
		{
		 IPL_DOUBLE score=0,xpos=0,ypos=0;
		 CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, m, PAT_SCORE, &score));
		 CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, m, PAT_XPOS, &xpos));
		 CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, m, PAT_YPOS, &ypos));

		 //save points of distorted image
		 pointsdist[m].xpos = xpos;
		 pointsdist[m].ypos = ypos;

		 //generate aoi point set
		 aoipoints[0].xpos = (IPL_LONG)xpos-widthmodel/2;
		 aoipoints[0].ypos = (IPL_LONG)ypos-heightmodel/2;
		 aoipoints[1].xpos = (IPL_LONG)xpos+widthmodel/2;
		 aoipoints[1].ypos = (IPL_LONG)ypos-heightmodel/2;
		 aoipoints[2].xpos = (IPL_LONG)xpos+widthmodel/2;
		 aoipoints[2].ypos = (IPL_LONG)ypos+heightmodel/2;
		 aoipoints[3].xpos = (IPL_LONG)xpos-widthmodel/2;
		 aoipoints[3].ypos = (IPL_LONG)ypos+heightmodel/2;
		 aoipoints[4].xpos = (IPL_LONG)xpos-widthmodel/2;
		 aoipoints[4].ypos = (IPL_LONG)ypos-heightmodel/2;

		 //create ooi
		 aoi = DISP_DONT_CARE;
		 CHECKSDKCALL(DISP_OoiAlloc(oois2, &aoi, DISP_AOI_TOOL, aoipoints, sizeof(aoipoints)/sizeof(aoipoints[0]),NULL));

		 // set some ooi properties
		 COLORREF color = PALETTERGB(0, 255, 0);
		 CHECKSDKCALL(DISP_ControlOoi(oois2, aoi, DISP_OOI_COLOR, &color));
		 char text[32];
		 sprintf(text,"M %ld",m+1);
		 CHECKSDKCALL(DISP_ControlOoi(oois2, aoi, DISP_OOI_TEXT, text));
		 COLORREF textcolor = PALETTERGB(255, 0, 0);
		 CHECKSDKCALL(DISP_ControlOoi(oois2, aoi, DISP_OOI_TEXT_COLOR, &textcolor));
		}
	}

    //free result
    CHECKSDKCALL(PAT_BufFree(resultbuf));

    //start search with model2 (master model) in second image and get result buffer handle
    resultbuf = PAT_DONT_CARE;
    CHECKSDKCALL(PAT_SearchModel(image2, model2, &resultbuf));

    //get number of actual matches
	nbmatches=0;
    CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, 0, PAT_NB_MATCHES_FOUND, &nbmatches));

    //get results
	IPL_POINT_DOUBLE point;
	if (nbmatches)
	{
		 IPL_DOUBLE score=0,xpos=0,ypos=0;
		 CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, 0, PAT_SCORE, &score));
		 CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, 0, PAT_XPOS, &xpos));
		 CHECKSDKCALL(PAT_GetResultBufData(resultbuf, 0, 0, 0, 0, PAT_YPOS, &ypos));

		 //save reference points
		 point.xpos = xpos;
		 point.ypos = ypos;

		 //generate aoi point set
		 aoipoints[0].xpos = (IPL_LONG)xpos-widthmodel2/2;
		 aoipoints[0].ypos = (IPL_LONG)ypos-heightmodel2/2;
		 aoipoints[1].xpos = (IPL_LONG)xpos+widthmodel2/2;
		 aoipoints[1].ypos = (IPL_LONG)ypos-heightmodel2/2;
		 aoipoints[2].xpos = (IPL_LONG)xpos+widthmodel2/2;
		 aoipoints[2].ypos = (IPL_LONG)ypos+heightmodel2/2;
		 aoipoints[3].xpos = (IPL_LONG)xpos-widthmodel2/2;
		 aoipoints[3].ypos = (IPL_LONG)ypos+heightmodel2/2;
		 aoipoints[4].xpos = (IPL_LONG)xpos-widthmodel2/2;
		 aoipoints[4].ypos = (IPL_LONG)ypos-heightmodel2/2;

		 //create ooi
		 aoi = DISP_DONT_CARE;
		 CHECKSDKCALL(DISP_OoiAlloc(oois2, &aoi, DISP_AOI_TOOL, aoipoints, sizeof(aoipoints)/sizeof(aoipoints[0]),NULL));

		 // set some ooi properties
		 COLORREF color = PALETTERGB(0, 255, 0);
		 CHECKSDKCALL(DISP_ControlOoi(oois2, aoi, DISP_OOI_COLOR, &color));
		 char text[32];
		 sprintf(text,"MR %d",1);
		 CHECKSDKCALL(DISP_ControlOoi(oois2, aoi, DISP_OOI_TEXT, text));
		 COLORREF textcolor = PALETTERGB(255, 0, 0);
		 CHECKSDKCALL(DISP_ControlOoi(oois2, aoi, DISP_OOI_TEXT_COLOR, &textcolor));
	} else {
		 point.xpos = point.ypos = 0; // prevent uninitialized use
	}

    //free result
    CHECKSDKCALL(PAT_BufFree(resultbuf));

	//we must sort the points based on some criteria (here distance) before we pass them
	//to the pass point transformation

	//get center of master model
	IPL_POINT_DOUBLE pointsmaster[3];
	pointsmaster[0].xpos = (widthmodel2 - 1)/2 + xposmodel2;
	pointsmaster[0].ypos = (heightmodel2 - 1)/2 + yposmodel2;

	//find point with shortest distance to center of master model
	double diffmin = 0xFFFFFFFF;
	int s;
	for (s=0; s < 3; s++)
	{
        double diffx = pointsmaster[0].xpos - pointsref[s].xpos;
        double diffy = pointsmaster[0].ypos - pointsref[s].ypos;
		double dist = sqrt(diffx*diffx + diffy*diffy);

		if (dist > 200 && dist < diffmin)
		{
			diffmin = dist;
			pointsmaster[1].xpos = pointsref[s].xpos;
			pointsmaster[1].ypos = pointsref[s].ypos;
		}
	}

	//find point with longest distance to center of master model
	double diffmax = 0.;
	for (s=0; s < 3; s++)
	{
        double diffx = pointsmaster[0].xpos - pointsref[s].xpos;
        double diffy = pointsmaster[0].ypos - pointsref[s].ypos;

		if (sqrt(diffx*diffx + diffy*diffy) > diffmax)
		{
			diffmax = sqrt(diffx*diffx + diffy*diffy);
			pointsmaster[2].xpos = pointsref[s].xpos;
			pointsmaster[2].ypos = pointsref[s].ypos;
		}
	}

	//get center of master model from second image
	IPL_POINT_DOUBLE pointsdistorted[3];
	pointsdistorted[0].xpos = point.xpos;
	pointsdistorted[0].ypos = point.ypos;

	//find point with shortest distance to center of master model
	diffmin = 0xFFFFFFFF;
	for (s=0; s < 3; s++)
	{
        double diffx = pointsdistorted[0].xpos - pointsdist[s].xpos;
        double diffy = pointsdistorted[0].ypos - pointsdist[s].ypos;
		double dist = sqrt(diffx*diffx + diffy*diffy);

		if (dist > 200 && dist < diffmin)
		{
			diffmin = dist;
			pointsdistorted[1].xpos = pointsdist[s].xpos;
			pointsdistorted[1].ypos = pointsdist[s].ypos;
		}
	}

	//find point with longest distance to center of master model
	diffmax = 0.;
	for (s=0; s < 3; s++)
	{
        double diffx = pointsdistorted[0].xpos - pointsdist[s].xpos;
        double diffy = pointsdistorted[0].ypos - pointsdist[s].ypos;

		if (sqrt(diffx*diffx + diffy*diffy) > diffmax)
		{
			diffmax = sqrt(diffx*diffx + diffy*diffy);
			pointsdistorted[2].xpos = pointsdist[s].xpos;
			pointsdistorted[2].ypos = pointsdist[s].ypos;
		}
	}



    //based on the points from the pattern matching process, we create a matrix from pass points
	IPL_DOUBLE ppmatrix[9];
    CHECKSDKCALL(IPL_TrGetMatrixPassPoint(pointsmaster,pointsdistorted,3,ppmatrix));

	// associate the ooiset with the thread (and all windows created thereafter)
	CHECKSDKCALL(DISP_ControlWindow(DISP_OOISET, hwnd2, &oois2));
	CHECKSDKCALL(DISP_UpdateWindow(hwnd2, DISP_INVALIDATE));

    //console message
	CHECKSDKCALL(DISP_KeepRunning("Hit space bar to rotate distorted image to reference image position.\n"));

    //do transformation with matrix from pass points
    CHECKSDKCALL(IPL_TrMatrix(image2,&image2,&ppmatrix[0],IPL_BILINEAR));

	//remove ooi set from second image
	DISP_OOISETHANDLE ooihandle = IPL_DONT_CARE;
	CHECKSDKCALL(DISP_ControlWindow(DISP_OOISET, hwnd2, &ooihandle));

	//update image
	CHECKSDKCALL(DISP_UpdateWindow(hwnd2, DISP_INVALIDATE));

    //free model buffers
    CHECKSDKCALL(PAT_BufFree(model));
    CHECKSDKCALL(PAT_BufFree(model2));

	// keep sample alive until console window is closed or space bar is pressed
	CHECKSDKCALL(DISP_KeepRunning("Click the space bar to end program.\n"));


	// free allocated buffers
	CHECKSDKCALL(IPL_BufFreeAll());

	return 0;
}
