/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page aoi2_page AOI2 example (CPP)

\section aoi2_section_purpose Purpose
The AOI2 example shows, how the new objects of interests can be used. Ellipse, 
polygon and polyline have been added to the existing set.

\section aoi2_section_intro What it does
This sample shows how to create different objects of interest (an area, an ellipse, a line, a polyline and a polygon) 
and how to change some properties. On top of the image the ObjectsOfInterest are displayed.

\section aoi2_section_description How it works
First of all an image is loaded into memory and a window is created to show
the image.

\code 
		// create new ImageBuffer
		Image image = loadImage("screw.bmp");
		Window window = Window(image);

\endcode

Then an AreaOfInterest object is created, which is associated to \c window.
The corresponding area of interest is then shown in this window. In the constructor a rectangle is defined, that
specifies the area where the AreaOfInterest should reside.

\code
		// create first area of interest and set some properties
		AreaOfInterest aoi = AreaOfInterest(window,Rectangle(10, 10, 110, 110));
\endcode

Afterwards some properties for the created AreaOfInterest object are set. The color of the AraOfInterest
object as well as the text and the text color is set. 

\code
		aoi.setColor(RGBColor(0, 255, 0));
		aoi.setText("Rectangle");
		aoi.setTextColor(RGBColor(255, 0, 0));
\endcode

Then an EllipseOfInterest object is created, similar to the first object.

\code
		// create ellipse of interest and set some properties
		EllipseOfInterest eoi = EllipseOfInterest(window, mvIMPACT::Rectangle(130, 10, 270, 110));
		eoi.setColor(RGBColor(255, 0, 0));
		eoi.setText("Ellipse");
		eoi.setTextColor(RGBColor(255, 0, 0));
\endcode

Afterwards, a LineOfInterest object is created. A LineOfInterest object has nearly the same interface than an AreaOfInterest. The 
constructor differs in the way, that it expects two points, that make up a line instead of a rectangle.

\code
		// create line of interest and set some properties
		LineOfInterest loi = LineOfInterest(window,Line(Point(290, 10),Point(290, 110)));
		loi.setColor(RGBColor(0, 0, 255)); 
		loi.setText("Line");
		loi.setTextColor(RGBColor(255, 0, 0));
\endcode


Then an PolyLineOfInterest object is created. The constructor expects an array of points that define the polygon. 
\code
		// create polyline of interest and set some properties
		std::vector<Point> points_ploi;
		points_ploi.push_back(Point(320,110));
		points_ploi.push_back(Point(310,50));
		points_ploi.push_back(Point(330,10));
		points_ploi.push_back(Point(410,10));
		points_ploi.push_back(Point(430,50));
		points_ploi.push_back(Point(420,110));
		PolylineOfInterest ploi = PolylineOfInterest(window, points_ploi);
		ploi.setColor(RGBColor(107, 142, 35)); 
		ploi.setText("Polyline");
		ploi.setTextColor(RGBColor(255, 0, 0));
\endcode

Then an PolygonOfInterest object is created, similar to the PolyLineOfInterest object.
\code
		// create polygon of interest and set some properties
		std::vector<Point> points_pgoi;
		Point p;
		for (int a=180; a<=360; a+=20)
		{
			p.setX( int( 490 + 50 * cos(a/180.*M_PI) ) );
			p.setY( int( 60 + 50 * sin(a/180.*M_PI) ) );
			points_pgoi.push_back(p);
		}
		points_pgoi.push_back(Point(490,110));
		
		PolygonOfInterest pgoi = PolygonOfInterest(window, points_pgoi);
		pgoi.setColor(RGBColor(255, 215, 0)); 
		pgoi.setText("Polygon");
		pgoi.setTextColor(RGBColor(255, 0, 0));
\endcode
Finally, the message loop of the windowing system is called. Now you can
interactively change the position and size (or length respectively) of the
generated objects of interest. Finally, press space to return from
the message loop function.

\code
		runMessageLoop();
\endcode

\section aoi2_section_source Source file

<matrixdoc>
*/



#include "mvIMPACT.h"
#include <vector>
#include <math.h>
using namespace std;
#ifndef M_PI
#	define M_PI       3.14159265358979323846
#endif 


int main(int , char*[] )
{
	try
	{
		// create new ImageBuffer
		Image image = loadImage("screw.bmp");
		Window window = Window(image);

		// create area of interest and set some properties
		AreaOfInterest aoi = AreaOfInterest(window, mvIMPACT::Rectangle(10, 10, 110, 110));
		aoi.setColor(RGBColor(0, 255, 0));
		aoi.setText("Rectangle");
		aoi.setTextColor(RGBColor(255, 0, 0));

		// create ellipse of interest and set some properties
		EllipseOfInterest eoi = EllipseOfInterest(window, mvIMPACT::Rectangle(130, 10, 270, 110));
		eoi.setColor(RGBColor(255, 0, 0));
		eoi.setText("Ellipse");
		eoi.setTextColor(RGBColor(255, 0, 0));

		// create line of interest and set some properties
		LineOfInterest loi = LineOfInterest(window,Line(Point(290, 10),Point(290, 110)));
		loi.setColor(RGBColor(0, 0, 255)); 
		loi.setText("Line");
		loi.setTextColor(RGBColor(255, 0, 0));

		// create polyline of interest and set some properties
		std::vector<Point> points_ploi;
		points_ploi.push_back(Point(320,110));
		points_ploi.push_back(Point(310,50));
		points_ploi.push_back(Point(330,10));
		points_ploi.push_back(Point(410,10));
		points_ploi.push_back(Point(430,50));
		points_ploi.push_back(Point(420,110));
		PolylineOfInterest ploi = PolylineOfInterest(window, points_ploi);
		ploi.setColor(RGBColor(107, 142, 35)); 
		ploi.setText("Polyline");
		ploi.setTextColor(RGBColor(255, 0, 0)); 


		// create polygon of interest and set some properties
		std::vector<Point> points_pgoi;
		Point p;
		for (int a=180; a<=360; a+=20)
		{
			p.setX( int( 490 + 50 * cos(a/180.*M_PI) ) );
			p.setY( int( 60 + 50 * sin(a/180.*M_PI) ) );
			points_pgoi.push_back(p);
		}
		points_pgoi.push_back(Point(490,110));
		
		PolygonOfInterest pgoi = PolygonOfInterest(window, points_pgoi);
		pgoi.setColor(RGBColor(255, 215, 0)); 
		pgoi.setText("Polygon");
		pgoi.setTextColor(RGBColor(255, 0, 0));


		runMessageLoop();

	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}
	return 0;

}

//</matrixdoc>
