/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page aoicallback_page AOI callback example (CPP)

\section aoicallback_section_purpose Purpose
The aoicallback example shows, how callbacks can be used with AOIs.

\section aoicallback_section_intro What it does
An Area of Interest (AOI) is used to define a child region of an image interactively.
This child region is shown in another window.

\section aoicallback_section_description How it works
To react on events triggered by an AOI, one must overwrite the corresponding event
handlers of class Window. This is done by deriving from Window. In the constructor, the
member variable childWindow is set which is used to refer to the second window which shows
the selected Region. Note that the constructor of the base class has to be called. Otherwise
the derived class won't work properly.

\code
//this class handles the track events
class MyWindow : public Window 
{
public:
	MyWindow() 
		: Window(), m_childWin()
	{ }

	MyWindow(const Image &image, const Window &childWin) 
		: Window(image), m_childWin(childWin)
	{ }

	MyWindow(const MyWindow& r)
		: Window(r), m_childWin(r.m_childWin)
	{ }

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		m_childWin = src.m_childWin;
		return *this;
	}

\endcode

The method onOoiTracked() is called whenever the AOI has been moved
or resized. So it is overwritten here. As the onOoiTracked method
delivers only an object of type ObjectOfInterest, the base class for all
types of Objects of Interest, the object has to be converted into an AreaOfInterest
object first. 

\code
	//implementation of the trackTo method
	bool onOoiTrackTo(const ObjectOfInterest *pOoi, int, const Point *) {

		//cast the Ooi to Aoi
		// The following methods exist to cast the ObjectOfInterest to 
		// an AreaOfInterest pointer:
		const AreaOfInterest *aoi = AreaOfInterest::castFromObjectOfInterest(pOoi);
\endcode

Now, the position and dimensions of the AOI can be retrieved.

\code
		//get bounds of the aoi
		long left = aoi->getRectangle().getLeft();
		long top = aoi->getRectangle().getTop();
		long width = aoi->getRectangle().getWidth()  ;
		long height = aoi->getRectangle().getHeight();
\endcode

After clipping the area to a valid image region, the child window is
updated to correspond to the region. Then the (small) window is redrawn.

\code
		//clipping
		if (left < 0) {
			width = width + left;
			left = 0;
		}
		if (top < 0 ) {
			height = height + top;
			top = 0;
		}

		if ((left + width) > getBuffer().getWidth()) width = getBuffer().getWidth() - left ;
		if ((top + height) > getBuffer().getHeight()) height = getBuffer().getHeight() - top;


		//create child image of the area that is covered by the aoi 
		if ((width > 0) && (height > 0)) { 
			Image childImage = m_childWin.getBuffer();
			childImage.translate(left, top, width, height);
		} 
		
		//redraw window
		m_childWin.redraw();
		
		return true;
	}
\endcode

In the main program first an image is loaded. Then the "child" window is created. Afterwards,
the derived class MyWindow can be instantiated. Finally, the AOI is created in \c wnd.

\code
		//load and display image
		Image img = loadImage("screw.bmp");

		//create another window 100x100 and copy actual aoi
		Image child = Image(
			img,0,0,0,
			100, 100, img.getBands()); 


		Window wnd2 = Window(child);

		MyWindow wnd = MyWindow(img, wnd2);

		//create an aoi 
		AreaOfInterest aoi = AreaOfInterest(wnd,Rectangle(Point(0,0),Point(100,100)));

\endcode

For the windowing system to work correctly, it is required to call the global function
runMessageLoop() to get the user interface to work. This function blocks until space key is 
pressed in one of the application's windows.

\code
		//keep demo alive
		runMessageLoop();
\endcode

\section aoicallback_section_source Source file

<matrixdoc>
*/




#include "mvIMPACT.h"

//this class handles the track events
class MyWindow : public Window 
{
public:
	MyWindow() 
		: Window(), m_childWin()
	{ }

	MyWindow(const Image &image, const Window &childWin) 
		: Window(image), m_childWin(childWin)
	{ }

	MyWindow(const MyWindow& r)
		: Window(r), m_childWin(r.m_childWin)
	{ }

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		m_childWin = src.m_childWin;
		return *this;
	}

	//implementation of the trackTo method
	bool onOoiTrackTo(const ObjectOfInterest *pOoi, int /*handle*/, const Point * /*pPoint*/) {

		//cast the Ooi to Aoi
		// The following methods exist to cast the ObjectOfInterest to 
		// an AreaOfInterest pointer:
		const AreaOfInterest *aoi = AreaOfInterest::castFromObjectOfInterest(pOoi);

		//get bounds of the aoi
		long left = aoi->getRectangle().getLeft();
		long top = aoi->getRectangle().getTop();
		long width = aoi->getRectangle().getWidth()  ;
		long height = aoi->getRectangle().getHeight();

		//clipping
		if (left < 0) {
			width = width + left;
			left = 0;
		}
		if (top < 0 ) {
			height = height + top;
			top = 0;
		}

		if ((left + width) > getBuffer().getWidth()) width = getBuffer().getWidth() - left ;
		if ((top + height) > getBuffer().getHeight()) height = getBuffer().getHeight() - top;


		//create child image of the area that is covered by the aoi 
		if ((width > 0) && (height > 0)) 
		{ 
			BufferBase & childImage = m_childWin.getBuffer();
			childImage.translate(left, top, width, height);
		} 

		//redraw window
		m_childWin.redraw();

		return true;
	}

private: 
	Window m_childWin;
};

int main(int , char* [])
{
	try
	{
		//load and display image
		Image img = loadImage("screw.bmp");

		//create another window 100x100 and copy actual aoi
		Image child = Image(
			img,0,0,0,
			100, 100, img.getBands()); 


		Window wnd2 = Window(child);

		MyWindow wnd = MyWindow(img, wnd2);

		//create an aoi 
		AreaOfInterest aoi = AreaOfInterest(wnd, mvIMPACT::Rectangle(Point(0,0),Point(100,100)));

		//keep demo alive
		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}


//</matrixdoc>




