/**
\xmlonly
mvIMPACT Measure
\endxmlonly

\page autogaugecircle_page AutogaugeCircle example (CPP)


\section autogaugecircle_section_purpose Purpose
This example shows how to use the MeasureCircles class to get information about a single circle.

\section autogaugecircle_section_intro What it does
The example shows an image and an AreaOfInterest. Every time the AreaOfInterest is moved or resized the mvIMPACT Measure
module is used to determine the radius and the center point of the underlying circle. The detected circle is drawn into 
the image and the radius and the coordinates of the center point are displayed.

\section autogaugecircle_section_description How it works
First of all the mouse handler has to be implemented. This is done by creating a subclass of class Window and
implementing the onMouseButtonUp method. This method will be called each time a mouse button is released
and starts the measurement process. The class also stores the results and the condition, if a circle 
could be detected at all, in member variables. These are used by the overloaded method onDraw to draw the circle into
the window. 

In the constructor of the subclass first the corresponding constructor of class Window is called. Then
the member variables for storing the results and the AreaOfInterest object for displaying
the results are initialized.

\code 
//this class handles the track events
class MyWindow: public Window 
{
private:
	AreaOfInterest dispAoi;

	double centX;
	double centY;
	double radius;
	bool circleIsValid;

public:

	MyWindow() 
		: Window(), dispAoi(), centX(0), centY(0), radius(0), circleIsValid(false)
	{ }

	MyWindow(const Image &image) 
		: Window(image), dispAoi(*this, mvIMPACT::Rectangle(Point(320,10),Point(320,10))), centX(0), centY(0), radius(0), circleIsValid(false)
	{
		dispAoi.setTextColor(RGBColor(255,0,0));
		dispAoi.setVisible(true);
		dispAoi.setUIActive(false);
	}

	MyWindow(const MyWindow& r)
		: Window(r), dispAoi(r.dispAoi), centX(r.centX), centY(r.centY), radius(r.radius), circleIsValid(r.circleIsValid)
	{ 
		dispAoi.setTextColor(RGBColor(255,0,0));
		dispAoi.setVisible(true);
		dispAoi.setUIActive(false);
	}

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		dispAoi = src.dispAoi;
		centX = src.centX;
		centY = src.centY;
		radius = src.radius;
		circleIsValid = src.circleIsValid;
		return *this;
	}
\endcode

The buttonUp method first retrieves the AreaOfInterest from the OoiSet associated with Window. Then the
coordinates of the AreaOfInterest are read.


\code
	//implementation of the buttonUp method
	bool onMouseButtonUp (MouseButton , const PointDouble *)
	{
		//get the coordinates of the rectangle
		Rectangle rectangle = aoi.getRectangle();
		long left = rectangle.getLeft();
		long top = rectangle.getTop();
		long right = rectangle.getRight();
		long bottom = rectangle.getBottom();
\endcode

Then a MeasureCircles object is created. This object is used to do the actual measuring. In the 
example we want to calculate the radius and the center point of the detected circle. 
Thus we have to add the corresponding features. In the example these are: 
cfRadius, cfXCenter and cfYCenter. 

\code
		//perform measurement for single circles
		MeasureCircles measure = MeasureCircles();

		//add requested features
		measure.addFeature(cfRadius);
		measure.addFeature(cfXCenter);
		measure.addFeature(cfYCenter);
\endcode
 
 In the next step the measurment process is started. This is done by calling
 mvIMPACT::measure::MeasureCircles::find. In the example the measurement window is 
 defined by the AreaOfInterest. The call is surrounded by a try-except clause, because the function throws an exception
 if it does not find a circle. The number of results is then set to 0.
 If the call was successful the number of results is queried by calling mvIMPACT::measure::MeasureCircles::count. 
 
\code 
		long edges;
		try {
			//define measurement window and search for circles
			measure.find(dynamic_cast<Image &>(getBuffer()),left,top, abs(right-left), abs(bottom-top));
			//get number of results
			edges = measure.count();
		} catch (ImpactException) {
			edges = 0;
		}
\endcode

 If there are any results they are queried by calling mvIMPACT::measure::MeasureCircles::get
 and passing the corresponding feature. The radius and the center point are retrieved and the
 member variables are set appropriately, so that they can be displayed. The drawing of the circle is done
 by the overloaded method onDraw.

\code
		std::stringstream message;
		if (edges > 0) {
			//get results
			radius = measure.getFeature(cfRadius);
			centX = measure.getFeature(cfXCenter);
			centY = measure.getFeature(cfYCenter);
			circleIsValid = true;

			//set displayed text message
			message 
				<< std::fixed << std::setprecision(3) 
				<< "Radius:" << radius
				<< ", x-center:" << centX 
				<< ", y-center:" << centY;
		} else {
			//set result to invalid
			circleIsValid = false;
			message << "Invalid result";
		}

		dispAoi.setText(message.str().c_str());

		redraw();
		return true;	
\endcode

 In the main program an instance of the new class MyWindow is created and 
 the AreaOfInterest object for specifying the measurement region is created.
 Afterwards a short help message is printed.
 
\code
		//load an image
		Image img = loadImage("stanz.bmp");

		//open window and display the image
		MyWindow window = MyWindow(img);

		//create a AreaOfInterest
		AreaOfInterest aoi = AreaOfInterest(window,Rectangle(Point(10,10),Point(110,110)));

		// output message
		printf("Drag the AOI over a circle or circular arc to be measured."
			"Releasing the left mouse button starts the measurement.\n\n");
\endcode

Finally the message loop is run.

\code
		//wait until the user presses space
		runMessageLoop();
\endcode

\section autogaugecircle_section_source Source file

<matrixdoc>
*/



#include "mvIMPACT.h"
#include "mvIMPACT_measure.h"

AreaOfInterest aoi;

//this class handles the track events
class MyWindow: public Window 
{
private:
	AreaOfInterest dispAoi;

	double centX;
	double centY;
	double radius;
	bool circleIsValid;

public:

	MyWindow() 
		: Window(), dispAoi(), centX(0), centY(0), radius(0), circleIsValid(false)
	{ }

	MyWindow(const Image &image) 
		: Window(image), dispAoi(*this, mvIMPACT::Rectangle(Point(320,10),Point(320,10))), centX(0), centY(0), radius(0), circleIsValid(false)
	{
		dispAoi.setTextColor(RGBColor(255,0,0));
		dispAoi.setVisible(true);
		dispAoi.setUIActive(false);
	}

	MyWindow(const MyWindow& r)
		: Window(r), dispAoi(r.dispAoi), centX(r.centX), centY(r.centY), radius(r.radius), circleIsValid(r.circleIsValid)
	{ 
		dispAoi.setTextColor(RGBColor(255,0,0));
		dispAoi.setVisible(true);
		dispAoi.setUIActive(false);
	}

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		dispAoi = src.dispAoi;
		centX = src.centX;
		centY = src.centY;
		radius = src.radius;
		circleIsValid = src.circleIsValid;
		return *this;
	}

	//implementation of the buttonUp method
	bool onMouseButtonUp (MouseButton /*button*/, const PointDouble * /*pPoint*/)
	{
		//get the coordinates of the rectangle
		mvIMPACT::Rectangle rectangle = aoi.getRectangle();
		long left = rectangle.getLeft();
		long top = rectangle.getTop();
		long right = rectangle.getRight();
		long bottom = rectangle.getBottom();

		printf("%ld %ld %ld %ld\n",left,top,right,bottom);

		//perform measurement for single circles
		MeasureCircles measure = MeasureCircles();

		//add requested features
		measure.addFeature(cfRadius);
		measure.addFeature(cfXCenter);
		measure.addFeature(cfYCenter);

		long edges;
		try 
		{
			//define measurement window and search for circles
			measure.find(dynamic_cast<Image &>(getBuffer()),left,top, abs(right-left), abs(bottom-top));
			//get number of results
			edges = measure.count();
		} 
		catch (ImpactException) 
		{
			edges = 0;
		}

		std::stringstream message;
		if (edges > 0) 
		{
			//get results
			radius = measure.getFeature(cfRadius);
			centX = measure.getFeature(cfXCenter);
			centY = measure.getFeature(cfYCenter);
			circleIsValid = true;

			//set displayed text message
			message 
				<< std::fixed << std::setprecision(3) 
				<< "Radius:" << radius
				<< ", x-center:" << centX 
				<< ", y-center:" << centY;
		} 
		else 
		{
			//set result to invalid
			circleIsValid = false;
			message << "Invalid result";
		}

		dispAoi.setText(message.str().c_str());

		redraw();
		return true;	
	}

	void onDraw
		(
		DeviceContext *pDc, 
		const mvIMPACT::Rectangle* /*pSourceRect*/,
		const mvIMPACT::Rectangle* /*pDestRect*/
		)
	{
		if(circleIsValid) 
		{
			// get the current scroll position of the window
			int spx = getScrollPosX();
			int spy = getScrollPosY();

			pDc->drawEllipse(
				mvIMPACT::Rectangle(
				(int)(centX-radius-spx), (int)(centY-radius-spy), 
				(int)(centX+radius-spx), (int)(centY+radius-spy)),
				RGBColor(100,100,100)
				);
		}
	}

};



int main(int , char* [])
{
	try
	{
		//load an image
		Image img = loadImage("stanz.bmp");

		//open window and display the image
		MyWindow window = MyWindow(img);

		//create a AreaOfInterest
		aoi = AreaOfInterest(window, mvIMPACT::Rectangle(Point(10,10), Point(110,110)));

		// output message
		printf("Drag the AOI over a circle or circular arc to be measured."
			"Releasing the left mouse button starts the measurement.\n\n");

		//wait until the user presses space
		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>

