/**
\xmlonly
mvIMPACT Blob
\endxmlonly

\page blob_page Blob example (CPP)

\section blob_section_purpose Purpose
This example shows how to calculate some simple features of the blobs in an image.

\section blob_section_intro What it does
The example calculates for each blob in the input image the coordinates of the extreme box and
the area.

\section blob_section_description How it works
First of all an image is loaded and binarized using a simple fixed threshold. Then a window is
displayed to show the binarized image.

\code
		//load image
		Image image = loadImage("money.tif");
		image = compareGreater(image,80);

		//display image
		Window window = Window(image);

		//wait until space was pressed
		runMessageLoop("Hit the space bar to perform blob analysis.\n");
\endcode

A new BlobFeatureList object is allocated to hold the list of features to be calculated. Then
the two features "area" and "extreme box" are added to the list. The add* member functions of
BlobFeatureList return IDs. These values are used to identify the features later when the results
are to be extracted (see below). The IDs are actually integer values.

\code
		//set the features
		BlobFeatureList features = BlobFeatureList();
		BlobFeatureID areaID = features.addArea();
		BlobFeatureID boxID = features.addExtremeBox();

\endcode

Then the analysis is started and the number of blobs in the image is retrieved. Note if there were
more than one image band, slice or frame, then for each such dimension a separate blob analysis is
performed by mvIMPACT::blob::analyze. The functions which are used to retrieve the results then provide
the values for the band, slice and frame they want the results from. The dimension arguments default to 0
in the functions to avoid tedious typing for the most common case of gray-valued images.

\code
		//run blob analysis
		BlobResultList result = analyze(image,features);

		//get the number of blobs
		int numberOfBlobs = result.getNumberOfBlobs(0,0,0);
\endcode

Finally, the results are displayed for each blob. The called variant of analyze labels the input image, thus
it has to be thresholded again. Another possibility would be to set a special color palette to show the labelled
blobs. After all the message loop is run.

\code
		//display results
		for (int i=0; i<numberOfBlobs; i++) {
			std::vector<double> area = result.getSingleResult(areaID,i,0,0,0);
			std::vector<double> box = result.getSingleResult(boxID,i,0,0,0);
			double& areaValue = area[0];
			double& left = box[0];
			double& top = box[1];
			double& right = box[2];
			double& bottom = box[3];
			printf("Object: %i\t Area: %g\t Bounds: %g, %g, %g, %g\n",i,areaValue,left,top,right,bottom);
		}

		image = compareGreater(image,0);
		window.setBuffer(image);

		runMessageLoop();
\endcode

\section blob_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"
#include "mvIMPACT_blob.h"


int main(int /*argc*/, char* /*argv*/[])
{
	try
	{
		//load image
		Image image = loadImage("money.tif");
		image = compareGreater(image,80);

		//display image
		Window window = Window(image);

		//wait until space was pressed
		runMessageLoop("Hit the space bar to perform blob analysis.\n");

		//set the features
		BlobFeatureList features = BlobFeatureList();
		BlobFeatureID areaID = features.addArea();
		BlobFeatureID boxID = features.addExtremeBox();

		//run blob analysis
		BlobResultList result = analyze(image,features);

		//get the number of blobs
		int numberOfBlobs = result.getNumberOfBlobs(0,0,0);

		//display results
		for (int i=0; i<numberOfBlobs; i++) {
			std::vector<double> area = result.getSingleResult(areaID,i,0,0,0);
			std::vector<double> box = result.getSingleResult(boxID,i,0,0,0);
			double& areaValue = area[0];
			double& left = box[0];
			double& top = box[1];
			double& right = box[2];
			double& bottom = box[3];
			printf("Object: %i\t Area: %g\t Bounds: %g, %g, %g, %g\n",i,areaValue,left,top,right,bottom);
		}

		image = compareGreater(image,0);
		window.setBuffer(image);

		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>
