/**
\xmlonly
mvIMPACT Calibration
\endxmlonly

\page calib_page Calibration example (CPP)

\section calib_section_purpose Purpose
This sample shows how to use class CameraCalibration to calculate calibration information
and calibrate an image

\section calib_section_intro What it does
The sample uses a calibration grid to calculate the calibration information and afterwards
calculate a calibrated version of the image for the grid image.

\section calib_section_description How it works
First the calibration image is loaded from disk and displayed in a Window. Then a new
CameratCalibration instance is generated and the user is requested to start the calibration
algorithm by pressing the space key.

\code
		// Load the grid image.
		Image image = loadImage("grid04.bmp");

		// Display the grid image. 
		Window window = Window(image);

		// Allocate a calibration buffer. All defaults are set.
		CameraCalibration calibration;

		runMessageLoop("Press space to teach calibration data\n");
\endcode

Then the teach process is started. Afterwards some statistics are displayed and
the program waits for space to be pressed.

\code
		// Start the teach process.
		calibration.teachByGrid(image);
		
		// Read some results.
		unsigned int icnt = calibration.getIterationCount();
		std::cout  << "iteration count: " << icnt << std::endl;

		CalibrationStatistics stat = calibration.getStatistics();
		
		std::cout 
		<< std::setprecision(4) << std::setw(8)
		<< "meanXDifference: " << stat.getMeanXDifference() 
		<< " meanStddevXDifference: " << stat.getStddevXDifference() << std::endl
		<< "meanYDifference: " << stat.getMeanYDifference() 
		<< " meanStddevYDifference: " << stat.getStddevYDifference() << std::endl
		<< "meanEuclideanDistance: " << stat.getMeanEuclideanDistance() 
		<< " stddevEuclideanDistance: " << stat.getStddevEuclideanDistance() << std::endl
		;

		runMessageLoop("Press space to transform calibration grid\n");

\endcode

Finally the grid image is transformed using the calculated calibration data. Then
the window is updated for the transformed image to be displayed. Afterwards the 
program waits for a key to be pressed.

\code
		// Transform/calibrate the image.
		calibration.transformImage(image, image);

		// Update the display.
		window.redraw();
\endcode

\section calib_section_source Source file

<matrixdoc>
*/


#include "mvIMPACT.h"
#include "mvIMPACT_calibration.h"


int main(int , char* [] )
{
	try
	{
		// Load the grid image.
		Image image = loadImage("grid04.bmp");

		// Display the grid image. 
		Window window = Window(image);

		// Allocate a calibration buffer. All defaults are set.
		CameraCalibration calibration;

		runMessageLoop("Press space to teach calibration data\n");

		// Start the teach process.
		calibration.teachByGrid(image);
		
		// Read some results.
		unsigned int icnt = calibration.getIterationCount();
		std::cout  << "iteration count: " << icnt << std::endl;

		CalibrationStatistics stat = calibration.getStatistics();
		
		std::cout 
		<< std::setprecision(4) << std::setw(8)
		<< "meanXDifference: " << stat.getMeanXDifference() 
		<< " meanStddevXDifference: " << stat.getStddevXDifference() << std::endl
		<< "meanYDifference: " << stat.getMeanYDifference() 
		<< " meanStddevYDifference: " << stat.getStddevYDifference() << std::endl
		<< "meanEuclideanDistance: " << stat.getMeanEuclideanDistance() 
		<< " stddevEuclideanDistance: " << stat.getStddevEuclideanDistance() << std::endl
		;

		runMessageLoop("Press space to transform calibration grid\n");

		// Transform/calibrate the image.
		calibration.transformImage(image, image);

		// Update the display.
		window.redraw();
		
		runMessageLoop("Press space to quit program");
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}


//</matrixdoc>

