/**
\xmlonly
mvIMPACT Data Matrix
\endxmlonly

\page datamatrix_page Datamatrix example (CPP)


\section datamatrix_section_purpose Purpose
This example shows how to read a datamatrix code in an image.

\section datamatrix_section_intro What it does
This example shows how to locate and decode a datamatrix code in an image and print the
result as well as some statistics.

\section datamatrix_section_description How it works
First the input image is loaded and displayed.

\code
		//load and diplay image
		Image image = loadImage("dmtrx_6.bmp");
		Window window = Window(image);
\endcode

Then a new DatamatrixConfiguration object is created and some of the settings are
changed.

\code
		//setup datamatrix configuration
		DatamatrixConfiguration config = DatamatrixConfiguration();
		config.setSpacing(20); 
		config.setContrast(50); 
\endcode

Afterwards the decoding process is started.

\code
		//decode image
		DatamatrixResults results = decodeDM(image,config);

\endcode

Then some general statistics about the decoding process are displayed:

\code
		//get candidates and decoding time
		int candidates = results.getNumberOfCandidates();
		printf("Candidates %i\n", results.getNumberOfCandidates());
		printf("Total decoding time: %3.2f ms\n",results.getTotalDecodingTime());

\endcode

Then for each found candidate some statistics are displayed. Afterwards
each code type which produces sensible results is displayed for the current
code.

\code
		//step through all candidates and display information
		for (int candCounter=0; candCounter < candidates;candCounter++) {
			printf("\tCandidate\t%i:\n",candCounter);
			printf("\t\tDecoding time:\t%3.2f ms\n",results.getDecodingTime(candCounter));
			int codes = results.getNumberOfCodes(candCounter);

			//setp through all codes and display information
			for (int codeCounter=0;codeCounter < codes; codeCounter++) {

				//get information
				std::string codeType = results.getCodeType(candCounter,codeCounter);
				std::string code = results.getCode(candCounter,codeCounter);
				Size dimensions = results.getDatamatrixDims(candCounter,codeCounter);
				int errors = results.getNumberOfErrors(candCounter,codeCounter);
				double score = results.getReadabilityScore(candCounter,codeCounter);
				bool flipped = results.getFlipped(candCounter, codeCounter);

				//print information
				printf("\t\tCode:\t\t%i\n",codeCounter);
				printf("\t\tCodetype:\t%s\n",codeType.c_str());
				printf("\t\tDimensions:\t%i x %i\n",dimensions.getWidth(), dimensions.getHeight());
				if (!flipped)
					printf("\t\tFlipped: \tfalse, code seen from front\n");
				else
					printf("\t\tFlipped: \ttrue, code seen from back\n");
				printf("\t\tErrors:\t\t%i\n",errors);
				printf("\t\tScore:\t\t%f\n",score);
				printf("\t\tData:\t\t%s\n",code.c_str());
			}
		}
\endcode

Finally the sample waits for space to be pressed.

\code
		//keep sample alive		
		runMessageLoop();
\endcode

\section datamatrix_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"
#include "mvIMPACT_datamatrix.h"


int main(int , char* [])
{
	try
	{

		//load and diplay image
		Image image = loadImage("dmtrx_6.bmp");
		Window window = Window(image);

		//setup datamatrix configuration
		DatamatrixConfiguration config = DatamatrixConfiguration();
		config.setSpacing(20); 
		config.setContrast(50); 

		//decode image
		DatamatrixResults results = decodeDM(image,config);

		//get candidates and decoding time
		int candidates = results.getNumberOfCandidates();
		printf("Candidates %i\n", results.getNumberOfCandidates());
		printf("Total decoding time: %3.2f ms\n",results.getTotalDecodingTime());

		//step through all candidates and display information
		for (int candCounter=0; candCounter < candidates;candCounter++) {
			printf("\tCandidate\t%i:\n",candCounter);
			printf("\t\tDecoding time:\t%3.2f ms\n",results.getDecodingTime(candCounter));
			int codes = results.getNumberOfCodes(candCounter);

			//setp through all codes and display information
			for (int codeCounter=0;codeCounter < codes; codeCounter++) {

				//get information
				std::string codeType = results.getCodeType(candCounter, codeCounter);
				std::string code = results.getCode(candCounter, codeCounter);
				Size dimensions = results.getDatamatrixDims(candCounter, codeCounter);
				int errors = results.getNumberOfErrors(candCounter, codeCounter);
				double score = results.getReadabilityScore(candCounter, codeCounter);
				bool flipped = results.getFlipped(candCounter, codeCounter);

				//print information
				printf("\t\tCode:\t\t%i\n",codeCounter);
				printf("\t\tCodetype:\t%s\n",codeType.c_str());
				printf("\t\tDimensions:\t%i x %i\n",dimensions.getWidth(), dimensions.getHeight());
				if (!flipped)
					printf("\t\tFlipped: \tfalse, code seen from front\n");
				else
					printf("\t\tFlipped: \ttrue, code seen from back\n");
				printf("\t\tErrors:\t\t%i\n",errors);
				printf("\t\tScore:\t\t%f\n",score);
				printf("\t\tData:\t\t%s\n",code.c_str());
			}
		}

		//keep sample alive		
		runMessageLoop();

	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>
