/**
\xmlonly
mvIMPACT Match
\endxmlonly

\page patmatch_page Pattern matching example (CPP)

\section patmatch_section_purpose Purpose
This example shows how to use the match module.

\section patmatch_section_intro What it does
Two rotation invariant models are defined in a model image. Then these models are searched for 
in a scene image. Afterwards the two sets of points in the model image and in the scene image are
used to calculate a transformation from the scene image to the model image. This transformation is finally
applied to the scene image.

\section patmatch_section_description How it works

First of all some constants are set regarding the dimensions of the two models.

\code
const int model1Width = 42;
const int model1Height = 45;
const int model2Width = 56;
const int model2Height = 54;
\endcode

Then a helper function is defined which is used to sort the matches. This is required for 
the final transformation to be calculated. The function sorts the given points by using their
distance to a reference point.

\code
///This function sorts the points given in vector "matches". The sorting
///criteria is the distance to the point given with parameter "measurePoint".
///The point nearest to "measurepoint" will be the first point in the result vector.
static std::vector<PointDouble> sortPoints(
			///the vector containing the unsortet points
			std::vector<MatchData> matches,
			///reference point for distance calculation
			MatchData measurePoint) 
{
	//setup some vectors
	std::vector<PointDouble> result;
	std::vector<double> distance;
	//step through each point in the unsortet vector	
	for (int pointCounter=0;pointCounter<(int)matches.size();pointCounter++) {
		//get point coordinates and calculate distance to the reference point
		MatchData point = matches.at(pointCounter);
		double diffx = measurePoint.getX() - point.getX();
		double diffy = measurePoint.getY() - point.getY();
		double mydist = sqrt((double)diffx*diffx + diffy*diffy);

		//insert point into result vector (sorting by insertion)
		int position = 0;
		bool foundPosition = false;
		if (pointCounter==0) foundPosition = true;
		//search for position to insert
		while (!foundPosition &&(position < (int)distance.size())) {
			if (mydist <= distance.at(position)) {
				foundPosition = true;
			} else {
				position++;
			}
		}
		distance.insert(distance.begin() + position,mydist);
		result.insert(result.begin() + position,PointDouble(point.getX(),point.getY()));
	}

	return result;
}
\endcode

In the main program, first the reference image and the scene image are loaded from disk and
displayed each in its own window.

\code
		//load reference image
		Image referenceImage = loadImage("stanz.bmp");
		Window window1 = Window(referenceImage);

		//load distorted image
		Image image2 = loadImage("stanz1.bmp");
		Window window2 = Window(image2);

\endcode

Then the two models are defined each as a part of the reference image.

\code
		//setup first model and display it in the image
		PatModel model1 = PatModel(referenceImage,314,45,model1Width,model1Height);
		model1.setInitialAcceptanceScore(80);
		model1.setFinalAcceptanceScore(70);
		model1.setNumberOfMatches(pmmFindAll);


		//setup second model and display it
		PatModel model2 = PatModel(referenceImage,285,385,model2Width,model2Height);
		model2.setInitialAcceptanceScore(80);
		model2.setFinalAcceptanceScore(70);
		model2.setNumberOfMatches(1);
\endcode

Afterwards the first model is searched for in the reference image after the user
pressed space. All occurences of the model are marked with an AOI.

\code
		//wait for space bar
		runMessageLoop("Press <space> to search for model1 in reference image\n");

		//search for model
		PatMatchResult matchResult = findPatModel(model1,referenceImage);
		std::vector<MatchData> matches1 = matchResult.getMatches(0,0,0);

		//mark all occurences of model1 
		std::vector<AreaOfInterest> aoi1;
		int counter;
		for (counter=0;counter<matchResult.getNumberOfMatches(0,0,0);counter++) {
			MatchData aMatch = matches1.at(counter);
			aoi1.push_back(
				AreaOfInterest(
				window1,
				Rectangle(
				Point(
				(long)aMatch.getX() - model1Width/2,
				(long)aMatch.getY() - model1Height/2),
				Point(
				(long)aMatch.getX() + model1Width/2,
				(long)aMatch.getY() + model1Height/2)
				)
				)
				);
			aoi1[counter].setLinesVisible(true);
			char text[256];
			sprintf(text,"Model 1:%i",counter);
			aoi1[counter].setTextColor(RGBColor(0,255,0));
			aoi1[counter].setText(text);
		}

		//redraw window and wait for space bar
		window1.redraw();
\endcode

Afterwards the second model is searched for in the reference image. Again, all
occurences are marked with an AOI.

\code
		runMessageLoop("Press <space> to start searching for model2 in reference image\n");

		//search for model2 in reference image
		matchResult = findPatModel(model2,referenceImage);
		std::vector<MatchData> matches2 = matchResult.getMatches(0,0,0);
		std::vector<AreaOfInterest> aoi2;	

		//mark all occurences of model2 
		for (counter=0;counter<matchResult.getNumberOfMatches(0,0,0);counter++) {
			MatchData aMatch = matches2.at(counter);
			aoi2.push_back(
				AreaOfInterest(
				window1,
				Rectangle(
				Point(
				(long)aMatch.getX() - model2Width/2,
				(long)aMatch.getY() - model2Height/2),
				Point(
				(long)aMatch.getX() + model2Width/2, 
				(long)aMatch.getY() + model2Height/2)
				)
				)
				);
			aoi2[counter].setLinesVisible(true);
			char text[256];
			sprintf(text,"Model 2:%i",counter);
			aoi2[counter].setTextColor(RGBColor(0,255,0));
			aoi2[counter].setText(text);
		}

		//redraw window and wait for space bar
		window1.redraw();
\endcode

Then the first model is searched for in the scene image.

\code
		runMessageLoop("Press <space> to search for model1 in distorted image\n");

		//search for model1 in distorted image
		matchResult = findPatModel(model1,image2);
		std::vector<MatchData> matches3 = matchResult.getMatches(0,0,0);
		std::vector<AreaOfInterest> aoi3;

		//mark all occurences of model1 
		for (counter=0;counter<matchResult.getNumberOfMatches(0,0,0);counter++) {
			MatchData aMatch = matches3.at(counter);
			aoi3.push_back(
				AreaOfInterest(
				window2,
				Rectangle(
				Point(
				(long)aMatch.getX() - model1Width/2,
				(long)aMatch.getY() - model1Height/2),
				Point(
				(long)aMatch.getX() + model1Width/2,
				(long)aMatch.getY() + model1Height/2)
				)
				)
				);
			aoi3[counter].setLinesVisible(true);
			char text[256];
			sprintf(text,"Model 1:%i",counter);
			aoi3[counter].setTextColor(RGBColor(0,255,0));
			aoi3[counter].setText(text);
		}

		//redraw window and wait for space bar
		window2.redraw();
\endcode

Now the second model is searched for in the scene image.

\code
		runMessageLoop("Press <space> to search for model2 in distorted image\n");

		//search for model2 in distorted image
		matchResult = findPatModel(model2,image2);
		std::vector<MatchData> matches4 = matchResult.getMatches(0,0,0);
		std::vector<AreaOfInterest> aoi4;	


		//mark all occurences of model2 
		for (counter=0;counter<matchResult.getNumberOfMatches(0,0,0);counter++) {
			MatchData aMatch = matches4.at(counter);
			aoi4.push_back(
				AreaOfInterest(
				window2,
				Rectangle(
				Point(
				(long)aMatch.getX() - model2Width/2,
				(long)aMatch.getY() - model2Height/2),
				Point(
				(long)aMatch.getX() + model2Width/2,
				(long)aMatch.getY() + model2Height/2)
				)
				)
				);
			aoi4[counter].setLinesVisible(true);
			char text[256];
			sprintf(text,"Model 2:%i",counter);
			aoi4[counter].setTextColor(RGBColor(0,255,0));
			aoi4[counter].setText(text);
		}

		//redraw window and wait for space bar
		window2.redraw();
\endcode

Afterwards the two point vectors representing occurences of the first model are sorted using the
helper function and the first (and only) occurence of the second model in the corresponding image as
the reference point for the sorting. Then the transformation matrix is calculated and the 
scene image is transformed to be congruent to the reference image (apart from border effects).

\code
		runMessageLoop("Press <space> to rotate image2\n");

		//sort the points 
		std::vector<PointDouble> referencePoints = sortPoints(matches1, matches2.at(0));
		std::vector<PointDouble> distortedPoints = sortPoints(matches3, matches4.at(0));

		//calculate matrix
		MatrixDouble rotationMatrix = getMatrixPassPoint(referencePoints,distortedPoints);

		//rotate and display image
		Image rotatedImage = matrix(image2,rotationMatrix,giBilinear);
		Window window3 = Window(rotatedImage);

\endcode

Finally the user is requested to press space to quit the program.

\code
		//keep sample alive
		runMessageLoop();
\endcode

\section patmatch_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"
#include "mvIMPACT_match.h"


const int model1Width = 42;
const int model1Height = 45;
const int model2Width = 56;
const int model2Height = 54;


///This function sorts the points given in vector "matches". The sorting
///criteria is the distance to the point given with parameter "measurePoint".
///The point nearest to "measurepoint" will be the first point in the result vector.
static std::vector<PointDouble> sortPoints(
			///the vector containing the unsortet points
			std::vector<MatchData> matches,
			///reference point for distance calculation
			MatchData measurePoint) 
{
	//setup some vectors
	std::vector<PointDouble> result;
	std::vector<double> distance;
	//step through each point in the unsortet vector	
	for (int pointCounter=0;pointCounter<(int)matches.size();pointCounter++) {
		//get point coordinates and calculate distance to the reference point
		MatchData point = matches.at(pointCounter);
		double diffx = measurePoint.getX() - point.getX();
		double diffy = measurePoint.getY() - point.getY();
		double mydist = sqrt((double)diffx*diffx + diffy*diffy);

		//insert point into result vector (sorting by insertion)
		int position = 0;
		bool foundPosition = false;
		if (pointCounter==0) foundPosition = true;
		//search for position to insert
		while (!foundPosition &&(position < (int)distance.size())) {
			if (mydist <= distance.at(position)) {
				foundPosition = true;
			} else {
				position++;
			}
		}
		distance.insert(distance.begin() + position,mydist);
		result.insert(result.begin() + position,PointDouble(point.getX(),point.getY()));
	}

	return result;
}


int main(int /*argc*/, char* /*argv*/[])
{
	try
	{

		//load reference image
		Image referenceImage = loadImage("stanz.bmp");
		Window window1 = Window(referenceImage);

		//load distorted image
		Image image2 = loadImage("stanz1.bmp");
		Window window2 = Window(image2);


		//setup first model and display it in the image
		PatModel model1 = PatModel(referenceImage,314,45,model1Width,model1Height);
		model1.setInitialAcceptanceScore(80);
		model1.setFinalAcceptanceScore(70);
		model1.setNumberOfMatches(pmmFindAll);


		//setup second model and display it
		PatModel model2 = PatModel(referenceImage,285,385,model2Width,model2Height);
		model2.setInitialAcceptanceScore(80);
		model2.setFinalAcceptanceScore(70);
		model2.setNumberOfMatches(1);

		//wait for space bar
		runMessageLoop("Press <space> to search for model1 in reference image\n");

		//search for model
		PatMatchResult matchResult = findPatModel(model1,referenceImage);
		std::vector<MatchData> matches1 = matchResult.getMatches(0,0,0);

		//mark all occurences of model1 
		std::vector<AreaOfInterest> aoi1;
		int counter;
		for (counter=0;counter<matchResult.getNumberOfMatches(0,0,0);counter++) {
			MatchData aMatch = matches1.at(counter);
			aoi1.push_back(
				AreaOfInterest(
				window1,
				mvIMPACT::Rectangle(
				Point(
				(long)aMatch.getX() - model1Width/2,
				(long)aMatch.getY() - model1Height/2),
				Point(
				(long)aMatch.getX() + model1Width/2,
				(long)aMatch.getY() + model1Height/2)
				)
				)
				);
			aoi1[counter].setLinesVisible(true);
			char text[256];
			sprintf(text,"Model 1:%i",counter);
			aoi1[counter].setTextColor(RGBColor(0,255,0));
			aoi1[counter].setText(text);
		}

		//redraw window and wait for space bar
		window1.redraw();


		runMessageLoop("Press <space> to start searching for model2 in reference image\n");

		//search for model2 in reference image
		matchResult = findPatModel(model2,referenceImage);
		std::vector<MatchData> matches2 = matchResult.getMatches(0,0,0);
		std::vector<AreaOfInterest> aoi2;	

		//mark all occurences of model2 
		for (counter=0;counter<matchResult.getNumberOfMatches(0,0,0);counter++) {
			MatchData aMatch = matches2.at(counter);
			aoi2.push_back(
				AreaOfInterest(
				window1,
				mvIMPACT::Rectangle(
				Point(
				(long)aMatch.getX() - model2Width/2,
				(long)aMatch.getY() - model2Height/2),
				Point(
				(long)aMatch.getX() + model2Width/2, 
				(long)aMatch.getY() + model2Height/2)
				)
				)
				);
			aoi2[counter].setLinesVisible(true);
			char text[256];
			sprintf(text,"Model 2:%i",counter);
			aoi2[counter].setTextColor(RGBColor(0,255,0));
			aoi2[counter].setText(text);
		}

		//redraw window and wait for space bar
		window1.redraw();


		runMessageLoop("Press <space> to search for model1 in distorted image\n");

		//search for model1 in distorted image
		matchResult = findPatModel(model1,image2);
		std::vector<MatchData> matches3 = matchResult.getMatches(0,0,0);
		std::vector<AreaOfInterest> aoi3;

		//mark all occurences of model1 
		for (counter=0;counter<matchResult.getNumberOfMatches(0,0,0);counter++) {
			MatchData aMatch = matches3.at(counter);
			aoi3.push_back(
				AreaOfInterest(
				window2,
				mvIMPACT::Rectangle(
				Point(
				(long)aMatch.getX() - model1Width/2,
				(long)aMatch.getY() - model1Height/2),
				Point(
				(long)aMatch.getX() + model1Width/2,
				(long)aMatch.getY() + model1Height/2)
				)
				)
				);
			aoi3[counter].setLinesVisible(true);
			char text[256];
			sprintf(text,"Model 1:%i",counter);
			aoi3[counter].setTextColor(RGBColor(0,255,0));
			aoi3[counter].setText(text);
		}

		//redraw window and wait for space bar
		window2.redraw();


		runMessageLoop("Press <space> to search for model2 in distorted image\n");

		//search for model2 in distorted image
		matchResult = findPatModel(model2,image2);
		std::vector<MatchData> matches4 = matchResult.getMatches(0,0,0);
		std::vector<AreaOfInterest> aoi4;	


		//mark all occurences of model2 
		for (counter=0;counter<matchResult.getNumberOfMatches(0,0,0);counter++) {
			MatchData aMatch = matches4.at(counter);
			aoi4.push_back(
				AreaOfInterest(
				window2,
				mvIMPACT::Rectangle(
				Point(
				(long)aMatch.getX() - model2Width/2,
				(long)aMatch.getY() - model2Height/2),
				Point(
				(long)aMatch.getX() + model2Width/2,
				(long)aMatch.getY() + model2Height/2)
				)
				)
				);
			aoi4[counter].setLinesVisible(true);
			char text[256];
			sprintf(text,"Model 2:%i",counter);
			aoi4[counter].setTextColor(RGBColor(0,255,0));
			aoi4[counter].setText(text);
		}

		//redraw window and wait for space bar
		window2.redraw();


		runMessageLoop("Press <space> to rotate image2\n");

		//sort the points 
		std::vector<PointDouble> referencePoints = sortPoints(matches1, matches2.at(0));
		std::vector<PointDouble> distortedPoints = sortPoints(matches3, matches4.at(0));

		//calculate matrix
		MatrixDouble rotationMatrix = getMatrixPassPoint(referencePoints,distortedPoints);

		//rotate and display image
		Image rotatedImage = matrix(image2,rotationMatrix,giBilinear);
		Window window3 = Window(rotatedImage);

		//keep sample alive
		runMessageLoop();

	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}
	
	return 0;
}

//</matrixdoc>
