//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>crtool<<
// Don Apr 29 14:28:38 CEST 2004
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>
#include <vl_ipl.h>
#include <vl_file.h>

#include <pgutils.h>
#include <samples.h>

#include "crtool.h"
#include "ftools.h"
#include "gui.h"
#include "preproc.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/************************/
/* Image cleaning (best)*/
/************************/
int img_clean_best( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength )
{
	dprintf((" ++ " TOOL_PREFIX ": img_clean_best - nStrength: %d\n", nStrength ));

	int ret = IPL_FALSE;
	IPL_BUFHANDLE structuringElement = IPL_DONT_CARE;

	if( sourceHandle != IPL_DONT_CARE )
	{
		if( nStrength == 1 )
		{
			// When the strength is 1, the predefined filter must be used.
			// Apply predefined median filter
			IPL_Call(( ret = IPL_Rank( sourceHandle, destinationHandle, IPL_RECT_3x3, IPL_MEDIAN, IPL_GRAYSCALE )));
		}
		else if( nStrength > 1 && nStrength <= 3 )
		{
			IPL_Call(( ret = IPL_BufAlloc2D( &structuringElement, 2 * nStrength + 1, 2 * nStrength + 1, IPL_DTULONG, IPL_STRUCTBUF, NULL )));

			// Adjust the overscan mode
			IPL_Call(( ret = IPL_BufSetupNeighborhood( structuringElement, IPL_CTLID_OVERSCAN, IPL_OVERSCAN_TRANS )));

			// Set to all ones
			int nData= 1;
			for( int nX= 0; nX < 2 * nStrength + 1; nX++ )
			{
				for( int nY= 0; nY < 2 * nStrength + 1; nY++ )
					IPL_Call(( ret = IPL_PutBufData( structuringElement, nX, nY, 0, 0, 0, 1, 1, &nData )));
			}

			// Then this structuring element can be repeatedly used for median filtering.
			// Apply rank filter
			IPL_Call(( ret = IPL_Rank( sourceHandle, destinationHandle, structuringElement, IPL_MEDIAN, IPL_GRAYSCALE )));
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": img_clean_best - wrong strength size (%d)!!\n", nStrength );
	}

FAILURE:
	if( structuringElement != IPL_DONT_CARE )
		IPL_BufFree( structuringElement );

	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Image cleaning (best) failed:\n" NO_SRC_STR, err_code );
		else
		{
			if( err_code == 0 )
				note_or_err_msg( AN_ERROR, "Image cleaning (best) failed:\nWrong strength size", err_code );
			else
				err_msg( "Image cleaning (best) failed", err_code );
		}
	}

	dprintf((" -- " TOOL_PREFIX ": img_clean_best (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
/************************/
/* Image cleaning (fast)*/
/************************/
int img_clean_fast( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength )
{
	dprintf((" ++ " TOOL_PREFIX ": img_clean_fast - nStrength: %d\n", nStrength ));

	int ret = IPL_FALSE;
	IPL_BUFHANDLE intermediateHandle = IPL_DONT_CARE;

	if( sourceHandle != IPL_DONT_CARE )
	{
		// Morphological smoothing filter.
		// Compute opening
		IPL_Call(( ret = IPL_MoOpen( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

		// Compute closing
		IPL_Call(( ret = IPL_MoClose( sourceHandle, &intermediateHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

		// Average
		IPL_Call(( ret = IPL_PtToPoint( *destinationHandle, intermediateHandle, destinationHandle, IPL_ADD_AVR )));
	}

FAILURE:
	if( intermediateHandle != IPL_DONT_CARE )
		IPL_BufFree( intermediateHandle );

	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Image cleaning (fast) failed:\n" NO_SRC_STR, err_code );
		else
			err_msg( "Image cleaning (fast) failed", err_code );
	}

	dprintf((" -- " TOOL_PREFIX ": img_clean_fast (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
/******************************/
/* Image cleaning (salt noise)*/
/******************************/
int img_clean_salt_noise( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength )
{
	dprintf((" ++ " TOOL_PREFIX ": img_clean_salt_noise - nStrength: %d\n", nStrength ));

	int ret = IPL_FALSE;

	if( sourceHandle != IPL_DONT_CARE )
	{
		// Apply opening
		IPL_Call(( ret = IPL_MoOpen( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));
	}

FAILURE:
	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Image cleaning (salt noise) failed:\n" NO_SRC_STR, err_code );
		else
			err_msg( "Image cleaning (salt noise) failed", err_code );
	}

	dprintf((" -- " TOOL_PREFIX ": img_clean_salt_noise (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
/********************************/
/* Image cleaning (pepper noise)*/
/********************************/
int img_clean_pepper_noise( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength )
{
	dprintf((" ++ " TOOL_PREFIX ": img_clean_pepper_noise - nStrength: %d\n", nStrength ));

	int ret = IPL_FALSE;

	if( sourceHandle != IPL_DONT_CARE )
	{
		// Apply closing
		IPL_Call(( ret = IPL_MoClose( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));
	}

FAILURE:
	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Image cleaning (pepper noise) failed:\n" NO_SRC_STR, err_code );
		else
			err_msg( "Image cleaning (pepper noise) failed", err_code );
	}

	dprintf((" -- " TOOL_PREFIX ": img_clean_pepper_noise (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
/******************************/
/* Cell improvement (too thin)*/
/******************************/
int cell_impr_2thin( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength, const int col_flag )
{
	dprintf((" ++ " TOOL_PREFIX ": cell_impr_2thin - nStrength: %d, col_flag: %d\n", nStrength, col_flag ));
	int ret = IPL_FALSE;

	if( sourceHandle != IPL_DONT_CARE )
	{
		switch( col_flag )
		{
		case POL_W2B:
			// Apply dilation
			IPL_Call(( ret = IPL_MoDilate( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));
			break;

		case POL_B2W:
			// Apply erosion
			IPL_Call(( ret = IPL_MoErode( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));
			break;

		default:
			fprintf( stderr, "*** " TOOL_PREFIX ": cell_impr_2thin - wrong polarity (%d)!!\n", col_flag );
			break;
		}
	}

FAILURE:
	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Cell improvement (too thin) failed:\n" NO_SRC_STR, err_code );
		else
		{
			if( err_code == 0 && col_flag )
				note_or_err_msg( AN_ERROR, "Cell improvement (too thin) failed:\n" NO_POLARITY_STR, err_code );
			else
				err_msg( "Cell improvement (too thin) failed", err_code );
		}
	}
	dprintf((" -- " TOOL_PREFIX ": cell_impr_2thin (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
/*******************************/
/* Cell improvement (too thick)*/
/*******************************/
int cell_impr_2thick( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength, const int col_flag )
{
	dprintf((" ++ " TOOL_PREFIX ": cell_impr_2thick - nStrength: %d, col_flag: %d\n", nStrength, col_flag ));

	int ret = IPL_FALSE;

	if( sourceHandle != IPL_DONT_CARE )
	{
		switch( col_flag )
		{
		case POL_W2B:
			// Apply erosion
			IPL_Call(( ret = IPL_MoErode( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));
			break;

		case POL_B2W:
			// Apply dilation
			IPL_Call(( ret = IPL_MoDilate( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));
			break;

		default:
			fprintf( stderr, "*** " TOOL_PREFIX ": cell_impr_2thick - wrong polarity (%d)!!\n", col_flag );
			break;
		}
	}

FAILURE:
	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Cell improvement (too thick) failed:\n" NO_SRC_STR, err_code );
		else
		{
			if( err_code == 0 && col_flag )
				note_or_err_msg( AN_ERROR, "Cell improvement (too thick) failed:\n" NO_POLARITY_STR, err_code );
			else
				err_msg( "Cell improvement (too thick) failed", err_code );
		}
	}

	dprintf((" -- " TOOL_PREFIX ": cell_impr_2thick (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
/*********************************/
/* Cell improvement (not uniform)*/
/*********************************/
int cell_impr_nu( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength, const int col_flag )
{
	dprintf((" ++ " TOOL_PREFIX ": cell_impr_nu - nStrength: %d, col_flag: %d\n", nStrength, col_flag ));

	int ret = IPL_FALSE;

	if( sourceHandle != IPL_DONT_CARE )
	{
		switch( col_flag )
		{
		case POL_W2B:
			// Apply closing
			IPL_Call(( ret = IPL_MoClose( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));
			break;

		case POL_B2W:
			// Apply opening
			IPL_Call(( ret = IPL_MoOpen( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));
			break;

		default:
			fprintf( stderr, "*** " TOOL_PREFIX ": cell_impr_nu - wrong polarity (%d)!!\n", col_flag );
			break;
		}
	}

FAILURE:
	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Cell improvement (not uniform) failed:\n" NO_SRC_STR, err_code );
		else
		{
			if( err_code == 0 && col_flag )
				note_or_err_msg( AN_ERROR, "Cell improvement (not uniform) failed:\n" NO_POLARITY_STR, err_code );
			else
				err_msg( "Cell improvement (not uniform) failed", err_code );
		}
	}

	dprintf((" -- " TOOL_PREFIX ": cell_impr_nu (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
/**********************************************/
/* Symbol location * shading correction (best)*/
/**********************************************/
int symb_loc_sc_best( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength, const int col_flag )
{
	dprintf((" ++ " TOOL_PREFIX ": symb_loc_sc_best - nStrength: %d, col_flag: %d\n", nStrength, col_flag ));

	int ret = IPL_FALSE;
	IPL_BUFHANDLE intermediateHandle = IPL_DONT_CARE;

	if( sourceHandle != IPL_DONT_CARE )
	{
		switch( col_flag )
		{
		case POL_W2B:
			// Compute opening
			IPL_Call(( ret = IPL_MoOpen( sourceHandle, &intermediateHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Additive shading correction
			IPL_Call(( ret = IPL_PtShadingCorrection( sourceHandle, destinationHandle, intermediateHandle, 0, IPL_ADDITIVE_BLACK_CST )));
			break;

		case POL_B2W:
			// Compute closing
			IPL_Call(( ret = IPL_MoClose( sourceHandle, &intermediateHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Multiplicative shading correction
			IPL_Call(( ret = IPL_PtShadingCorrection( sourceHandle, destinationHandle, intermediateHandle, 0, IPL_MULTIPLICATIVE_BLACK_CST )));
			break;

		default:
			fprintf( stderr, "*** " TOOL_PREFIX ": symb_loc_sc_best - wrong polarity (%d)!!\n", col_flag );
			break;
		}
	}

FAILURE:
	if( intermediateHandle != IPL_DONT_CARE )
		IPL_BufFree( intermediateHandle );

	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Symbol location * shading correction (best) failed:\n" NO_SRC_STR, err_code );
		else
		{
			if( err_code == 0 && col_flag )
				note_or_err_msg( AN_ERROR, "Symbol location * shading correction (best) failed:\n" NO_POLARITY_STR, err_code );
			else
				err_msg( "Symbol location * shading correction (best) failed", err_code );
		}
	}

	dprintf((" -- " TOOL_PREFIX ": symb_loc_sc_best (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
/**********************************************/
/* Symbol location * shading correction (fast)*/
/**********************************************/
int symb_loc_sc_fast( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength, const int col_flag )
{
	dprintf((" ++ " TOOL_PREFIX ": symb_loc_sc_fast - nStrength: %d, col_flag: %d\n", nStrength, col_flag ));

	int ret = IPL_FALSE;
	IPL_BUFHANDLE intermediateHandle = IPL_DONT_CARE;

	if( sourceHandle != IPL_DONT_CARE )
	{
		switch( col_flag )
		{
		case POL_W2B:
			// Compute erosion
			IPL_Call(( ret = IPL_MoErode( sourceHandle, &intermediateHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Additive shading correction
			IPL_Call(( ret = IPL_PtShadingCorrection( sourceHandle, destinationHandle, intermediateHandle, 0, IPL_ADDITIVE_BLACK_CST )));
			break;

		case POL_B2W:
			// Compute dilation
			IPL_Call(( ret = IPL_MoDilate( sourceHandle, &intermediateHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Multiplicative shading correction
			IPL_Call(( ret = IPL_PtShadingCorrection( sourceHandle, destinationHandle, intermediateHandle, 0, IPL_MULTIPLICATIVE_BLACK_CST )));
			break;

		default:
			fprintf( stderr, "*** " TOOL_PREFIX ": symb_loc_sc_fast - wrong polarity (%d)!!\n", col_flag );
			break;
		}
	}

FAILURE:
	if( intermediateHandle != IPL_DONT_CARE )
		IPL_BufFree( intermediateHandle );

	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Symbol location * shading correction (fast) failed:\n" NO_SRC_STR, err_code );
		else
		{
			if( err_code == 0 && col_flag )
				note_or_err_msg( AN_ERROR, "Symbol location * shading correction (fast) failed:\n" NO_POLARITY_STR, err_code );
			else
				err_msg( "Symbol location * shading correction (fast) failed", err_code );
		}
	}

	dprintf((" -- " TOOL_PREFIX ": symb_loc_sc_fast (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
/********************************************/
/* Symbol location * region selection (best)*/
/********************************************/
int symb_loc_rs_best( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength, const int col_flag )
{
	dprintf((" ++ " TOOL_PREFIX ": symb_loc_rs_best - nStrength: %d, col_flag: %d\n", nStrength, col_flag ));

	int ret = IPL_FALSE;
	IPL_BUFHANDLE intermediateHandle = IPL_DONT_CARE;
	IPL_LONG nThreshold = -1;

	if( sourceHandle != IPL_DONT_CARE )
	{
		switch( col_flag )
		{
		case POL_W2B:
			// Compute opening
			IPL_Call(( ret = IPL_MoOpen( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Compute closing
			IPL_Call(( ret = IPL_MoClose( sourceHandle, &intermediateHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Difference
			IPL_Call(( ret = IPL_PtToPoint( intermediateHandle, *destinationHandle, &intermediateHandle, IPL_SUB_CLIP )));

			// Automatic threshold
			IPL_Call(( ret = IPL_StatBestThreshold( intermediateHandle, &nThreshold )));
			IPL_Call(( ret = IPL_PtToPoint( intermediateHandle, nThreshold, &intermediateHandle, IPL_GREATER_CST )));

			// Masking
			IPL_Call(( ret = IPL_PtToPoint( sourceHandle, intermediateHandle, destinationHandle, IPL_AND )));
			break;

		case POL_B2W:
			// Compute opening
			IPL_Call(( ret = IPL_MoOpen( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Compute closing
			IPL_Call(( ret = IPL_MoClose( sourceHandle, &intermediateHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Difference
			IPL_Call(( ret = IPL_PtToPoint( intermediateHandle, *destinationHandle, &intermediateHandle, IPL_SUB_CLIP )));

			// Automatic threshold
			IPL_Call(( ret = IPL_StatBestThreshold( intermediateHandle, &nThreshold )));
			IPL_Call(( ret = IPL_PtToPoint( intermediateHandle, nThreshold, & intermediateHandle, IPL_LESS_CST )));

			// Masking
			IPL_Call(( ret = IPL_PtToPoint( sourceHandle, intermediateHandle, destinationHandle, IPL_OR )));
			break;

		default:
			fprintf( stderr, "*** " TOOL_PREFIX ": symb_loc_rs_best - wrong polarity (%d)!!\n", col_flag );
			break;
		}
	}

FAILURE:
	if( intermediateHandle != IPL_DONT_CARE )
		IPL_BufFree( intermediateHandle );

	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Symbol location * region selection (best) failed:\n" NO_SRC_STR, err_code );
		else
		{
			if( err_code == 0 && col_flag )
				note_or_err_msg( AN_ERROR, "Symbol location * region selection (best) failed:\n" NO_POLARITY_STR, err_code );
			else
				err_msg( "Symbol location * region selection (best) failed", err_code );
		}
	}

	dprintf((" -- " TOOL_PREFIX ": symb_loc_rs_best (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
/********************************************/
/* Symbol location * region selection (fast)*/
/********************************************/
int symb_loc_rs_fast( const IPL_BUFHANDLE sourceHandle, IPL_BUFHANDLE *destinationHandle, const int nStrength, const int col_flag )
{
	dprintf((" ++ " TOOL_PREFIX ": symb_loc_rs_fast - nStrength: %d, col_flag: %d\n", nStrength, col_flag ));
	int ret = IPL_FALSE;
	IPL_BUFHANDLE intermediateHandle = IPL_DONT_CARE;
	IPL_LONG nThreshold = -1;

	if( sourceHandle != IPL_DONT_CARE )
	{
		switch( col_flag )
		{
		case POL_W2B:
			// Compute erosion
			IPL_Call(( ret = IPL_MoErode( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Compute dilation
			IPL_Call(( ret = IPL_MoDilate( sourceHandle, &intermediateHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Difference
			IPL_Call(( ret = IPL_PtToPoint( intermediateHandle, *destinationHandle, &intermediateHandle, IPL_SUB )));

			// Automatic threshold
			IPL_Call(( ret = IPL_StatBestThreshold( intermediateHandle, &nThreshold )));
			IPL_Call(( ret = IPL_PtToPoint( intermediateHandle, nThreshold, &intermediateHandle, IPL_GREATER_CST )));

			// Masking
			IPL_Call(( ret = IPL_PtToPoint( sourceHandle, intermediateHandle, destinationHandle, IPL_AND )));
			break;

		case POL_B2W:
			// Compute erosion
			IPL_Call(( ret = IPL_MoErode( sourceHandle, destinationHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Compute dilation
			IPL_Call(( ret = IPL_MoDilate( sourceHandle, &intermediateHandle, IPL_RECT_3x3, nStrength, IPL_GRAYSCALE )));

			// Difference
			IPL_Call(( ret = IPL_PtToPoint( intermediateHandle, *destinationHandle, &intermediateHandle, IPL_SUB )));

			// Automatic threshold
			IPL_Call(( ret = IPL_StatBestThreshold( intermediateHandle, &nThreshold )));
			IPL_Call(( ret = IPL_PtToPoint( intermediateHandle, nThreshold, & intermediateHandle, IPL_LESS_CST )));

			// Masking
			IPL_Call(( ret = IPL_PtToPoint( sourceHandle, intermediateHandle, destinationHandle, IPL_OR )));
			break;

		default:
			fprintf( stderr, "*** " TOOL_PREFIX ": symb_loc_rs_fast - wrong polarity (%d)!!\n", col_flag );
			break;
		}
	}

FAILURE:
	if( intermediateHandle != IPL_DONT_CARE )
		IPL_BufFree( intermediateHandle );

	if( ret == IPL_FALSE )
	{
		long err_code = IPL_GetErrorCode( );

		if( sourceHandle == IPL_DONT_CARE )
			note_or_err_msg( AN_ERROR, "Symbol location * region selection (fast) failed:\n" NO_SRC_STR, err_code );
		else
		{
			if( err_code == 0 && col_flag )
				note_or_err_msg( AN_ERROR, "Symbol location * region selection (fast) failed:\n" NO_POLARITY_STR, err_code );
			else
				err_msg( "Symbol location * region selection (fast) failed", err_code );
		}
	}

	dprintf((" -- " TOOL_PREFIX ": symb_loc_rs_fast (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------

