//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - demo
// Mon Feb 14 11:50:05 CET 2005
//----------------------------------------------------------------------
#ifndef __preproc_h__
#define __preproc_h__

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <vl_base.h>
#include <vl_ipl.h>

#include "ocrtool.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \brief Threshold types. */
typedef enum THTESHOLD_TYPE
{
	TH_MANUAL = 0,			/*!< \brief Use manual set threshold (fix value). */
	TH_STAT_BEST,			/*!< \brief Use static estimated threshold. */
	TH_DYN_LINE,			/*!< \brief Use dynamic estimated threshold. */
};

/*----------------------------------------------------------------*/
/*! \brief The rank filter types. */
typedef enum RANK_TYPE
{
	RANK_NONE = 0,			/*!< \brief No rank filter. */
	RANK_HV,				/*!< \brief Horizontal & vertical rank filter. */
	RANK_VERTICAL,			/*!< \brief Vertical rank filter. */
	RANK_HORIZONTAL,		/*!< \brief Horizontal rank filter. */
};

/*----------------------------------------------------------------*/
/*! \brief The morphological filter types. */
typedef enum MORPH_TYPE
{
	MORPH_NONE = 0,			/*!< \brief No morphological filter. */
	MORPH_OPEN,				/*!< \brief Morphological open. */
	MORPH_CLOSE,			/*!< \brief Morphological close. */
};

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \class CDmoPreproc
 *  \brief This is the preprocessing class.
 *  \author MATRIX VISION GmbH (td)
 *  \date 14.02.2005
 *
 *  \li This class contends different image preprocessing functions which
 *   are used to prprocess the image for the image processing. */
class CDmoPreproc
{
public:
	/*----------------------------------------------------------------*/
	/*! \brief The constructor. */
	CDmoPreproc( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		m_LastRankType   = RANK_NONE;
		m_binImageHandle = IPL_DONT_CARE;
		m_ppImageHandle  = IPL_DONT_CARE;
		m_threshold      = 127;
		m_hRank          = IPL_DONT_CARE;
		m_plRankData     = NULL;
		m_opLen          = 80;
		m_minDist        = 64;

		ResetPP( );

		dprintf(( " -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief The destructor. */
	virtual ~CDmoPreproc( )
	{
		dprintf(( " ++ ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		freeBufHandle( m_binImageHandle );
		freeBufHandle( m_ppImageHandle );
		freeBufHandle( m_hRank );
		freeRankData();
		dprintf(( " -- ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Reset the preprocessing. */
	void ResetPP( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		m_RankType  = RANK_NONE;
		m_MorphType = MORPH_NONE;
		m_ThType    = TH_STAT_BEST;
		ResetPPHandle( );

		dprintf(( " -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Reset the preprocessing image handle. */
	void ResetPPHandle( )
	{
		dprintf(( " ++ %s (%s: %d) - ppImageHandle: %ld\n", __FUNCTION__, __FILE__, __LINE__, m_ppImageHandle ));

		if( m_RankType == RANK_NONE &&  m_MorphType == MORPH_NONE )
			freeBufHandle( m_ppImageHandle );

		dprintf(( " -- %s (%s: %d) - ppImageHandle: %ld\n", __FUNCTION__, __FILE__, __LINE__, m_ppImageHandle ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Preprocess the input image
	 *  \param imageHandle input image handle
	 *  \param bgrd the color of the background ( \sa BGRD_WHITE of \sa BGRD_BLACK )
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int Preprocess( const IPL_BUFHANDLE& imageHandle, const int bgrd = BGRD_WHITE )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret = IPL_TRUE;
		DMO_CHECK_ERR( imageHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );

		/* setting to take care of the border */
		DMO_CHECK_ERR(( ret = IPL_BufSetupNeighborhood( IPL_RECT_3x3, IPL_CTLID_OVERSCAN, IPL_OVERSCAN_TRANS )), -1 );

		/* make preproc image */
		DMO_CHECK_ERR(( ret = rank( imageHandle )), -1 );
		DMO_CHECK_ERR(( ret = morphFilt( m_ppImageHandle != IPL_DONT_CARE ? m_ppImageHandle : imageHandle )), -1 );

		/* make binary image */
		DMO_CHECK_ERR(( ret = getThreshold( m_ppImageHandle != IPL_DONT_CARE ? m_ppImageHandle : imageHandle )), -1 );
		DMO_CHECK_ERR(( ret = mkBinary( m_ppImageHandle != IPL_DONT_CARE ? m_ppImageHandle : imageHandle, bgrd )), -1 );

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Get the threshold value needed for the binary image.
	 *  \param imageHandle input image handle
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE .
	 *
	 * \li The threshold \c m_threshold will be computed depending on the \sa m_ThType . */
	int getThreshold( const IPL_BUFHANDLE imageHandle )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret = IPL_TRUE;

		switch( m_ThType )
		{
		case TH_STAT_BEST:
			DMO_CHECK_ERR(( ret = IPL_StatBestThreshold( imageHandle, &m_threshold )), IPL_GetErrorCode2( ));
			break;
		case TH_MANUAL:
		case TH_DYN_LINE:
			break;
		}

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Binarize the image.
	 *  \param imageHandle input image handle
	 *  \param bgrd the color of the background ( \sa BGRD_WHITE of \sa BGRD_BLACK )
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE .
	 *
	 * \li The resulting image is \c m_binImageHandle . */
	int mkBinary( const IPL_BUFHANDLE imageHandle, const int bgrd = BGRD_WHITE )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		dprintf(( " == %s - bgrd: %s m_ThType: %d\n", __FUNCTION__, bgrd == BGRD_WHITE ? "WHITE" : "BLACK", m_ThType ));
		int ret = IPL_TRUE;

		freeBufHandle( m_binImageHandle );
		if( TH_DYN_LINE == m_ThType )
		{
			dprintf(( " == %s - m_opLen: %d m_minDist: %d\n",
						__FUNCTION__, m_opLen, m_minDist ));

			IPL_LONG imgWidth;
			DMO_CHECK_ERR(( ret = IPL_BufInquire( imageHandle, IPL_XSIZE, &imgWidth )), IPL_GetErrorCode2( ));

			if( m_opLen >= imgWidth )
				m_opLen = imgWidth - 1;

			DMO_CHECK_ERR(( ret = IPL_PtDynamicLineThreshold( imageHandle, &m_binImageHandle,
						bgrd ? IPL_WHITE_ON_BLACK : IPL_BLACK_ON_WHITE,
						m_opLen, m_minDist )), IPL_GetErrorCode2( ));
		}
		else
		{
			dprintf(( " == %s - threshold: %ld\n", __FUNCTION__, m_threshold ));

			DMO_CHECK_ERR(( ret = IPL_PtToPoint( imageHandle, m_threshold, &m_binImageHandle,
							IPL_GREATER_CST )), IPL_GetErrorCode2( ));
		}

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Apply a \b rank \b filter to the image.
	 *  \param imageHandle input image handle
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE .
	 *
	 * \li The rank filter will be applied to the input image \c imageHandle if
	 * this is not \c IPL_DONT_CARE . If the no imput image is used the rank
	 * filter will be applied to the member preprocessed image \c m_binImageHandle .
	 * The resulting image is \c m_binImageHandle . */
	int rank( const IPL_BUFHANDLE imageHandle /*= IPL_DONT_CARE*/ )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		/* if nothing to do */
		if( m_RankType == RANK_NONE )
		{
			m_LastRankType = m_RankType;
			dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
			return ret;
		}

		static int dataSizeX = 0;
		static int dataSizeY = 0;
		/* if the rank type changed */
		if( m_RankType != m_LastRankType )
		{
			/* free the old rank data */
			freeRankData( );
			/* free the old rank structure */
			freeBufHandle( m_hRank );

			/* check the new type */
			switch( m_RankType )
			{
			case RANK_HV:											/* horizontal & vertical */
				dataSizeX = 3;
				dataSizeY = 3;
				break;
			case RANK_VERTICAL:
				dataSizeX = 1;
				dataSizeY = 3;
				break;
			case RANK_HORIZONTAL:
				dataSizeX = 3;
				dataSizeY = 1;
				break;
			default:
				DMO_CHECK_ERR( false, -1 );
				break;
			}

			m_plRankData = new IPL_LONG[dataSizeX*dataSizeY];	/* alloc new memory */
			fillRankData( dataSizeX*dataSizeY );				/* fill with 1 */
			ret = mkRankHandle( dataSizeX, dataSizeY );			/* make the data structure */

			m_LastRankType = m_RankType;
		}

		/* check if mkRankHandle was OK */
		DMO_CHECK_ERR( ret == IPL_TRUE, -1 );

		/* check if the structure is available */
		DMO_CHECK_ERR( m_hRank != IPL_DONT_CARE, IPL_HANDLE_ERROR );

		freeBufHandle( m_ppImageHandle );
		/* rank the input gray scale image */
		DMO_CHECK_ERR(( ret = IPL_Rank( imageHandle, &m_ppImageHandle, m_hRank, 1, IPL_GRAYSCALE )), IPL_GetErrorCode2( ));

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Apply \b morphological filter to the image.
	 *  \param imageHandle input image handle
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE .
	 *
	 * \li A morphological filter will be applied to the input image \c imageHandle.*/
	int morphFilt( const IPL_BUFHANDLE imageHandle = IPL_DONT_CARE )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		switch( m_MorphType )
		{
		case MORPH_OPEN:
			DMO_CHECK_ERR(( ret = moOpen( imageHandle )), -1 );
			break;

		case MORPH_CLOSE:
			DMO_CHECK_ERR(( ret = moClosing( imageHandle )), -1 );
			break;

		default:
			break;
		}

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Apply \b morphological \b closing to the image.
	 *  \param imageHandle input image handle
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE .
	 *
	 * \li The morphological closing will be applied to the input image \c imageHandle if
	 * this is not \c IPL_DONT_CARE . If the no imput image is used the morphological
	 * closing will be applied to the member preprocessed image \c m_ppImageHandle .
	 * The resulting image is \c m_ppImageHandle . */
	int moClosing( const IPL_BUFHANDLE imageHandle = IPL_DONT_CARE )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		if( imageHandle == IPL_DONT_CARE )
		{
			/* apply closing to the already preprocessed image */
			DMO_CHECK_ERR(( ret = IPL_MoClose( m_ppImageHandle, &m_ppImageHandle, IPL_RECT_3x3, 1, IPL_GRAYSCALE )), IPL_GetErrorCode2( ));
		}
		else
		{
			/* apply closing to the input gray scale image */
			DMO_CHECK_ERR(( ret = IPL_MoClose( imageHandle, &m_ppImageHandle, IPL_RECT_3x3, 1, IPL_GRAYSCALE )), IPL_GetErrorCode2( ));
		}

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Apply \b morphological \b open to the image.
	 *  \param imageHandle input image handle
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE .
	 *
	 * \li The morphological open will be applied to the input image \c imageHandle if
	 * this is not \c IPL_DONT_CARE . If the no imput image is used the morphological
	 * open will be applied to the member preprocessed image \c m_ppImageHandle .
	 * The resulting image is \c m_ppImageHandle . */
	int moOpen( const IPL_BUFHANDLE imageHandle = IPL_DONT_CARE )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		if( imageHandle == IPL_DONT_CARE )
		{
			/* apply opening to the already preprocessed image */
			DMO_CHECK_ERR(( ret = IPL_MoOpen( m_ppImageHandle, &m_ppImageHandle, IPL_RECT_3x3, 1, IPL_GRAYSCALE )), IPL_GetErrorCode2( ));
		}
		else
		{
			/* apply opening to the input gray scale image */
			DMO_CHECK_ERR(( ret = IPL_MoOpen( imageHandle, &m_ppImageHandle, IPL_RECT_3x3, 1, IPL_GRAYSCALE )), IPL_GetErrorCode2( ));
		}

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	MORPH_TYPE m_MorphType;					/*!< \brief \b Morphological \b filter type.*/

	RANK_TYPE m_RankType;					/*!< \brief \b Rank \b filter type.*/
	THTESHOLD_TYPE m_ThType;				/*!< \brief Get threshold type.*/

	IPL_BUFHANDLE m_binImageHandle;			/*!< \brief Handle for the binary image.*/
	IPL_BUFHANDLE m_ppImageHandle;			/*!< \brief Handle for the preprocessed image.*/
	IPL_LONG m_threshold;					/*!< \brief Thrshold value.*/

	int m_opLen;							/*!< \brief Operator lenght. */
	int m_minDist;							/*!< \brief Minaimal grey value distance. */

	/*----------------------------------------------------------------*/
	/*----------------------------------------------------------------*/
private:
	/*----------------------------------------------------------------*/
	IPL_LONG* m_plRankData;					/*!< \brief Pointer for the rank filter structure data.*/
	IPL_BUFHANDLE m_hRank;					/*!< \brief Rank filter structure data.*/
	RANK_TYPE m_LastRankType;

	/*----------------------------------------------------------------*/
	/*! \brief Free a buffer handle.
	 *  \param Handle the buffer handle */
	void freeBufHandle( IPL_BUFHANDLE& Handle )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		if( Handle != IPL_DONT_CARE )
		{
			IPL_BufFree( Handle );
			Handle = IPL_DONT_CARE;
		}

		dprintf(( " -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Make a rank structure handle.
	 *  \param width rank structure width
	 *  \param height rank structure height
 	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE .*/
	int mkRankHandle( const unsigned int width, const unsigned int height )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret;
		DMO_CHECK_ERR( m_plRankData != NULL, -1 );

		DMO_CHECK_ERR(( ret = IPL_BufAlloc2D( &m_hRank, width, height, IPL_DTLONG, IPL_STRUCTBUF, 0 )), IPL_GetErrorCode2( ));
		DMO_CHECK_ERR(( ret = IPL_PutBufData( m_hRank, 0,0, 0,0,0, width, height, m_plRankData )), IPL_GetErrorCode2( ));

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Free the rank filter structure data. */
	void freeRankData( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		if( m_plRankData != NULL )
		{
			delete [] m_plRankData;
			m_plRankData = NULL;
		}

		dprintf(( " -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Fill the rank filter structure data with 1. */
	void fillRankData( const int size )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		if( size > 0 )
		{
			for( int i = 0; i < size; i++ )
				m_plRankData[i] = 1;
		}

		dprintf(( " -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
};

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#endif	// __preproc_h__
